/* ---------------------------------------------------------------------------
          Copyright (c) 2004-2006 Micrel, Inc.  All rights reserved.
   ---------------------------------------------------------------------------

    ks_vlan.c - KS884X switch VLAN functions.

    Author  Date      Version  Description
    DC		2/21/2007				 Add more VLAN functions
    PCD     08/15/06  1.?.?    (1) Fix bug: device vlan table become invalid after call SwitchEnableVlan().
    THa     02/28/06           (2) Do not use HW_WRITE_BYTE because of limitation of
                                   some hardware platforms.
    PCD     08/10/05  0.1.7    Fix incorrect register offset in configuration of VLAN.
    THa     10/11/04           Updated with latest specs.
    THa     09/28/04           Updated for PCI version.
    THa     02/13/04           Created file.
   ---------------------------------------------------------------------------
*/


#ifdef DEF_KS8842
#include "target.h"
#include "hardware.h"
#include "RegFunc.h"


TCHAR * gszTNUM[]={
TEXT("0"),
TEXT("1"),
TEXT("2"),
TEXT("3"),
TEXT("4"),
TEXT("5"),
TEXT("6"),
TEXT("7"),
TEXT("8"),
TEXT("9"),
TEXT("10"),
TEXT("11"),
TEXT("12"),
TEXT("13"),
TEXT("14"),
TEXT("15"),
TEXT("16")
};

TCHAR * gMIBCounterName[]=
{
TEXT("RxLoPriorityByte"),
TEXT("RcHiPriorityByte"),
TEXT("RxUndersizePkt"),
TEXT("RxFragments"),
TEXT("RxOversize"),
TEXT("RxJabbers"),
TEXT("RxSymbolError"),
TEXT("RxCRCerror"),
TEXT("RxAlignmentError"),
TEXT("RxControl8808Pkts"),
TEXT("RxPausePkts"),
TEXT("RxBroadcast"),
TEXT("RxMulticast"),
TEXT("RxUnicast"),
TEXT("Rx64Octets"),
TEXT("Rx65to127Octets"),
TEXT("Rx128to255Octets"),
TEXT("Rx256to511Octets"),
TEXT("Rx512to1023Octets"),
TEXT("Rx1024to1522Octets"),
TEXT("TxLoPriorityByte"),
TEXT("TxHiPriorityByte"),
TEXT("TxLateCollision"),
TEXT("TxPausePkts"),
TEXT("TxBroadcastPkts"),
TEXT("TxMulticastPkts"),
TEXT("TxUnicastPkts"),
TEXT("TxDeferred"),
TEXT("TxTotalCollision"),
TEXT("TxExcessiveCollision"),
TEXT("TxSingleCollision"),
TEXT("TxMultipleCollision"),
};

/* -------------------------------------------------------------------------- */
/* 
    CleanMIBCounterRegistr
    Description:
       This function clean all MIB Counters in the registry
    
*/    

void CleanMIBCounterRegistry()
{
 	TCHAR szSecName[64];
 	int i, j;
 	DWORD dw=0;
 	for(i=0;i<SWITCH_PORT_NUM ;i++)
	{	
 		_tcscpy(szSecName,TEXT("Comm\\KS8842Switch\\Port"));
 		_tcscat(szSecName,gszTNUM[i]);
 	  for(j=0;j<sizeof(gMIBCounterName)/sizeof(TCHAR *);j++)
 	     RegFunc_SetDWord(szSecName,gMIBCounterName[j],dw,HKEY_LOCAL_MACHINE); 
 	}
}

/* 
    CleanMIBCounterRegistr
    Description:
       This function update all MIB Counters in the registry
    
*/    

void UpdateMIBCounterRegistry(PHARDWARE pHardware)
{
  TCHAR szSecName[64];
 	int i, j;
 	DWORD dw=0;
 	//ULONGLONG uul;
 	PPORT_CONFIG pPort;
 	for(i=0;i<SWITCH_PORT_NUM ;i++)
	{	
 		pPort = &pHardware->m_Port[i]; // 0-1
 		_tcscpy(szSecName,TEXT("Comm\\KS8842Switch\\Port"));
 		_tcscat(szSecName,gszTNUM[i]);
 	  for(j=0;j<sizeof(gMIBCounterName)/sizeof(TCHAR *);j++)
 	  {
 	     dw=(DWORD)pPort->cnCounter[j];
 	     RegFunc_SetDWord(szSecName,gMIBCounterName[j],dw,HKEY_LOCAL_MACHINE); 
 	  }
 	}

}

/* -------------------------------------------------------------------------- */

/*
    PortConfigDefaultVID

    Description:
        This routine configures the default VID of the port.

    Parameters:
        PHARDWARE pHardware
            Pointer to hardware instance.

        UCHAR bPort
            The port index.

        USHORT wVID
            The VID value.

    Return (None):
*/

#ifdef KS_PCI_BUS
void PortConfigDefaultVID_PCI
#else
void PortConfigDefaultVID_ISA
#endif
(
    PHARDWARE pHardware,
    UCHAR     bPort,
    USHORT    wVID )
{
#ifdef KS_ISA_BUS
    HardwareSelectBank( pHardware, ( UCHAR )
        ( REG_PORT_CTRL_BANK + bPort * PORT_BANK_INTERVAL ));

    HW_WRITE_WORD( pHardware, REG_PORT_CTRL_VID_OFFSET, wVID );

#else  /* PCI BUS */
    ULONG ulAddr;

    PORT_CTRL_ADDR( bPort, ulAddr );
    ulAddr += REG_PORT_CTRL_VID_OFFSET;

    HW_WRITE_WORD( pHardware, ulAddr, wVID );
#endif

}  /* PortConfigDefaultVID */


/*
    PortGetDefaultVID

    Description:
        This routine retrieves the default VID of the port.

    Parameters:
        PHARDWARE pHardware
            Pointer to hardware instance.

        UCHAR bPort
            The port index.

        PUSHORT pwVID
            Buffer to store the VID.

    Return (None):
*/

#ifdef KS_PCI_BUS
void PortGetDefaultVID_PCI
#else
void PortGetDefaultVID_ISA
#endif
(
    PHARDWARE pHardware,
    UCHAR     bPort,
    PUSHORT   pwVID )
{
#ifdef KS_ISA_BUS
    HardwareSelectBank( pHardware, ( UCHAR )
        ( REG_PORT_CTRL_BANK + bPort * PORT_BANK_INTERVAL ));

    HW_READ_WORD( pHardware, REG_PORT_CTRL_VID_OFFSET, pwVID );

#else  /* PCI BUS */
    ULONG ulAddr;

    PORT_CTRL_ADDR( bPort, ulAddr );
    ulAddr += REG_PORT_CTRL_VID_OFFSET;

    HW_READ_WORD( pHardware, ulAddr, pwVID );
#endif
}  /* PortGetDefaultVID */





/*
    SwitchDisableVlan

    Description:
        This routine disables the VLAN function of the switch.

    Parameters:
        PHARDWARE pHardware
            Pointer to hardware instance.

    Return (None):
*/

void SwitchDisableVlan (
    PHARDWARE pHardware )
{
    ASSERT( pHardware->m_bAcquire );

#ifdef KS_ISA_BUS
#ifdef KS_PCI
    if ( !pHardware->m_fPCI )
#endif
    {
        SwitchConfigSet_ISA( pHardware, REG_SWITCH_CTRL_2_HI_OFFSET,
            SWITCH_VLAN_ENABLE, FALSE );
    }

#ifdef KS_PCI
    else
#endif
#endif

#ifdef KS_PCI
    {
        SwitchConfigSet_PCI( pHardware, REG_SWITCH_CTRL_2_HI_OFFSET,
            SWITCH_VLAN_ENABLE, FALSE );
    }
#endif
}  /* SwitchDisableVlan */


/*
    SwitchEnableVlan

    Description:
        This routine enables the VLAN function of the switch.

    Parameters:
        PHARDWARE pHardware
            Pointer to hardware instance.

    Return (None):
*/

#ifdef KS_PCI_BUS
void SwitchEnableVlan_PCI
#else
void SwitchEnableVlan_ISA
#endif
(
    PHARDWARE pHardware
)
{
    UCHAR bEntry;


    ASSERT( pHardware->m_bAcquire );
    //DAVIDCAI add 12-20-2006
    for ( bEntry = 0; bEntry <STATIC_MAC_TABLE_ENTRIES; bEntry++ )
      {
      	SwitchWriteStaticMacTable(pHardware, bEntry,
      	&pHardware->m_MacTable[bEntry].MacAddr[0],
      	pHardware->m_MacTable[bEntry].bPorts,
      	pHardware->m_MacTable[bEntry].fOverride,
      	pHardware->m_MacTable[bEntry].fValid,
      	pHardware->m_MacTable[bEntry].fUseFID,
      	pHardware->m_MacTable[bEntry].bFID );
      }
   	//DAVIDCAI End
   	

    /* Create 16 VLAN entries in the VLAN table */
    for ( bEntry = 0; bEntry < VLAN_TABLE_ENTRIES; bEntry++ )
    {
        if ( pHardware->m_VlanTable[ bEntry ].wVID )
        {
            SwitchWriteVlanTable( pHardware, bEntry,
                pHardware->m_VlanTable[ bEntry ].wVID,
                pHardware->m_VlanTable[ bEntry ].bFID,
                pHardware->m_VlanTable[ bEntry ].bMember,
                TRUE );
        }
        else
        {
            SwitchWriteVlanTable( pHardware, bEntry, 0, 0, 0, FALSE );
        }
    }

    /* Enable 802.1q VLAN mode */
    SwitchConfigSet( pHardware, REG_SWITCH_CTRL_2_OFFSET,
                     UNICAST_VLAN_BOUNDARY, TRUE );
    SwitchConfigSet( pHardware, REG_SWITCH_CTRL_2_HI_OFFSET,
                     SWITCH_VLAN_ENABLE, TRUE );

}  /* SwitchEnableVlan */


#if defined( KS_ISA_BUS )  ||  !defined( KS_ISA )
/*
    SwitchInitVlan

    Description:
        This routine initializes the VLAN of the switch.

    Parameters:
        PHARDWARE pHardware
            Pointer to hardware instance.

    Return (None):
*/

void SwitchInitVlan (
    PHARDWARE pHardware )
{
    UCHAR bPort;
    UCHAR bEntry;


    /* Read 16 VLAN entries from device's VLAN table */
    for ( bEntry = 0; bEntry < VLAN_TABLE_ENTRIES; bEntry++ )
    {
       SwitchReadVlanTable( pHardware, bEntry,
                            &pHardware->m_VlanTable[ bEntry ].wVID,
                            &pHardware->m_VlanTable[ bEntry ].bFID,
                            &pHardware->m_VlanTable[ bEntry ].bMember );
    }

    pHardware->m_VlanTable[ 0 ].wVID = 1;
    pHardware->m_VlanTable[ 0 ].bFID = 0;
    pHardware->m_VlanTable[ 0 ].bMember = 7;
    for ( bPort = 0; bPort < TOTAL_PORT_NUM; bPort++ )
    {
        pHardware->m_Port[ bPort ].wVID = 1;
        pHardware->m_Port[ bPort ].bMember = 7;
    }
}  /* SwitchInitVlan */


/*
    HardwareConfigDefaultVID

    Description:
        This routine configures the default VID of the port.  It is called from
        user functions.  The hardware should be acquired first.

    Parameters:
        PHARDWARE pHardware
            Pointer to hardware instance.

        UCHAR bPort
            The port index.

        USHORT wVID
            The VID value.

    Return (None):
*/

void HardwareConfigDefaultVID (
    PHARDWARE pHardware,
    UCHAR     bPort,
    USHORT    wVID )
{
    pHardware->m_Port[ bPort ].wVID = wVID;

#ifdef KS_ISA_BUS
#ifdef KS_PCI
    if ( !pHardware->m_fPCI )
#endif
    {
        PortConfigDefaultVID_ISA( pHardware, bPort, wVID );
    }

#ifdef KS_PCI
    else
#endif
#endif

#ifdef KS_PCI
    {
        PortConfigDefaultVID_PCI( pHardware, bPort, wVID );
    }
#endif
}  /* HardwareConfigDefaultVID */


/*
    HardwareConfigPortBaseVlan

    Description:
        This routine configures the port-base VLAN membership of the port.
        It is called from user functions.  The hardware should be acquired first.

    Parameters:
        PHARDWARE pHardware
            Pointer to hardware instance.

        UCHAR bPort
            The port index.

        UCHAR bMember
            The port-base VLAN membership.

    Return (None):
*/

void HardwareConfigPortBaseVlan
(
    PHARDWARE pHardware,
    UCHAR     bPort,
    UCHAR     bMember
)
{
#ifdef KS_ISA_BUS
    USHORT usData;

    HardwareSelectBank( pHardware, ( UCHAR )
        ( REG_PORT_CTRL_BANK + bPort * PORT_BANK_INTERVAL ));

    HW_READ_WORD( pHardware, REG_PORT_CTRL_2_OFFSET, &usData );
    usData &= ~PORT_VLAN_MEMBERSHIP;
    usData |= ( bMember & PORT_VLAN_MEMBERSHIP );
    HW_WRITE_WORD( pHardware, REG_PORT_CTRL_2_OFFSET, usData );

#else  /* PCI BUS */
    ULONG ulAddr;
    UCHAR bData;

    PORT_CTRL_ADDR( bPort, ulAddr );
    ulAddr += REG_PORT_CTRL_2_OFFSET;

    HW_READ_BYTE( pHardware, ulAddr, &bData );
    bData &= ~PORT_VLAN_MEMBERSHIP;
    bData |= ( bMember & PORT_VLAN_MEMBERSHIP );
    HW_WRITE_BYTE( pHardware, ulAddr, bData );
#endif

    pHardware->m_Port[ bPort ].bMember = bMember;

}  /* HardwareConfigPortBaseVlan */


/*
    SwitchVlanConfigDiscardNonVID

    Description:
        This routine configures Discard Non PVID packets of the switch port.
        If enabled, the device will discard packets whose VLAN id does not match
        ingress port-base default VLAN id.

    Parameters:
        PHARDWARE pHardware
            Pointer to hardware instance.

        UCHAR bPort
            The port index.

        BOOLEAN  fSet
            TRUE, enable; FALSE, disable.

    Return (None):
*/

void SwitchVlanConfigDiscardNonVID
(
    PHARDWARE pHardware,
    UCHAR     bPort,
    BOOLEAN   fSet
)
{

    ASSERT( pHardware->m_bAcquire );

    PortConfigDiscardNonVID( pHardware, bPort, fSet );

}  /* SwitchVlanConfigDiscardNonVID */

/*
    SwitchVlanConfigIngressFiltering

    Description:
        This routine configures Ingress VLAN filtering of the switch port.
        If enabled, the device will discard packets whose VLAN id membership
        in the VLAN table bits [18:16] does not include the ingress port that
        received this packet.

    Parameters:
        PHARDWARE pHardware
            Pointer to hardware instance.

        UCHAR bPort
            The port index.

        BOOLEAN  fSet
            TRUE, enable; FALSE, disable.

    Return (None):
*/

void SwitchVlanConfigIngressFiltering
(
    PHARDWARE pHardware,
    UCHAR     bPort,
    BOOLEAN   fSet
)
{

    ASSERT( pHardware->m_bAcquire );

    PortConfigIngressFiltering( pHardware, bPort, fSet );

}  /* SwitchVlanConfigIngressFiltering */

/*
    SwitchVlanConfigInsertTag

    Description:
        This routine configures 802.1q Tag insertion to the switch port.
        If enabled, the device will insert 802.1q tag to the transmit packet
        on this port if received packet is an untagged packet. The device
        will not insert 802.1q tag if received packet is tagged packet.

    Parameters:
        PHARDWARE pHardware
            Pointer to hardware instance.

        UCHAR bPort
            The port index.

        BOOLEAN  fSet
            TRUE, enable; FALSE, disable.

    Return (None):
*/

void SwitchVlanConfigInsertTag
(
    PHARDWARE pHardware,
    UCHAR     bPort,
    BOOLEAN   fSet
)
{

    ASSERT( pHardware->m_bAcquire );

    PortConfigInsertTag( pHardware, bPort, fSet );

}  /* SwitchVlanConfigInsertTag */

/*
    SwitchVlanConfigRemoveTag

    Description:
        This routine configures 802.1q Tag removal to the switch port.
        If enabled, the device will removed 802.1q tag to the transmit packet
        on this port if received packet is a tagged packet. The device will
        not remove 802.1q tag if received packet is untagged packet.

    Parameters:
        PHARDWARE pHardware
            Pointer to hardware instance.

        UCHAR bPort
            The port index.

        BOOLEAN  fSet
            TRUE, enable; FALSE, disable.

    Return (None):
*/

void SwitchVlanConfigRemoveTag
(
    PHARDWARE pHardware,
    UCHAR     bPort,
    BOOLEAN   fSet
)
{

    ASSERT( pHardware->m_bAcquire );

    PortConfigRemoveTag( pHardware, bPort, fSet );

}  /* SwitchVlanConfigRemoveTag */




/* -------------------------------------------------------------------------- */

/* 
   SetupKS8842Vlan     

    Description:
        This routine setup a VLAN based on the Registry information

    Parameters:
        PHARDWARE pHardware
            Pointer to hardware instance.

/******************************************************************************/
void SetupKS8842Vlan(PHARDWARE pHardware)
{
  BOOL  f;
	DWORD dw=0;
	int i;
	TCHAR szSecName[64];
	 WCHAR szUnicode[64];
	 char szValue[64];
	 // first find the what VLAN mode we use
   f=RegFunc_GetDWord(TEXT("Comm\\KS8842Switch"), TEXT("Vlan"),&dw, HKEY_LOCAL_MACHINE);
#ifdef DBG_VLAN   
   RETAILMSG(TRUE, (TEXT("reg Comm\\KS8842Switch Vlan=%d re=%d \r\n"),dw,f));
#endif
	 if(!f) // fail to find Comm\\KS8842Switch or Vlan key, 
	 	{
	 		// try to find HKEY_LOCAL_MACHINE\Comm\KS88421\Parms\TcpIp
      f=RegFunc_GetDWord(TEXT("Comm\\KS88421\\Parms\\TcpIp"), TEXT("VlanId"),&dw, HKEY_LOCAL_MACHINE);
	 		if(!f)
	 			pHardware->m_nVlan=VLAN_NONE; //disable VLAN 
	 		else
	 		  pHardware->m_nVlan=VLAN_TAG_NIC;	
	 	}
	 else
	 	{
	 	  if(dw<=2)
	 	    pHardware->m_nVlan=dw;
	 	  else
	 	  	pHardware->m_nVlan=VLAN_NONE; //disable VLAN 
	 		
	  }
//#ifdef DBG_VLAN   
   RETAILMSG(TRUE, (TEXT("KS8842M Vlan Mode=%d \r\n"),pHardware->m_nVlan));
//#endif

	  SwitchInitVlan(pHardware);
	  // now we need to setup a VLAN
	  if(pHardware->m_nVlan ==VLAN_NONE)
	  	{
	  		//set the member ship to default;
	 	    for(i=0;i<TOTAL_PORT_NUM;i++)
	 	      HardwareConfigPortBaseVlan(pHardware,i,0x07);
	 	    
	  		//disable VLAN
	  		SwitchDisableVlan(pHardware);
	  	}
	  else if(pHardware->m_nVlan ==VLAN_TAG_NIC)
	  	{
	  		RegFunc_GetDWord(TEXT("Comm\\KS88421\\Parms\\TcpIp"), TEXT("VlanId"),&dw, HKEY_LOCAL_MACHINE);
	 	    //set the member ship to default;
	 	    for(i=0;i<TOTAL_PORT_NUM;i++)
	 	      HardwareConfigPortBaseVlan(pHardware,i,0x07);
	 	 
	 	    //set each port VID to Same VID 
	 	    for(i=0;i<2;i++)
	 	      PortConfigDefaultVID(pHardware,i,(USHORT)dw);
				
	 	    	 	    
	 	    for (i = 0; i < VLAN_TABLE_ENTRIES; i++ )
	 	    {
	 	    	 pHardware->m_VlanTable[i].wVID=(WORD)dw;
           pHardware->m_VlanTable[i].bFID=0;
           pHardware->m_VlanTable[i].bMember=0x07;
	 	    }   
	 	    //set host port to tag removal
	 	    SwitchVlanConfigRemoveTag(pHardware,0,1);
				
	  		SwitchEnableVlan(pHardware);
	  	}
	 else if(pHardware->m_nVlan ==VLAN_PORTBASED)
	  	{
	  // read each port attr, 0 host port, 1 port1 2, port2	
			 	for(i=0;i<TOTAL_PORT_NUM;i++)
			 	{
			 		
					dw=0;
					_tcscpy(szSecName,TEXT("Comm\\KS8842Switch\\Port"));
			 		_tcscat(szSecName,gszTNUM[i]);
					f=RegFunc_GetDWord(szSecName,TEXT("MemberShip"),&dw,HKEY_LOCAL_MACHINE);
		#ifdef DBG_VLAN  
					RETAILMSG(TRUE, (TEXT("reg %s MemberShip=0x%x re=%d \r\n"),szSecName,dw,f));
		#endif	 		
			 		if(f)
			 			HardwareConfigPortBaseVlan(pHardware,i,(UCHAR)dw);
				}
						SwitchDisableVlan(pHardware); 
	
			}
	else if(pHardware->m_nVlan ==VLAN_TAG_SWITCH)  
	 {		
			DWORD mac[6];
			DWORD FID,UseFID,ForwardingPorts,MemberShip,VID,Override;
     		for(i=0;i<TOTAL_PORT_NUM;i++)
			 	{
			 		
					dw=0;
					_tcscpy(szSecName,TEXT("Comm\\KS8842Switch\\Port"));
			 		_tcscat(szSecName,gszTNUM[i]);
	
				      f=RegFunc_GetDWord(szSecName,TEXT("PVID"),&dw,HKEY_LOCAL_MACHINE);
				#ifdef DBG_VLAN  
							RETAILMSG(TRUE, (TEXT("Port%d PVID=0x%x re=%d \r\n"),i,dw,f));
				#endif	
		          if(f)
		          	VID=dw; 	 		
				  else
					 VID=0;
					 			
					  
				  f=RegFunc_GetDWord(szSecName,TEXT("Priority"),&dw,HKEY_LOCAL_MACHINE);
				  #ifdef DBG_VLAN  
					 RETAILMSG(TRUE, (TEXT("Port%d Priority=0x%x re=%d \r\n"),i,dw,f));
				  #endif	
		  		  if(f)
		              VID|=dw<<13;
		           		
				  PortConfigDefaultVID(pHardware,i,(USHORT)VID);
							   
							
							f=RegFunc_GetDWord(szSecName,TEXT("VLanFilteringEnable"),&dw,HKEY_LOCAL_MACHINE);
				#ifdef DBG_VLAN  
							RETAILMSG(TRUE, (TEXT("Port%i VLanFilteringEnable=0x%x re=%d \r\n"),i,dw,f));
				#endif	 		
					 		if(f)
					 			SwitchVlanConfigIngressFiltering(pHardware,i,(dw==0?FALSE:TRUE));
							  
						  f=RegFunc_GetDWord(szSecName,TEXT("DiscardNonPVIDPk"),&dw,HKEY_LOCAL_MACHINE);
				
				#ifdef DBG_VLAN  
							RETAILMSG(TRUE, (TEXT("Port%d DiscardNonPVIDPk=0x%x re=%d \r\n"),i,dw,f));
				#endif		
					 		
					 		if(f)
							  SwitchVlanConfigDiscardNonVID(pHardware,i,(dw==0?FALSE:TRUE));
								
							f=RegFunc_GetDWord(szSecName,TEXT("TagRemoval"),&dw,HKEY_LOCAL_MACHINE);
				#ifdef DBG_VLAN  
							RETAILMSG(TRUE, (TEXT("Port%d TagRemoval=0x%x re=%d \r\n"),i,dw,f));
				#endif		
					 	 if(f)
				        SwitchVlanConfigRemoveTag(pHardware,i,(dw==0?FALSE:TRUE));
							 
						 	f=RegFunc_GetDWord(szSecName,TEXT("TagInsertion"),&dw,HKEY_LOCAL_MACHINE);
							RETAILMSG(TRUE, (TEXT("Port%d TagInsertion=0x%x re=%d \r\n"),i,dw,f));
					 		if(f)
								SwitchVlanConfigInsertTag(pHardware,i,(dw==0?FALSE:TRUE));
				}						
		    
		   for(i=1;i<=STATIC_MAC_TABLE_ENTRIES;i++) // 8 entries for the static MAC table
	 	   {
	   		  dw=0;
					_tcscpy(szSecName,TEXT("Comm\\KS8842Switch\\StaticMACTable"));
		 			_tcscat(szSecName,gszTNUM[i]);
					szValue[0]='\0';
                    szUnicode[0]=0;
		 			f=RegFunc_GetString(szSecName,TEXT("MAC"),(char *)szUnicode,64,HKEY_LOCAL_MACHINE);

			#ifdef DBG_VLAN  	
					RETAILMSG(TRUE, (TEXT("VLAN Static Table%d: MAC=%s return=%d \r\n"),i,szValue,f));
	   		#endif
	   		if(!f)
	   	  	{
	   	  	  pHardware->m_MacTable[i-1].fValid=0;
	   	  	  continue;
			}
			// because the szUnicode is a Unicode string, we need to convert to ASCII  	
			dw=WideCharToMultiByte(CP_ACP,WC_COMPOSITECHECK,
				szUnicode,-1, szValue, 64, NULL,NULL);

			if(sscanf(szValue,"%x:%x:%x:%x:%x:%x",
				 	&mac[5],&mac[4],&mac[3],&mac[2],&mac[1],&mac[0])!=6)
			  	{
             mac[5]=mac[4]=mac[3]=mac[2]=mac[1]=mac[0]=0;
					}
				 pHardware->m_MacTable[i-1].MacAddr[0]=(UCHAR)mac[0];
				 pHardware->m_MacTable[i-1].MacAddr[1]=(UCHAR)mac[1];
				 pHardware->m_MacTable[i-1].MacAddr[2]=(UCHAR)mac[2];
				 pHardware->m_MacTable[i-1].MacAddr[3]=(UCHAR)mac[3];
				 pHardware->m_MacTable[i-1].MacAddr[4]=(UCHAR)mac[4];
				 pHardware->m_MacTable[i-1].MacAddr[5]=(UCHAR)mac[5];
				 
				 
				 	f=RegFunc_GetDWord(szSecName,TEXT("FID"),&FID,HKEY_LOCAL_MACHINE);
        #ifdef DBG_VLAN  
				 	RETAILMSG(TRUE, (TEXT("VLAN Static Table%d: FID=0x%x return=%d \r\n"),i,FID,f));
		    #endif
		    if(!f)
		    	FID=0;
		    pHardware->m_MacTable[i-1].bFID=(UCHAR)FID; 	
		    	
				f=RegFunc_GetDWord(szSecName,TEXT("UseFID"),&UseFID,HKEY_LOCAL_MACHINE);
				#ifdef DBG_VLAN  
					RETAILMSG(TRUE, (TEXT("VLAN Static Table%d: UseFID=%d return=%d \r\n"),i,UseFID,f));
		    #endif
		    if(!f)
		    	UseFID=0; 
		   pHardware->m_MacTable[i-1].fUseFID=(UCHAR)UseFID; 
		    
		    f=RegFunc_GetDWord(szSecName,TEXT("ForwardingPorts"),&ForwardingPorts,HKEY_LOCAL_MACHINE);
        #ifdef DBG_VLAN  
					RETAILMSG(TRUE, (TEXT("VLAN Static Table%d: ForwardingPorts=0x%x return=%d \r\n"),i,ForwardingPorts,f));
		    #endif
		    if(!f)
		    	ForwardingPorts=0;
		    pHardware->m_MacTable[i-1].bPorts=(UCHAR)ForwardingPorts;
		    
		    f=RegFunc_GetDWord(szSecName,TEXT("Override"),&Override,HKEY_LOCAL_MACHINE);
        #ifdef DBG_VLAN  
					RETAILMSG(TRUE, (TEXT("VLAN Static Table%d: Override=%d return=%d \r\n"),i,Override,f));
		    #endif
		    if(!f)
		    	Override=0;
		    pHardware->m_MacTable[i-1].fOverride=(UCHAR)Override;
		   
		   
		    pHardware->m_MacTable[i-1].fValid=1;   
		    	
	   	}
		    
		    for (i = 1; i <= VLAN_TABLE_ENTRIES; i++ )
	 	    {
	 	    	 dw=0;
					_tcscpy(szSecName,TEXT("Comm\\KS8842Switch\\VlanTable"));
		 			_tcscat(szSecName,gszTNUM[i]);
					f=RegFunc_GetDWord(szSecName,TEXT("VID"),&VID,HKEY_LOCAL_MACHINE);
				#ifdef DBG_VLAN  
					RETAILMSG(TRUE, (TEXT("VLAN Table%d: VID=0x%x return=%d \r\n"),i,VID,f));
		    #endif
		    if(!f)
		    {
		    	// disable this VLAN table entry
		  		pHardware->m_VlanTable[i-1].wVID=0;
		    	continue;
		    }
	 		  else
	 		  	pHardware->m_VlanTable[i-1].wVID=(USHORT)VID;
	 		  	
	 			f=RegFunc_GetDWord(szSecName,TEXT("MemberShip"),&MemberShip,HKEY_LOCAL_MACHINE);
      	#ifdef DBG_VLAN  
			      RETAILMSG(TRUE, (TEXT("VLAN Table%d: MemberShip=0x%x return=%d \r\n"),i,MemberShip,f));
		    #endif
		    if(!f)
		    	MemberShip=0x1F;
	     pHardware->m_VlanTable[i-1].bMember=(UCHAR)MemberShip;
	       
	 		  f=RegFunc_GetDWord(szSecName,TEXT("FID"),&FID,HKEY_LOCAL_MACHINE);
       #ifdef DBG_VLAN  
			 	 RETAILMSG(TRUE, (TEXT("VLAN Table%d: FID=0x%x re=%d \r\n"),i,FID,f));
		   #endif 
		    if(!f)
		    	FID=0;
		    pHardware->m_VlanTable[i-1].bFID=(UCHAR)FID;
		     	
	     
	 	    }   
				SwitchEnableVlan(pHardware); 
	
	 	}
	  		
	  	
	   	
}

#endif
#endif
