/*****************************************************************/
/* Driver for ACM0802C LCD Module (8 characters by 2 lines )
/* on the Renesas SKP boards
/*
/*  Copyright (c) 2003 Renesas Technology America, Inc.
/*  All rights reserved
/*  v0.1 08/06/2003
/*****************************************************************/

#include "SKP_LCD.h"
#include "sfr62p.h"

void LCD_write(unsigned char data_or_ctrl, _far unsigned char value);
void DisplayDelay(unsigned int units);

/* used to keep track of what is currently on the lcd */
char lcd_text1[9],lcd_text2[9];

const logo_map[8*8] = 
{
	0x03,0x00,0x00,0x00,0x07,0x07,0x07,0x00,	// R
	0x1E,0x03,0x03,0x1E,0x18,0x0C,0x03,0x00,	// R
	0x0F,0x10,0x10,0x1F,0x10,0x10,0x0F,0x00,	// E
	0x11,0x19,0x1D,0x15,0x17,0x13,0x11,0x00,	// N
	0x0F,0x10,0x10,0x1F,0x10,0x10,0x0F,0x00,	// E
	0x0F,0x10,0x10,0x0E,0x01,0x01,0x1E,0x00,	// S
	0x04,0x0A,0x0A,0x11,0x11,0x11,0x17,0x00,	// A
	0x0F,0x10,0x10,0x0E,0x01,0x01,0x1E,0x00,	// S
};

/*****************************************************************************
Name:          	InitDisplay 
Parameters:     none                
Returns:        none
Description:    Intializes the LCD display. 
*****************************************************************************/
void InitDisplay( )
{
	int i;

	// initial port directions
	prc2=1;							// unprotect as Port 9 is used
	PORT_DDR = PORT_DDR_VALUE; 

	EN_PIN = HIGH;
	EN_PIN_DDR = HIGH;				// set port that controls EN as output
	RS_PIN = HIGH;
	RS_PIN_DDR = HIGH;				// set port that controls RS as output

	EN_PIN = LOW;

	LCD_write(CTRL_WR,0x33);
	DisplayDelay(20);
	LCD_write(CTRL_WR,0x32);
	DisplayDelay(20);
	LCD_write(CTRL_WR,FUNCTION_SET);	/* reset sequence */
	LCD_write(CTRL_WR,FUNCTION_SET);
	LCD_write(CTRL_WR,LCD_CURSOR_OFF);
	LCD_write(CTRL_WR,LCD_CLEAR);
	LCD_write(CTRL_WR,LCD_HOME_L1);

	// Map the Renesas logo characters into CG RAM
	LCD_write(CTRL_WR,0x40);
	for( i=0;i<64;i++)			
		LCD_write(DATA_WR,logo_map[i]);
}

/*****************************************************************************
Name:        	DisplayString   
Parameters:  	position	Line number of display
			    *Buff		Buffer containing data to be written to display. 
			    			Last character should be null.
Returns:        none
Description:    This function writes characters to the LCD display.  
*****************************************************************************/
void DisplayString(unsigned char position, _far char * string)
{                
	_far char * letter = string;

	if(position < LCD_LINE2)
		LCD_write(CTRL_WR,LCD_HOME_L1);
	else
		LCD_write(CTRL_WR,LCD_HOME_L2);

	do
		LCD_write(DATA_WR,*string++);
	while(*string);
	
	// Save the same string to the buffer
	if( position < LCD_LINE2)			// On line1
		while(*letter)
			lcd_text1[position++] = *letter++;
	else								// On line2
	{
//		position -= LCD_LINE2;
		position = 0;
		while(*letter)
			lcd_text2[position++] = *letter++;
	}


}

/*****************************************************************************
Name:           LCD_write
Parameters:     value - the value to write
				data_or_ctrl - To write value as DATA or CONTROL
								1 = DATA
								0 = CONTROL
Returns:        none
Description:    Writes data to display. Sends command to display.  
*****************************************************************************/
void LCD_write(unsigned char data_or_ctrl, unsigned char value){

    RS_PIN = data_or_ctrl;     	/* RS SELECT (HIGH=DATA, LOW=CTRL */

	/* Write upper nibble first */
	DATA_PORT &= 0xF0;					// Clear lower part of port
	DATA_PORT |= (value & 0xF0)>>4;		// OR in upper nibble
    EN_PIN = HIGH;          			// EN enable chip (HIGH) */
	DisplayDelay(0);					// We only need a very little delay
    EN_PIN = LOW;          				// Latch data by dropping EN
	DisplayDelay(0);					// We only need a very little delay

	if(data_or_ctrl == CTRL_WR)
		DisplayDelay(1);				// extra delay needed for control writes

	/* Write lower nibble second */
	DATA_PORT &= 0xF0;					// Clear lower part of port
	DATA_PORT |= (value & 0x0F) ;		// write to port
    EN_PIN = HIGH;
	DisplayDelay(0);					// We only need a very little delay
    EN_PIN = LOW;          				// Latch data by dropping EN
	DisplayDelay(1);					// needed to put delay in between writes.

	if(data_or_ctrl == CTRL_WR)
		DisplayDelay(40);				// extra delay needed for control writes
}

/*****************************************************************************
Name:          DisplayDelay 
Parameters:    unit                   
Returns:       none 
Description:   Delay routine for LCD display.   
*****************************************************************************/
void DisplayDelay(unsigned int units){

	unsigned long counter = units * 0x100;
	while(counter--){
		_asm ("NOP");
		_asm ("NOP");
		_asm ("NOP");
	}
}
