/*
 *Copyright (c) 2000-2002 Viola Systems Ltd.
 *All rights reserved.
 *
 *Redistribution and use in source and binary forms, with or without 
 *modification, are permitted provided that the following conditions 
 *are met:
 *
 *1. Redistributions of source code must retain the above copyright 
 *notice, this list of conditions and the following disclaimer.
 *
 *2. Redistributions in binary form must reproduce the above copyright 
 *notice, this list of conditions and the following disclaimer in the 
 *documentation and/or other materials provided with the distribution.
 *
 *3. The end-user documentation included with the redistribution, if 
 *any, must include the following acknowledgment:
 *	"This product includes software developed by Viola 
 *	Systems (http://www.violasystems.com/)."
 *
 *Alternately, this acknowledgment may appear in the software itself, 
 *if and wherever such third-party acknowledgments normally appear.
 *
 *4. The names "OpenTCP" and "Viola Systems" must not be used to 
 *endorse or promote products derived from this software without prior 
 *written permission. For written permission, please contact 
 *opentcp@opentcp.org.
 *
 *5. Products derived from this software may not be called "OpenTCP", 
 *nor may "OpenTCP" appear in their name, without prior written 
 *permission of the Viola Systems Ltd.
 *
 *THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED 
 *WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 *MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 *IN NO EVENT SHALL VIOLA SYSTEMS LTD. OR ITS CONTRIBUTORS BE LIABLE 
 *FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 *CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
 *BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 *OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 *EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *====================================================================
 *
 *OpenTCP is the unified open source TCP/IP stack available on a series 
 *of 8/16-bit microcontrollers, please see <http://www.opentcp.org>.
 *
 *For more information on how to network-enable your devices, or how to 
 *obtain commercial technical support for OpenTCP, please see 
 *<http://www.violasystems.com/>.
 */

/** \file ethernet.c
 *	\brief OpenTCP Ethernet protocol and driver for Realtek's RTL8019AS
 *	\author 
 *		\li Jari Lahti (jari.lahti@violasystems.com)
 *		\li Vladan Jovanovic (vladan.jovanovic@violasystems.com)
 *	\version 1.0
 *	\date 19.2.2002
 *	\bug
 *	\warning
 *	\todo
 *		\li How to adopt to different MCU speeds in the code?
 *		\li NE2000DMAInit ??? What is it used for?
 *  
 *	OpenTCP implementation of Ethernet protocol and driver for
 *	Realtek's RTL8019AS Ethernet controller. Function declarations,data
 *	structures etc. may be found in inet/ethernet.h.
 */
#include <inet/system.h>

/************************************************/
/* Modified by Renesas Technology America, Inc. */
/* Makers of M16C, H8, SH                       */
/* http://www.renesas.com                       */
/************************************************/

#include "eth_7616.h"
// #include "DP83843.h"

#define	LED	(*(unsigned short *)0x22200000)


/************ Configurations ************/
#define POLLED_ETHERNET 1
#define INTERRUPT_ETHERNET 0

#define EEPROM_PRESENT  0     /* set to 1 if EEPROM is being used */
#define HARDWARE_PAD   0     /* set to 1 to enable hardware padding */

void DP83843_init(void);
void GetEthConfig(void);
static void delay(void);

UINT8	NE2000NextPktPtr;			/**< Start address of next packet in the Ethernet controller */
UINT8 	NE2000CurrPktPtr;			/**< Start address of current packet in the Ethernet controller */

//UINT8 	EtherSleep = 0;	/**< Used for storing state of Ethernet controller (0 = awake; 1 = sleeping) */


/* Buffers and pointers to hold the incoming and outgoing packets in */
UINT8 eth_pkt_rx_buf[1550];
UINT8 * eth_rx_buf_ptr;
UINT8 eth_pkt_tx_buf[1550];
UINT8 * eth_tx_buf_ptr;

UINT8 recv_fifo_already_dumped = 0;

/* put at the begining of a section so that the data will be alligned on a 16-bit boundry */
#pragma section Descriptors
rx_desc rx_list[ENTRY];		/* receive packet discripters */
tx_desc tx_list[ENTRY];		/* transmit packet discripters */
char	rxbuf[ENTRY][FIFOSIZE];		/* receive data buffer */
char	txbuf[ENTRY][FIFOSIZE];		/* transmit data buffer */
#pragma section

struct _ETH_INFO {
	int cur_rx;
	int cur_tx;
	long total_rx_packets;
	long total_tx_packets;
	char full_duplex;
	char speed_100;
} eth_info;



/* Functions not used */
//void outNE2000 (UINT8 reg , UINT8 dat)
//{
//}	
//UINT8 inNE2000 (UINT8 reg)
//{
//}
//void NE2000CheckOverFlow (void)
//{
//}
//void NE2000DMAInit (UINT8 page)
//{	
//}
//void NE2000EnterSleep (void) 
//{
//}
//void NE2000ExitSleep (void) 
//{
//}



/** \brief Used for storing various information about the received Ethernet frame
 *	
 *	Fields from Ethernet packet (dest/source hardware address, 
 *	protocol, frame size, start of the Ethernet packet in Ethernet controller)
 *	are stored in this structure. These values are later used from upper
 *	layer protocols (IP, ARP). See ethernet_frame definition for more
 *	information about struct fields.
 *
 */	
struct ethernet_frame received_frame;

/** \brief Used for storing various information about the Ethernet frame that will be sent
 *	
 *	Fields from Ethernet packet (dest/source hardware address, 
 *	protocol, frame size) are stored in this structure by the
 *	upper layer protocols (IP, ARP, other). These values are 
 *	then used for initializing transmission of an Ethernet frame.
 *	See ethernet_frame definition for more
 *	information about struct fields.
 *
 */
struct ethernet_frame send_frame;


/*	Invoke this function to write data to the same register as 
 *	in previously invoked outNE2000(). Saves some cycles because
 *	data direction registers and address are not set up.
 *
 *	Use SEND_NETWORK_B() macro instead of invoking this function
 *	directly.
 */
void outNE2000again (UINT8 dat)
{
	/* Actually, because this function wants to write a byte and our */
	/* Ethernet controller only wants to write words, we will write
	/* every thing to a buffer, then send it later */
	*eth_tx_buf_ptr++ = dat;
}

/**  Write buffer data to the same NE2000 register as before
 *	\param buf pointer to buffer from which we're taking data
 *	\param len number of bytes from buffer to write
 *
 *	Use SEND_NETWORK_BUF() macro instead of invoking this function
 *	directly.
 */
void outNE2000againbuf (UINT8* buf, UINT16 len)
{

	while(len--)
	{
		*eth_tx_buf_ptr++ = *buf++;
	}
}


/*  Invoke this function to continue reading contents of a NE2000 register,
 *  usefull when reading a sequence of bytes from Ethernet controller.
 *
 *	Use RECEIVE_NETWORK_B() macro instead of directly invoking this
 *	function for easier driver change.
 */
UINT8 inNE2000again (void)
{
	/* Actually, because this function wants to read a byte and our */
	/* Ethernet controller only wants to read words, we will read
	/* every thing from a buffer that we filled already */
	return 	*eth_rx_buf_ptr++;
}

/* Read bytes from NE2000 to buffer
 *
 *	Invoke this function to continue reading contents of a NE2000 register,
 *	only this time to a buffer directly.
 *
 *	Use RECEIVE_NETWORK_BUF() macro instead of directly invoking this
 *	function for easier driver change.
 */
void inNE2000againbuf (UINT8* buf, UINT16 len)
{
	while(len--)
	{
		*buf++ = *eth_rx_buf_ptr++;
	}
}
/*
 *	Invoke this function to check if there is a new Ethernet frame
 *	in the NIC's buffer waiting to be processed.
 *
 *	return
 *		* #TRUE - if there is a new packet waiting
 *		* #FALSE - if there is no new packet waiting
 */
UINT8 NE2000CheckRxFrame (void)
{
	return(FALSE);
}

/*
 *	Discard the current received frame.
 */
void NE2000DumpRxFrame (void)
{
	if(recv_fifo_already_dumped)
	{
		// the transmit routine already cleared this packet because it needed the space
		recv_fifo_already_dumped = 0;
	}
	else
	{
	}
}
 
 
/*
 *	Invoke this function at startup to properly initialize NIC's registers,
 *	HW address and operation.
 */ 
void NE2000Init (UINT8* mac)
{
	unsigned long tmp;
	int i;

	/* initialize receive & transmit descriptors */
	for(i=0;i<ENTRY;i++)
	{
		rx_list[i].status = ACT;
		rx_list[i].buf_len = FIFOSIZE;
		rx_list[i].buf_addr = &rxbuf[i][0];

		tx_list[i].status = 0;
		tx_list[i].buf_addr = &txbuf[i][0];
	}
	/* the last entry also gets a end of list marker */
	rx_list[ENTRY - 1].status = ACT | DLE;
	tx_list[ENTRY - 1].status = DLE;

	eth_info.cur_rx = 0;
	eth_info.cur_tx = 0;
	eth_info.total_rx_packets = 0;
	eth_info.total_tx_packets = 0;

	/* Set the E-DMAC to copy frames into a buffer */
	EDMAC.EDMR.LONG = 0x00000001;			/* reset EDMAC and EtherC */
	for(i=0;i<3;)							/* delay at least 16 clock cycles */
		i++;
	EDMAC.RDLAR.LONG  = (long)rx_list;		/* initialaize Rx Descriptor List Address */
	EDMAC.TDLAR.LONG  = (long)tx_list;		/* initialaize Tx Descriptor List Address */
	EDMAC.TRSCER.LONG = 0x00000000;			/* copy-back status is RFE & TFE only */
	EDMAC.TFTR.LONG   = 0x00000000;			/* threshold of Tx_FIFO */
	EDMAC.FDR.LONG    = 0x00000000;			/* transmit fifo & receive fifo is 256 bytes */
	EDMAC.RCR.LONG    = 0x00000001;			/* receive function is normal mode(continued) */

	/* Initialaze MAC addres */
	/* ADDRESS 0 corresponds to the first address byte on the cable */
	/* OpenTCP defines the localHW[6] array as [5]:[4]:[3]:[2]:[1]:[0] */
	tmp = ((unsigned long)mac[5] << 24)
		+ ((unsigned long)mac[4] << 16)
		+ ((unsigned long)mac[3] << 8)
		+ ((unsigned long)mac[2]);
	EtherC.MAHR.LONG = tmp;
	tmp = ((unsigned long)mac[1] << 8)
		+ ((unsigned long)mac[0]);
	EtherC.MALR.LONG = tmp;

	EtherC.ECSR.LONG   = (long)0x00000007;	/* clear all EtherC status */
	EtherC.ECSIPR.LONG = (long)0x00000000;	/* disable EtherC status chenge interrupt */
	EtherC.RFLR.LONG   = (long)1518;		/* ether payload is 1500+ CRC */

	/* Init PHY chip */
	DP83843_init();					/* initialize DP83843 */
// eth_info.full_duplex = 1;

	if( eth_info.full_duplex )
		EtherC.ECMR.LONG   = (long)0x00000062;	/* disable MagicPacket enable Tx & Rx Full-duplex mode */
	else
		EtherC.ECMR.LONG   = (long)0x00000060;	/* disable MagicPacket enable Tx & Rx Half-duplex mode */

	EDMAC.EDRRR.LONG   = (long)0x00000001;	/* receive EDMAC start */


#if (!POLLED_ETHERNET)
	/*
	*   enter interrupt level
	*/

	/* NOT DONE YET */

	_set_vector(ETHER_INT, (void *)ether_int);	/* set number of vector ETHER_INT */

	_set_intc(INTC_EDMAC, INTLVL_ETHER);		/* enter interrupt level */
	INTC.IPRC = ((INTLVL_ETHER << 12) & 0xf000);
	le0.irqlock = 0;

	if((le0.save_irq = get_imask()) > (INTLVL_ETHER - 1)){
		set_imask(INTLVL_ETHER - 1);			/* set interrupt mask */
	}
	EDMAC.EESIPR.LONG	=0x017f1f9f;			/* enable interrupt of any bit with all status register */
#endif

}




/** \brief Checks if new Ethernet frame exists and initializes variables
 *		accordingly
 * 	\author 
 *		\li Jari Lahti (jari.lahti@violasytems.com)
 *	\date 19.02.2002
 *	\return
 *		\li #TRUE - new frame exists, variables initialized so further
 *			processing may start
 *		\li #FALSE - no new Ethernet frame detected
 *
 *	Invoke this function periodically (better not directly but by invoking
 *	NETWORK_CHECK_IF_RECEIVED() macro) to test for newly-arrived Ethernet
 *	packets.
 *
 *	Once this function detects a new Etherent packet, internal variables 
 *	are initialized (NE2000CurrPktPtr, NE2000NextPktPtr and received_frame
 *	structure) and TRUE is returned, indicating that processing of
 *	newly-arrived frame may begin.
 */
UINT8 NE2000ReceiveFrame (void)
{

//	UINT16 frame_status;
	UINT16 packet_size;
	UINT16 i;

	UINT8 got_all = 0;
	UINT8 error_flag = 0;
	UINT8 * buff_index;
	UINT8 * copy_from, *copy_to;

	rx_desc * new_desc;


// 	if( NE2000CheckRxFrame() == FALSE ) 
// 		return(FALSE);

	buff_index = eth_pkt_rx_buf;	// start at the begining of our buffer

	/* check if current descr is active (not ready)*/	
	new_desc = &(rx_list[eth_info.cur_rx]);
	if( new_desc->status & ACT)
		return(FALSE);		/* no packets yet */

	/* A new packet has arrived */
	/* copy out data into our buffer */
	packet_size = new_desc->len;
	for(i=0;i<new_desc->len;i++)
	{
		*buff_index++ = new_desc->buf_addr[i];
	}

	/* should be start of new frame */
	if( !(new_desc->status & FP1))
		error_flag = 1;

	/* check if last packet in the frame */
	if( new_desc->status & FP0)
		got_all = 1;

	/* check for errors */
	if( new_desc->status & FE)
		error_flag = 1;

	/* mark that descriptor */
	if( new_desc->status & DLE)
	{
		new_desc->status = ACT | DLE;	/* signal that we read that descriptor */
		eth_info.cur_rx = 0;			/* wrap around to begining of list */
	}
	else
	{
		new_desc->status = ACT;		/* signal that we read that descriptor */
		eth_info.cur_rx++;			/* advance to next descriptor */
	}

	/* keep reading the descriptors untill the packet is all read out (if there are more) */
	while( !got_all )
	{
		new_desc = &(rx_list[eth_info.cur_rx]);
	
		/* wait for next frame descriptor to come in */	
		while( new_desc->status & ACT);

		/* copy out data into our buffer */
		packet_size += new_desc->len;
		for(i=0;i<new_desc->len;i++)
		{
			*buff_index++ = new_desc->buf_addr[i];
		}

		/* check if last packet in the frame */
		if( new_desc->status & FP0)
			got_all = 1;
			
		/* check for errors */
		if( new_desc->status & FE)
			error_flag = 1;

		/* mark that descriptor */
		/* also preserve end of list marker */
		if( new_desc->status & DLE)
		{
			new_desc->status = ACT | DLE;	/* signal that we read that descriptor */
			eth_info.cur_rx = 0;			/* wrap around to begining of list */
		}
		else
		{
			new_desc->status = ACT;		/* signal that we read that descriptor */
			eth_info.cur_rx++;			/* advance to next descriptor */
		}
	}

	if(error_flag)
 		return(FALSE);		// an error occurred		


/* NOT USED
 	NE2000CurrPktPtr = 0;
 	NE2000NextPktPtr = 0;
*/
 	/* Record Frame Size					*/
 	received_frame.frame_size = packet_size;

 	/* Record destination Ethernet Address	*/
	received_frame.destination[0] = eth_pkt_rx_buf[5];
	received_frame.destination[1] = eth_pkt_rx_buf[4];
	received_frame.destination[2] = eth_pkt_rx_buf[3];
	received_frame.destination[3] = eth_pkt_rx_buf[2];
	received_frame.destination[4] = eth_pkt_rx_buf[1];
	received_frame.destination[5] = eth_pkt_rx_buf[0];

 	/* Record senders Ethernet address 		*/
	received_frame.source[0] = eth_pkt_rx_buf[11];
	received_frame.source[1] = eth_pkt_rx_buf[10];
	received_frame.source[2] = eth_pkt_rx_buf[9];
	received_frame.source[3] = eth_pkt_rx_buf[8];
	received_frame.source[4] = eth_pkt_rx_buf[7];
	received_frame.source[5] = eth_pkt_rx_buf[6];

 	/* Record Protocol	*/
 	received_frame.protocol = *(UINT16 *)(eth_pkt_rx_buf + 12);

	/* Now strip off the ethernet header in our receive buffer */
	copy_from = eth_pkt_rx_buf + ETH_HEADER_LEN;
	copy_to = eth_pkt_rx_buf;
	i = received_frame.frame_size - ETH_HEADER_LEN;
	while(i--)
	{
		*copy_to++ = *copy_from++;
	}

	/* Reset pointer to the begining of the buffer */
	eth_rx_buf_ptr = eth_pkt_rx_buf;

 	/* Give the next layer data start buffer index from the start	*/
 	received_frame.buf_index = 0;
 	
 	ETH_DEBUGOUT("Ethernet Frame Received\n\r");
 
 	eth_info.total_rx_packets++;	
 	return(TRUE);						/* Indicate we got packet */

}

/*	This function is used to initialize transmission of an Ethernet 
 *	packet. Packet is created starting from page address.
 *
 *	Do not invoke this function directly, but instead use 
 *	NETWORK_SEND_INITIALIZE() macro.
 */
void InitTransmission (UINT8 page)
{
	/* Simply set up pointer for transmit buffer used by outNE2000again() */
	eth_tx_buf_ptr = eth_pkt_tx_buf;
}

/* Write Ethernet Header to transmission buffer
 *
 *	frame - Pointer to ethernet_frame structure containing information
 *		about the new Ethernet frame
 *
 *	Invoke this function (through NETWORK_ADD_DATALINK() macro) to create
 *	an Ethernet header in the NIC's transmit buffer. Do this only after
 *	invoking NETWORK_SEND_INITIALIZE() macro.
 *
 */
void NE2000WriteEthernetHeader (struct ethernet_frame* frame)
{
	INT8 i;
	
	/* Write destination Ethernet Address	*/
	
	for(i=ETH_ADDRESS_LEN-1; i >= 0; i--) {
		outNE2000again(frame->destination[i]);
	}

	/* Write sender (our) Ethernet address	*/
	
	for(i=ETH_ADDRESS_LEN-1; i >= 0; i--) {
		outNE2000again(frame->source[i]);
	}
	
	/* Write protocol						*/
	
	outNE2000again( (UINT8)(frame->protocol >> 8) );
	outNE2000again( (UINT8)frame->protocol );

}


/* Initialize reading from NE2000
 *
 * pos - Position from the start of current frame from which DMA
 *		is initialized.
 *
 *	Invoke this function (through NETWORK_RECEIVE_INITIALIZE() macro) 
 *	to prepair NIC for reading from the specified position.
 */
void NE2000DMAInit_position (UINT16 pos)
{
	/* Simply set up pointer for receive buffer used by inNE2000again() */
	eth_rx_buf_ptr = eth_pkt_rx_buf + pos;
}

/* Instruct NIC to send the Ethernet frame
 *
 *	len - Length of packet stored in the NIC's buffer
 *
 *	Invoke this function (through NETWORK_COMPLETE_SEND() macro) when
 *	the whole packet is formed inside NIC's memory and is ready to be 
 *	sent. Proper length of the packet must be supplied so that NIC
 *	knows how much data to put on the line.
 *
 *	Warning - Minimum Ethernet packet size is 64 bytes
 */
void NE2000SendFrame ( UINT16 len )
{
 	/* After filling Tx buffer, call this function to send it */
 	/* Input: 	Len: packet length							  */

	long	flag_bits;
	int i;

	tx_desc * new_desc;

 	/* Check if we need to insert Pad bytes */
 	while(len < 50)
	{
 		outNE2000again((BYTE)0x00);
 		len++;
 	}
 	
 	len += ETH_HEADER_LEN;
	
	/* Now write the data out to Ethernet */
	eth_tx_buf_ptr = eth_pkt_tx_buf;

	/* set first packet bit */
	flag_bits = FP1;

	/* loop till no more data */
	while( len)
	{
		new_desc = &(tx_list[eth_info.cur_tx]);

		/* write data to the next descritor. if not descritor is ready, then wait */
		/* check if current descr is active (not ready)*/	
		while( new_desc->status & ACT);		/* if current discripter is active */
											/* Can't transfer the data */

		/* fill up buffer */
		i = 0;
		while( len )
		{
			new_desc->buf_addr[i++] = *eth_tx_buf_ptr++;			/* transfer packet data */		
			
			/* you can only write FIFOSIZE at a time */
			if(i == FIFOSIZE )
				break;
			len--;
		}
		/* write in desc how much is in the buffer */
		new_desc->len = i;

		/* set end flag if done */
		if( len == 0 )
			flag_bits |= FP0;
		
		/* signal that we used that frame, and set any bits needed */
		/* also preserve end of list marker */
		if( new_desc->status & DLE)
		{
			new_desc->status = flag_bits | ACT | DLE;	/* signal that we read that descriptor */
			eth_info.cur_tx = 0;			/* wrap around to begining of list */
		}
		else
		{
			new_desc->status = flag_bits | ACT;		/* signal that we read that descriptor */
			eth_info.cur_tx++;			/* advance to next descriptor */
		}
		flag_bits = 0;					/* reset bits */

	}

	eth_info.total_tx_packets++;						// keep track of total number of frames sent

	/* start transmision if not already started */
//	if(EDMAC.EDTRR.LONG == 0x00000000)
//	{
			EDMAC.EDTRR.LONG = 0x00000001;
//	}
}


/******************** PHY layer functions for DP83843 Device *****************/
/* NOTES
 100BASE-T: can be 100BASE-TX, 100BASE-T4, 100BASE-T2

 100BASE-TX: 100 Mbs over two-pair Cat5 or better cable
 100BASE-T4: 100 Mbs over four-pair Cat3 or better cable
 100BASE-T2: 100 Mbs over two-pair Cat2 or better cable

 100BaseFX: Fast Ethernet over optical fiber
 100BaseVG: Voice Grade

*********	DP83843	INITIALIZE	 *********
****   MII REGISTER(WRITE COMMAND)	 ****/
#define		BMCR		0x5082
#define		ANAR		0x5092
#define		ANLPAR		0x5096
#define		ANER		0x509a
#define		ANNPTR		0x509e
#define		MIPSCR		0x50c6
#define		DCR			0x50ce
#define		FCSCR		0x50d2
#define		RECR		0x50d6
#define		PCSR		0x50da
#define		LBR			0x50de
#define		BTSCR 		0x50e2
#define		PHYCTRL		0x50e6

/***	MII REGISTER(READ COMMAND)	   ***/
#define		R_BMCR		0x6082
#define		R_BMSR		0x6086
#define		R_ANAR		0x6092
#define		R_ANLPAR	0x6096
#define		R_ANER		0x609a
#define		R_ANNPTR	0x609e
#define		R_PHYSTS	0x60c2
#define		R_MIPSCR	0x60c6
#define		R_DCR		0x60ce
#define		R_FCSCR		0x60d2
#define		R_RECR		0x60d6
#define		R_PCSR		0x60da
#define		R_LBR		0x60de
#define		R_BTSCR		0x60e2
#define		R_PHYCTRL 	0x60e6

/***	MII REGISTER(DATA)	***/
#define		D_BMCR		0x1000			/* Basic Mode Control Register */
										// B13: Speed @ 100Mbs (0=10Mbs, 1=100Mbs)
										// B12: Auto-Negotiation Enabled
										//  B8: Duplex Mode (0=Half, 1=Full)
										//  B9: Restart Auto-Negotiation

#define		D_ANAR		0x01E0			/* Auto-Negotiation Advertisement Register */
										// B8: 100BaseTX Full Duplex supported
										// B7: 100BaseTX supported
										// B6:  10BaseT Full Duplex supported
										// B5:  10BaseT supported
										// B0-B4: IEEE 802.3 CSMA/CD support (0=no, 1=yes)
#define		D_ANLPAR	0x0000
#define		D_ANER		0x0000
#define		D_ANNPTR	0x3801
#define		D_MIPSCR	0x0002			/* b1:Interrupt Enable 1:Enable event based interrupts 0:Disable event based interrupts	*/
#define		D_DCR		0x0000
#define		D_FCSCR		0x0000
#define		D_RECR		0x0000
#define		D_PCSR		0x0000
#define		D_LBR		0x0000
#define		D_10BTSCR	0x1880			/* NORMAL(0080);LINK PULSE FORCED(0180) b12:10Base-T RX serial mode 1:serial 0:nibble
																				b11:10Base-T TX serial Mode 1:serial 0:nibble	*/
#define		D_PHYCTRL	0x0801

/****************************************************/
static	void DP83843_init(void);
static	unsigned short read_PHY(unsigned short rdata);
static	void write_PHY(unsigned short wreg, unsigned short wdata);
static	void write_0();
static  void write_1();



/****************************************************/

void DP83843_init(void)
{
	unsigned short status = 0;
	
	write_PHY(BMCR, 0x8000);				/* Software Reset */
	while( read_PHY(R_BMCR) & 0x8000);		/* wait for reset bit to clear */


	write_PHY(ANAR,D_ANAR);					/* Write ANAR(PHY_Register)*/
	write_PHY(BMCR, D_BMCR);				/* Write BMCR(PHY_Register) */
	write_PHY(BMCR,D_BMCR | 0x0200);		/* restart auto-neg */

	/* now wait till auto-negotiation to complete (in case the AN0 and AN1 pins are set to do it automatically) */
	while( !(read_PHY(R_BMSR) & 0x0020) );		/* wait for auto-neg bit to clear */

	/* now wait till we get a valid link */
	while( !(read_PHY(R_BMSR) & 0x0004) );		/* wait for link bit to be set */

	status = read_PHY(R_PHYSTS);

	/* check if full duplex */
	if(status & 0x0004)
		eth_info.full_duplex = 1;		/* full duplex */
	else
		eth_info.full_duplex = 0;		/* half duplex */

	/* check if 10 base T */
	if(status & 0x0002)
		eth_info.speed_100 = 0;		/* 10Mbs */
	else
		eth_info.speed_100 = 1;		/* 100Mbs */


/*
		write_PHY(0xffff);
		write_PHY(0xffff);
		write_PHY(ANLPAR);
		write_PHY(D_ANLPAR);
		EtherC.PIR.bit.MDC=0;

		write_PHY(0xffff);
		write_PHY(0xffff);
		write_PHY(ANER);
		write_PHY(D_ANER);
		EtherC.PIR.bit.MDC=0;

		write_PHY(0xffff);
		write_PHY(0xffff);
		write_PHY(ANNPTR);
		write_PHY(D_ANNPTR);
		EtherC.PIR.bit.MDC=0;

		write_PHY(0xffff);
		write_PHY(0xffff);
		write_PHY(MIPSCR);
		write_PHY(D_MIPSCR);
		EtherC.PIR.bit.MDC=0;

		write_PHY(0xffff);
		write_PHY(0xffff);
		write_PHY(DCR);
		write_PHY(D_DCR);
		EtherC.PIR.bit.MDC=0;

		write_PHY(0xffff);
		write_PHY(0xffff);
		write_PHY(FCSCR);
		write_PHY(D_FCSCR);
		EtherC.PIR.bit.MDC=0;

		write_PHY(0xffff);
		write_PHY(0xffff);
		write_PHY(RECR);
		write_PHY(D_RECR);
		EtherC.PIR.bit.MDC=0;

		write_PHY(0xffff);
		write_PHY(0xffff);
		write_PHY(PCSR);
		write_PHY(D_PCSR);
		EtherC.PIR.bit.MDC=0;

		write_PHY(0xffff);
		write_PHY(0xffff);
		write_PHY(LBR);
		write_PHY(D_LBR);
		EtherC.PIR.bit.MDC=0;

		write_PHY(0xffff);
		write_PHY(0xffff);
		write_PHY(BTSCR);
		write_PHY(D_10BTSCR);
		EtherC.PIR.bit.MDC=0;
*/
}


static	 void write_1()
{
	EtherC.PIR.LONG=0x00000006;	// a 1 on data line
	delay();
    EtherC.PIR.LONG=0x00000007;	// clock pin high
	delay();
    EtherC.PIR.LONG=0x00000006;	// clock pin low
	delay();
}

static	  void write_0()
{
	EtherC.PIR.LONG=0x00000002;	// a 0 on data line
	delay();
    EtherC.PIR.LONG=0x00000003;	// clock pin high
	delay();
    EtherC.PIR.LONG=0x00000002;	// clock pin low
	delay();
}


static	void write_PHY(unsigned short wreg, unsigned short wdata)
{
	int i;

	/* write out preamble sequence */
	for(i=0;i<32;i++)
	{
		write_1();
	}

	/* write out reg to access */
	for(i=0;i<16;i++)
	{
		if(wreg & 0x8000)
			write_1();
		else
			write_0();

		wreg <<= 1;
	}

	/* write out data value */
	for(i=0;i<16;i++)
	{
		if(wdata & 0x8000)
			write_1();
		else
			write_0();

		wdata <<= 1;
	}
    EtherC.PIR.LONG=0;		/* Put MDIO line back to HI-Z */
}

static unsigned short read_PHY(unsigned short rdata)
{
    int i, count;
    unsigned short reg_val = 0;

	/* write out preamble sequence */
	for(i=0;i<32;i++)
	{
		write_1();
	}

	/* first 14 bits are driven by us */
    for(i=0;i<14;i++)
    {
        if(rdata & 0x8000)
            write_1();
        else
            write_0();
        rdata <<= 1;
    }

    /* now the PHY chip will take over the bus */
	EtherC.PIR.LONG=0;			// set pin direction as input
	delay();
	EtherC.PIR.bit.MDC=1;		// clock once (line should be in Hi-Z)
	delay();
	EtherC.PIR.LONG=0;			// clock low
	delay();

    /* now the PHY chip has taken over the bus */

	/* now clock in data */
    for(i=0; i<16; i++)
    {   
		reg_val <<= 1;

		EtherC.PIR.bit.MDC=1;		// clock pin HIGH
		delay();
		reg_val |= EtherC.PIR.bit.MDI;		// read data pin and add in value
		EtherC.PIR.bit.MDC=0;		// clock pin LOW
		delay();
    }

    EtherC.PIR.LONG=0;		/* Put MDIO line back to HI-Z */

    return (reg_val);
}

static void delay()
{
	volatile int i;
	for(i=0;i<10;i++)
	{
		i++;
		i--;	
	}

}





