/*
 *Copyright (c) 2000-2002 Viola Systems Ltd.
 *All rights reserved.
 *
 *Redistribution and use in source and binary forms, with or without 
 *modification, are permitted provided that the following conditions 
 *are met:
 *
 *1. Redistributions of source code must retain the above copyright 
 *notice, this list of conditions and the following disclaimer.
 *
 *2. Redistributions in binary form must reproduce the above copyright 
 *notice, this list of conditions and the following disclaimer in the 
 *documentation and/or other materials provided with the distribution.
 *
 *3. The end-user documentation included with the redistribution, if 
 *any, must include the following acknowledgment:
 *	"This product includes software developed by Viola 
 *	Systems (http://www.violasystems.com/)."
 *
 *Alternately, this acknowledgment may appear in the software itself, 
 *if and wherever such third-party acknowledgments normally appear.
 *
 *4. The names "OpenTCP" and "Viola Systems" must not be used to 
 *endorse or promote products derived from this software without prior 
 *written permission. For written permission, please contact 
 *opentcp@opentcp.org.
 *
 *5. Products derived from this software may not be called "OpenTCP", 
 *nor may "OpenTCP" appear in their name, without prior written 
 *permission of the Viola Systems Ltd.
 *
 *THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED 
 *WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 *MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 *IN NO EVENT SHALL VIOLA SYSTEMS LTD. OR ITS CONTRIBUTORS BE LIABLE 
 *FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 *CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 *SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
 *BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 *OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
 *EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *====================================================================
 *
 *OpenTCP is the unified open source TCP/IP stack available on a series 
 *of 8/16-bit microcontrollers, please see <http://www.opentcp.org>.
 *
 *For more information on how to network-enable your devices, or how to 
 *obtain commercial technical support for OpenTCP, please see 
 *<http://www.violasystems.com/>.
 */

/** \file https_callbacks.c
 *	\brief HTTP server callback functions
 *	\author 
 *		\li Jari Lahti (jari.lahti@violasystems.com)
 *	\version 1.0
 *	\date 9.10.2002
 *  \bug
 *	\warning
 *	\todo
 *
 *	This file holds definitions and descriptions of HTTP callback functions
 *	that will be overridden by user code. These callback functions are 
 *	invoked by HTTP server to get a feedback from the part of HTTP server
 *	who's behaviour is defined by the user application.
 */ 
//#include<inet/datatypes.h>
//#include<inet/debug.h>
//#include<inet/globalvariables.h>
#include<inet/system.h>
#include<inet/http/http_server.h>


/** \brief Brief function description here
 * 	\author 
 *		\li Jari Lahti (jari.lahti@violasystems.com)
 *	\date 09.10.2002
 *	\param hash Calculated file-name hash value. Used so that the whole
 *		file name doesn't need to be stored in RAM
 *	\param ses HTTP session identifier
 *	\return
 *		\li -1 - This function should return -1 if no file has been found
 *		\li 1 - This function should return 1 if a file with appropriate
 *			hash value has been found.
 *	\warning
 *		\li This function <b>MUST</b> be implemented by user application
 *		to work with local configuration
 *
 *	Appropriate https session entry is filled accordingly.	
 *
 */
INT16 https_findfile (UINT8 hash, UINT8 ses)
{
	/* we are not using the hash system, so "hash" can be ignored */
	INT8 i,form_req = 0,j;
	char file_to_find[MAX_FILENAME+1];
	char lcd_string[9] = "        ";
	
	for( i=0; i<MAX_FILENAME; i++)
	{
		file_to_find[i] = RECEIVE_NETWORK_B();
		if(file_to_find[i] == ' ')		// look for end of filename
			break;
		if(file_to_find[i] == '?')		// look '?' which means we are getting a FORM request
		{
			form_req = 1;
			break;
		}

	}
	file_to_find[i] = 0;		// insert NULL at the end

	if( form_req )
	{
		RECEIVE_NETWORK_B();	// 'l'
		RECEIVE_NETWORK_B();	// 'n'
		RECEIVE_NETWORK_B();	// '1'
		RECEIVE_NETWORK_B();	// '='
		j=0;
		while(form_req != ' ')
		{
			form_req = RECEIVE_NETWORK_B();
			switch( form_req )
			{
				case '&':
					//lcd_string[j] = 0;
					DisplayString(LCD_LINE1,lcd_string);
					for(j=0;j<8;j++)
						lcd_string[j] = ' ';
					j=0;
					RECEIVE_NETWORK_B();	// 'l'
					RECEIVE_NETWORK_B();	// 'n'
					RECEIVE_NETWORK_B();	// '2'
					RECEIVE_NETWORK_B();	// '='
					break;
				case ' ':		// end of string
					DisplayString(LCD_LINE2,lcd_string);
					break;
					break;
				case '+':		// spaces are represented by '+'
					lcd_string[j++] = ' ';
					break;
				case '%':		// special characters are represneted in hex by %xx
						//lcd_string[j++] = '%';
						form_req = RECEIVE_NETWORK_B() - '0';	// remove ASCII offset
						lcd_string[j] = form_req <<= 4;			// shift to upper nibble and save
						form_req = RECEIVE_NETWORK_B() - '0';
						if( form_req > 9)
							form_req = form_req +  '0' - 'A' + 10;	// char was A - F
						lcd_string[j++] += form_req;
						break;
				default:
					lcd_string[j++] = form_req;
					
			}
		}
	}

	/* If there was no filename, then client side is requesting the default file */
	if( i )
		i = FindFile((UINT32) file_to_find);
	else
		i = 2;		// index.html should be kept in the 3nd FAT location

	/* File Not Found? */
	if( i < 0 )
		i = 1;		// set the file found to be the 404 File Not Found page

	/* OK, file found. */	
	https[ses].fstart = i;		/* we will use fstart to hold the FAT index (can never be 0)*/
	https[ses].flen = GetFileSize(i);
	https[ses].fpoint = 0;		// current pointer within the file to start with
	https[ses].funacked = 0;	// no unacked data for now
	
	return(1);  
	
}

/** \brief Fill network transmit buffer with HTTP headers&data
 * 	\author
 *		\li Jari Lahti (jari.lahti@violasystems.com)
 *	\date 09.10.2002
 *	\param ses HTTP session identifier
 *	\param buf Pointer to buffer where data is to be stored
 *	\param buflen Length of the buffer in bytes
 *	\return
 *		\li >=0 - Number of bytes written to buffer
 *	\warning 
 *		\li This function <b>MUST</b> be implemented by user application
 *		to work with local configuration
 *
 *	This handlers' job is to fill the buffer with the data that web server
 *	should return back through the TCP connection. This is accomplished
 *	based session identifer and values of variables in appropriate
 *	https entry.
 */
INT16 https_loadbuffer (UINT8 ses, UINT8* buf, UINT16 buflen) {
	UINT16 i,bytes_to_send;
	
	/* Send the file */
	bytes_to_send = https[ses].flen - https[ses].fpoint;	// bytes left to send
	if( bytes_to_send > buflen)
		bytes_to_send = buflen;		// can only send up to buflen of bytes now

	/* use files system copy routine */
	ReadFile(	https[ses].fstart,		// hold FAT index of our file
				https[ses].fpoint,		// our current index into the file
				buf,					// The buffer to copy into
				bytes_to_send);			// how much to copy
	
	return(bytes_to_send);	// return how many bytes we got.
}
