/***********************************************************************
 * DISCLAIMER:                                                         *
 * THIS SOFTWARE IS PROVIDED "AS IS". NO WARRANTIES, WHETHER EXPRESS,  *
 * IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO IMPLIED 		   *
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE  *
 * APPLY TO THIS SOFTWARE. RENESAS TECHNOLOGY AMERICA, INC. AND        *
 * AND RENESAS TECHNOLOGY CORPORATION RESERVE THE RIGHT, WITHOUT       *
 * NOTICE, TO MAKE CHANGES TO THIS SOFTWARE. NEITHER RENESAS           *
 * TECHNOLOGY AMERICA, INC. NOR RENESAS TECHNOLOGY CORPORATION SHALL,  * 
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR         *
 * CONSEQUENTIAL DAMAGES FOR ANY REASON WHATSOEVER ARISING OUT OF THE  *
 * USE OR APPLICATION OF THIS SOFTWARE.                                *
 ***********************************************************************/

/*********************************************************************
 *											   						 *
 *    DESCRIPTION: main routine for Timer - Event Counter Mode 	 	 *
 *											  						 *
 *											   						 *
 *    PURPOSE: Outlines how to setup the M16C/62P Timer A in Event 	 *
 *             Counter mode.  			 							 *
 *											   						 *
 *			   Timer A3 is setup in timer even mode.    			 *
 ********************************************************************/
/*===========================================================================
*   Revision history: 
*
*==========================================================================*/

/* Include the required header files */
#include "skp_bsp.h"	// include SKP board support package
#include "sfr62p.h"
#include <inet/datatypes.h>

#define  TIME_CONFIG 0x11  
				/*  00010001 value to load into timer mode register
		            ||||||||_  TMOD0,TMOD1: EVENT COUNTER MODE 
		            ||||||____ MR0:         NO PULSE OUTPUT   
		            |||||_____ MR1:         COUNT FALLING EDGES
		            ||||_______MR2:         TAiOut PIN'S INPUT SIGNAL
		            |||_______ MR3:         = 0 IN EVENT COUNTER MODE     
		            ||________ TCK0:        RELOAD TYPE
		            |__________TCK1:        BIT NOT USED                   */

                             
#define  CNTR_IPL 	    0x03	// TA3 priority interrupt level

#define	WOL_DDR 		pd7_6   // Wake-on-LAN LED (LED7) is connected to PD7-6
#define	WOL_LED		    p7_6    // LED7 is on when PMEN pin is low

#define	WOL_PMEN_DDR 	pd7_7   // KS8841 Wake-on-LAN PMEN pin is connected to PD7-7

// Global varibal 

UINT8 pmen_pin_low = 1;	        /* Hold which PMEN pin is currently low */
UINT8 wol_lcd = 0;              /* TRUE to display WOL message on LCD */

extern const char OpenTCP_EthDriver_ver[];
extern const char QC_EthDriver_ver[];


//prototypes

void timerTA3_init(void);
void wol_irq(void);


/*****************************************************************************
Name:  timerTA3_init()   
Parameters: none                   
Returns:  nothing      
Description: Initializes Timer TA3 to event counter mode. An interrupt is
			 generated by external signal - when KS8841 PMEN is low
**************************************************************************** */
void timerTA3_init()
{

#if defined (DEF_KS8841) || defined (KS_QC_TEST)

    WOL_PMEN_DDR = 0;	// set KS8841 Wake-on-LAN PMEN port as input
    WOL_DDR = 1;		// set Wake-on-LAN LED port as output
    WOL_LED = LED_OFF; 	  
    ta3 = 0xffff;       

    ta3ic = CNTR_IPL;	// Enable TA3IC, falling edge triggered, priority level 3 
    ta3mr = TIME_CONFIG;// Configure Timer TA3 to Timer Event mode

    ta3s = 1;           // start timers 

#endif /* #if defined (DEF_KS8841) || defined (KS_QC_TEST) */
}

/*****************************************************************************
Name: wol_irq()       
Parameters: none                     
Returns:  	nothing      
Description: Wake-on-LAN (Timer A3) Interrupt Service Routine. Interrupts falling
             edges on the TA3in pin (also KS8841 PMEN pin). Turn on the LED7.

**************************************************************************** */
#pragma INTERRUPT wol_irq 
void wol_irq(void)
{ 
	UINT8 old_value;

   	WOL_LED = LED_ON;
    
  	old_value = pmen_pin_low;	// Save old value
    pmen_pin_low = 0;           // PMEN pin is low

    /* Signal to display WOL message to LCD */
    if ( pmen_pin_low != old_value)
    {
        wol_lcd = TRUE;
    }
}  


/*****************************************************************************
Name: wol_lcd_on()       
Parameters: none                     
Returns:  	nothing      
Description: Display "Received Magic" on LCD when KS8841 detected the Magic Packet

**************************************************************************** */
void wol_lcd_on ()
{ 

	DisplayString(LCD_LINE1,"Received");
	DisplayString(LCD_LINE2,"Magic   ");
    wol_lcd = FALSE;
} 

/*****************************************************************************
Name: wol_lcd_off()       
Parameters: none                     
Returns:  	nothing      
Description: Clear display "Received Magic" on LCD when clear KS8841 PME_Status

**************************************************************************** */
void wol_lcd_off ()
{ 

   WOL_LED = LED_OFF; 
   pmen_pin_low = 1;
   wol_lcd = FALSE;
   DisplayString(LCD_LINE1, "Version ");
#ifndef KS_QC_TEST
   DisplayString(LCD_LINE2, OpenTCP_EthDriver_ver);
#else
   DisplayString(LCD_LINE2, QC_EthDriver_ver);
#endif
} 

