/* aliM1543CPibc.h - ALi M1543C PIBC, PIC and SuperIO header file */

/* Copyright 2000-2001 Wind River Systems, Inc. */


/*
modification history
--------------------
01c,11jul01,h_k  added macros for PIBC.
01b,11jun01,h_k  added PMU ABA and SBA definition.
01a,23apr01,zl   written.
*/

#ifndef __INCaliM1543CPibch
#define __INCaliM1543CPibch

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/* M1543C SuperIO defines */

#define	SUPERIO_CFG_PORT	0x3f0
#define	SUPERIO_INDEX_PORT	0x3f0
#define	SUPERIO_DATA_PORT	0x3f1

#define	CFG_KEY1		0x51
#define	CFG_KEY2		0x23
#define	EXIT_KEY		0xbb

#define	LOG_DEV_FDC		0x00	/* Floppy Disk Ctrl */
#define	LOG_DEV_LPT		0x03	/* Parallel port */
#define	LOG_DEV_UART1		0x04	/* UART1 */
#define	LOG_DEV_UART2		0x05	/* UART3 */
#define	LOG_DEV_KBD		0x07	/* Keyboard & mouse */

#define INDEX_RESET		0x02	/* Reset index */
#define	INDEX_DEV		0x07	/* Logical device number */
#define	INDEX_IDL		0x20    /* Chip ID Low */
#define	INDEX_IDH		0x21    /* Chip ID High */
#define	INDEX_DEVEN		0x30	/* Logical device enable */
#define	INDEX_IOBASEH		0x60	/* IO Base MSB */
#define	INDEX_IOBASEL		0x61	/* IO BAse LSB */
#define	INDEX_IRQSEL1		0x70	/* Primary IRQ */
#define	INDEX_IRQSEL2		0x72	/* Secondary IRQ (e.g. mouse) */

#define CFG_REG_INDEX_PIC	0x40	/* PCI Interface Control */
#define CFG_REG_INDEX_IORC	0x41	/* IO Recovery Control */
#define CFG_REG_INDEX_ISAC1	0x42	/* ISA Bus Cycle Control 1 */
#define CFG_REG_INDEX_ISAC2	0x43	/* ISA Bus Cycle Control 2 */
#define CFG_REG_INDEX_IDENR1	0x44	/* IDE Native Mode Interrupt Routing 1 */
#define CFG_REG_INDEX_PIPM	0x45	/* PCI Interrupt Polling Mode */
#define CFG_REG_INDEX_BCSC	0x47	/* BIOS Chip Select Contorol */
#define CFG_REG_INDEX_PIRT1	0x48	/* PCI Interrupt to ISA IRQ Routing Table 1 */
#define CFG_REG_INDEX_PIRT2	0x49	/* PCI Interrupt to ISA IRQ Routing Table 2 */
#define CFG_REG_INDEX_PIRT3	0x4a	/* PCI Interrupt to ISA IRQ Routing Table 3 */
#define CFG_REG_INDEX_PIRT4	0x4b	/* PCI Interrupt to ISA IRQ Routing Table 4 */
#define CFG_REG_INDEX_PILET	0x4c	/* PCI Interrupt Level to Edge Transfer */
#define CFG_REG_INDEX_IDEIC	0x58	/* IDE Interface Control */
#define CFG_REG_INDEX_USBIR	0x74	/* USB Interrupt Routing Table */
#define CFG_REG_INDEX_IDENR2	0x75	/* IDE Native Mode Interrupt Routing 2 */

/* PCI Interface Control */
#define PIC_DMA_LINE		0x1
#define PIC_ISA_MASTER_LINE	0x2
#define PIC_PCI_TO_ISA		0x4
#define PIC_DELAY_TRANSACT	0x8
#define PIC_PASSIVE_REL		0x10
#define PIC_SELECT_ISA_MASTER	0x20
#define PIC_SOUND		0x40

/* IO Recovery Control */
#define IORC_ISA		0x1
#define IORC_ONCHIP		0x2
#define IORC_0_00		0x0
#define IORC_0_25		0x4
#define IORC_0_50		0x8
#define IORC_0_75		0xc
#define IORC_1_00		0x10
#define IORC_1_25		0x14
#define IORC_1_50		0x18
#define IORC_1_75		0x1c
#define IORC_2_00		0x20
#define IORC_2_25		0x24
#define IORC_2_50		0x28
#define IORC_2_75		0x2c
#define IORC_3_00		0x30
#define IORC_3_25		0x34
#define IORC_3_50		0x38
#define IORC_3_75		0x3c

/* ISA Bus Cycle Control 1 */
#define ISAC1_7_16		0x0
#define ISAC1_PCICLK_DIV2	0x1
#define ISAC1_PCICLK_DIV3	0x2
#define ISAC1_PCICLK_DIV4	0x3
#define ISAC1_PCICLK_DIV5	0x4
#define ISAC1_PCICLK_DIV6	0x5
#define ISAC1_DECOUP_REFRESH	0x8
#define ISAC1_DMA_HIGH_PAGE	0x40
#define ISAC1_CPRD_MASK		0x80

/* ISA Bus Cycle Control 2 */
#define ISAC2_ISA_IO_NORM_16	0x0
#define ISAC2_ISA_IO_1_WAIT	0x1
#define ISAC2_ISA_IO_2_WAIT	0x2
#define ISAC2_ISA_IO_3_WAIT	0x3
#define ISAC2_ISA_MEM_NORM_16	0x0
#define ISAC2_ISA_MEM_1_WAIT	0x4
#define ISAC2_ISA_MEM_2_WAIT	0x8
#define ISAC2_ISA_MEM_3_WAIT	0xc
#define ISAC2_ISA_REFRESH_15	0x0
#define ISAC2_ISA_REFRESH_30	0x10
#define ISAC2_ISA_REFRESH_60	0x20
#define ISAC2_ISA_REFRESH_120	0x30
#define ISAC2_COPRO_INTERFACE	0x40
#define ISAC2_PORT_92H_SELECT	0x80

/* IDE Native Mode Interrupt Routing 1 */
#define IDENR1_PRIM_CHAN_INT	0x10
#define IDENR1_IO_DECODE	0x40
#define IDENR1_SOFT_RESET_CTL	0x80

/* IDE Native Mode Interrupt Routing 2 */
#define IDENR2_SEC_CHAN_INT_SIG	0x10

/* USB Interrupt Routing Table */
#define USBIR_MASTER_INTAJ	0x10

/* PCI Interrupt Polling Mode */
#define PIPM_PARITY_CHECK	0x1
#define PIPM_DISTR_DMA		0x2
#define PIPM_DELAY_TRAN_TIMEOUT	0x8
#define PIPM_ROMKBCSJ		0x40
#define PIPM_INT_POL_MODE	0x80

/* BIOS Chip Select Contorol */
#define BCSC_ROM_64KB		0x0
#define BCSC_ROM_128KB		0x1

/* PCI Interrupt to ISA IRQ Routing Table 1 - 4 */
#define PIRT_INT_DISABLE	0x0

/* PCI Interrupt Level to Edge Transfer */
#define PILET_INT1J		0x1
#define PILET_INT2J		0x2
#define PILET_INT3J		0x4
#define PILET_INT4J		0x8
#define PILET_INT5J		0x10
#define PILET_INT6J		0x20
#define PILET_INT7J		0x40
#define PILET_INT8J		0x80

/* IDE Interface Control */
#define IDEIC_SIRQI_SIRQII	0x0
#define IDEIC_IRQ14_IRQ15	0x1
#define IDEIC_IRQ14_SIRQII	0x2
#define IDEIC_IRQ14_SIRQI	0x3
#define IDEIC_ATA_PRI_SIG_PAD	0x4
#define IDEIC_ATA_SEC_SIG_PAD	0x8
#define IDEIC_IDSEL_A27		0x0
#define IDEIC_IDSEL_A26		0x10
#define IDEIC_IDSEL_A25		0x20
#define IDEIC_IDSEL_A24		0x30
#define IDEIC_ONCHIP_IDE_CONTR	0x40

#define	DEV_EN			0x01

/* SuperIO definitions */

#define M1543SIO_ID		0x1543

#define UART1_OFFSET		0x03f8
#define UART2_OFFSET		0x02f8
#define KBD_OFFSET		0x0060
#define MOUSE_OFFSET		0x0060

/* Programmable Interrupt Controller (PIC) Registers */

#define PIC1_PORT1		0x20
#define PIC1_PORT2		0x21
#define PIC2_PORT1		0xa0
#define PIC2_PORT2		0xa1

#define PIC1_ICW1		PIC1_PORT1
#define PIC1_OCW2		PIC1_PORT1
#define PIC1_OCW3		PIC1_PORT1
#define PIC1_ICW2		PIC1_PORT2
#define PIC1_ICW3		PIC1_PORT2
#define PIC1_ICW4		PIC1_PORT2
#define PIC1_OCW1		PIC1_PORT2

#define PIC2_ICW1		PIC2_PORT1
#define PIC2_OCW2		PIC2_PORT1
#define PIC2_OCW3		PIC2_PORT1
#define PIC2_ICW2		PIC2_PORT2
#define PIC2_ICW3		PIC2_PORT2
#define PIC2_ICW4		PIC2_PORT2
#define PIC2_OCW1		PIC2_PORT2

/* PMU ABA and SBA IO Base Address */
#define PMU_ABA_IO_BASE_ADRS	0x00000801		/* 2048 byte size */
#define PMU_SBA_IO_BASE_ADRS	0x00000841		/* 2048 byte size */

/* RTC port definitions */

#define	RTC_AREG		0x070
#define	RTC_DREG		0x071

#ifndef _ASMLANGUAGE

/* function declarations */

#if (defined(__STDC__) || defined(__cplusplus))

STATUS  sysPicIntEnable (int intNum);
STATUS  sysPicIntDisable (int intNum);
STATUS  sysPibcInit ();
UCHAR   aliM1543CRtcRead (UCHAR reg);
void    aliM1543CRtcWrite (UCHAR reg, UCHAR val);

#else   /* __STDC__ */

STATUS  sysPicIntEnable ();
STATUS  sysPicIntDisable ();
STATUS  sysPibcInit ();
UCHAR   aliM1543CRtcRead ();
void    aliM1543CRtcWrite ();

#endif  /* __STDC__ */

#endif  /* _ASMLANGUAGE */

#ifdef __cplusplus
}
#endif /* __cplusplus */
#endif /* __INCaliM1543CPibch */
