/* config.h - Hitachi ULSI MS7751RSE configuration header */

/* Copyright 2003      Wind River Systems, Inc. */
	
/*
modification history
--------------------
20Aug04     pcd  added support for Micrel ks884x network controller by "INCLUDE_KS884X_END"
01a,16jun03,h_k  written based on ms7751se/config.h 01k. (SPR #87000)
*/

/*
This file contains the configuration parameters for the
Hitachi MS7751RSE01 target.
*/

#ifndef	INCconfigh
#define	INCconfigh

#ifdef __cplusplus
extern "C" {
#endif

/* BSP version/revision identification, before configAll.h */

#define BSP_VER_1_1		1		/* 1.2 is backward compatible */
#define BSP_VER_1_2		1		/* with 1.1 */
#define BSP_VERSION		"1.2"
#define BSP_REV			"/0"		/* 0 for first revision */

#include "configAll.h"


/* CPU type dependent macros */

#define	SH_CACHE_THRU		SH7751_P2_BASE	/* in cacheShLib.h */
#define CACHE_ARCH_INIT_RTN	cacheSh7750LibInit
#define INTEVT_ADRS		0xff000028
#define SYS_MODEL		"HITACHI ULSI MS7751RSE"

/* Supported CPU and PCLK clock rate */

#define SYS_CPU_FREQ		240000000	/* 240MHz internal */
#define SYS_PCLK_FREQ		60000000	/* 60MHz peripheral clock */
#define SYS_BUS_FREQ		60000000	/* 60MHz external bus */

#define INCLUDE_FLASH_BOOT	/* flash driver in boot ROM */

/* Defualt boot line */

#define DEFAULT_BOOT_LINE \
"lnPci(0,0)host:/tor/target/config/ms7751Rse/vxWorks h=90.0.0.1 e=90.0.0.2 u=target"

/*
 * PCI memory (and IO) space extension
 *
 * SH7751 has some size limitations on its PCI windows (16MB for memory space,
 * and 256KB for IO space).  The INCLUDE_SH7751_VIRTUAL_PCI_MEMIO option enables
 * to extend the PCI memory space using a special MMU support with a minimal
 * performance penalty.  Also the INCLUDE_SH7751_VIRTUAL_PCI_IO option
 * extends the PCI IO space in addition to the PCI memory space, but it adds
 * another few more runtime overhead in the MMU support so that it should be
 * undefined if you do not need the PCI IO space extension.
 *
 * Note that both options should be undefined when you build a Boot ROM,
 * since Boot ROMs do not enable MMU so that the virtual PCI spaces are unseen
 * to the default boot device (lnPci) in Boot ROM.
 */

#undef	INCLUDE_SH7751_VIRTUAL_PCI_MEMIO	/* extend PCI memory space */
#undef	INCLUDE_SH7751_VIRTUAL_PCI_IO		/* extend PCI memory/IO space */

/* Bundled MMU support */

#undef	INCLUDE_MMU_BASIC			/* default is MMU OFF */

/* 
 * Cache mode
 * SH7751 CPU has a capability to allocate "cache-safe" buffers from P2 space
 * regardless of MMU.  The CACHE_DMA_BYPASS_P[013] macros are desiged to use
 * this capability.  Here we assume that the memory pool is to be allocated
 * on P0 space.  Change it if you put the memory pool on other space.
 *
 * The SH7751R version of the CPU has larger, 2-way associative cache. In
 * order to take advantage of this feature, add CACHE_2WAY_MODE to the 
 * USER_D_CACHE_MODE definition. This will instruct the cache library to
 * enable the Enhanced Mode bit in the cache configuration register (CCR).
 */

#undef	USER_D_CACHE_MODE
#define	USER_D_CACHE_MODE \
		(CACHE_COPYBACK | CACHE_COPYBACK_P1 | CACHE_DMA_BYPASS_P0 | \
		 CACHE_2WAY_MODE)

/* serial ports */

#undef  INCLUDE_SCI_SIO			    /* SCI channel on SH7751 */
#define INCLUDE_SCIF_SIO		    /* SCIF channel on SH7751 */
#define INCLUDE_NS16550_SIO		    /* Serial port on M1543C */
#define NUM_NS16550_CHANNELS	1	/* number of ns16550 channels */
#undef  NUM_TTY
#define NUM_TTY			        2	/* one SCIF and ns16550 */
#undef  CONSOLE_BAUD_RATE
#define CONSOLE_BAUD_RATE    38400

 
/* network interface */

#ifdef INCLUDE_NETWORK

#define INCLUDE_END 			/* Enhanced Network Driver */
#define INCLUDE_LN_97X_END		/* Use the AMD 79C973 */
#define ETHERNET_ADR_SET		/* use bootrom N command to set 
					               Ethernet addrs. in serial EEPROM */

#ifdef KS_PCI_BUS
#define INCLUDE_KS884X_END		 /* Use the Micrel KS884x with PCI bus interface */
#endif

#ifdef KS_ISA_BUS
#define INCLUDE_KS884X_SHBUS_END /* Use the Micrel KS884x with Generic bus interface */
#endif



/* 
 * manufacturer's code (example 00:00:87:xx:xx:xx) upper 3 bytes;
 * fixed MAC addr. Byte order needs to comply with mEnet() routine 
 * in bootConfig.c.
 */

#if _BYTE_ORDER == _BIG_ENDIAN
# define ENET_DEFAULT		0x87000000
#else
# define ENET_DEFAULT		0x00000087
#endif


#ifdef  INCLUDE_END
#undef  WDB_COMM_TYPE
#define WDB_COMM_TYPE		WDB_COMM_END
#endif  /* INCLUDE_END */

#endif  /* INCLUDE_NETWORK */


/* IDE/ATA */

#undef INCLUDE_ATA

#define ATA_SEM_TIMEOUT		0	/* value 0 means use default value */
#define ATA_WDG_TIMEOUT		0	/* value 0 means use default value */

/* External Interrupt vectors */

#define INT_VEC_ABORT		INUM_TO_IVEC (INUM_NMI)     /* abort      */
#define INT_VEC_IRQ8		INUM_TO_IVEC (INUM_IRL15)   /* Exp. IRQ8  */
#define INT_VEC_IRQ7		INUM_TO_IVEC (INUM_IRL14)   /* Exp. IRQ7  */
#define INT_VEC_PCMCIA2		INUM_TO_IVEC (INUM_IRL13)   /* MR-SHPC 2  */
#define INT_VEC_PIBC		INUM_TO_IVEC (INUM_IRL12)   /* M1543C     */
#define INT_VEC_IRQ6		INUM_TO_IVEC (INUM_IRL11)   /* Exp. IRQ6  */
#define INT_VEC_IRQ5		INUM_TO_IVEC (INUM_IRL10)   /* Exp. IRQ5  */
#define INT_VEC_PCMCIA1		INUM_TO_IVEC (INUM_IRL9)    /* MR-SHPC 1  */
/* #define not used		INUM_TO_IVEC (INUM_IRL8)                  */
#define INT_VEC_IRQ4		INUM_TO_IVEC (INUM_IRL7)    /* Exp. IRQ4  */
#define INT_VEC_IRQ3		INUM_TO_IVEC (INUM_IRL6)    /* Exp. IRQ3  */
/* #define not used		INUM_TO_IVEC (INUM_IRL5)                  */
#define INT_VEC_IRQ2		INUM_TO_IVEC (INUM_IRL4)    /* Exp. IRQ2  */
/* #define not used     INUM_TO_IVEC (INUM_IRL3)                  */
#define INT_VEC_KS884X		INUM_TO_IVEC (INUM_IRL2)    /* Exp. IRQ1 (KS884x IRQ1) */
#define INT_VEC_PCMCIA0		INUM_TO_IVEC (INUM_IRL1)    /* MR-SHPC 0  */

/* 
 * On-chip device interrupt levels. The PCIC levels are set in INTPRI00,
 * PCIERR, PCIPWDN, ... PCIDMA0 sharing the same level.
 */

#define INT_LVL_SYSCLK		15	/* system clock      (TMU0) */
#define INT_LVL_AUXCLK		14	/* aux clock         (TMU1) */
#define INT_LVL_PCISERR		10	/* PCI SERR error interrupt */
#define INT_LVL_PCIERR		9	/* PCIC error interrupt     */
#define INT_LVL_PCIPWDN		9	/* PCIC power down req intr */
#define INT_LVL_PCIPWON		9	/* PCIC power on req intr   */
#define INT_LVL_PCIDMA3		9	/* PCIC DMA3 trans end intr */
#define INT_LVL_PCIDMA2		9	/* PCIC DMA2 trans end intr */
#define INT_LVL_PCIDMA1		9	/* PCIC DMA1 trans end intr */
#define INT_LVL_PCIDMA0		9	/* PCIC DMA0 trans end intr */
#define INT_LVL_SCI		8	/* serial port       (SCI)  */
#define INT_LVL_SCIF		7	/* serial port       (SCIF) */
#define INT_LVL_TSTAMP		1	/* timestamp counter (TMU2) */
 
 
/* M1543C PIC interrupt definitions */

#define PIC_INT_TABLESIZE	16
#define PIC_INUM_BASE		0x70	/* vector numbers 112-127 reserved 
					   for ISA and PCI */
#define PIC_IVEC_BASE		INUM_TO_IVEC (PIC_INUM_BASE)

/* 
 * M1543C on-chip PIC interrupt numbers. Valid values in priority order 
 * starting with the highest: 0, 1, 9, 10, 11, 12, 14, 15, 3, 4, 5, 6, 7, 8.
 * PCI interrupts must be consecutive numbers for the pciIntCOnnect() routine,
 * INT_NUM_IRQ0 being set to the PIC_INTA interrupt number. IRQ0 (Timer) and
 * IRQ8 (RTC) are fixed and cannot be used by any other device.
 */

#define PIC_STU			0	/* M1543C System Timer */
#define PIC_KBD			1	/* Keyboard */
#define PIC_RTC			8	/* RTC */
#define PIC_MOUSE		9	/* Mouse */
#define PIC_UART1		10	/* UART1 */
#define PIC_UART2		11	/* UART2 */
#define PIC_USB			12	/* USB */
#define PIC_IDE1		14	/* IDE1 */
#define PIC_INTA		3	/* PCI INTA */
#define PIC_INTB		4	/* PCI INTB */
#define PIC_INTC		5	/* PCI INTC */
#define PIC_INTD		6	/* PCI INTD */
#define PIC_IDE2		7	/* IDE2 */


#define INT_NUM_IRQ0		(PIC_INUM_BASE + PIC_INTA)	
					/* PCI ints base, see pciIntLib.c */
#define PIC_IVEC_TO_IRQ(v)	(((UINT)(v) - (UINT)PIC_IVEC_BASE) >> 2)
					/* Convert a vector to PIC IRQ */
#define PIC_IRQ_TO_IVEC(n)	((VOIDFUNCPTR *)(((n) << 2) + (UINT)PIC_IVEC_BASE))
					/* Convert a PIC IRQ to a vector */


/* Miscellaneous definitions */

#define TIMESTAMP_LEVEL 	INT_LVL_TSTAMP  /* needed by sh7700Timer.c */


/* Reserve last pair of sectors in flash for NVRAM */

#define INCLUDE_FLASH
#undef  INCLUDE_NVM_SPACE

#define NV_OFF_BINDFIX		0x0108	    /* See sysLib.c:sysBindFix() (1b) */

#define INCLUDE_NVRAM_TO_FLASH
#define FLASH_WIDTH		4
#define FLASH_CHIP_WIDTH	2
#define FLASH_ADRS		(SH7751_P2_BASE + 0x3F8000)
						/* last sector in Flash */
#define FLASH_SIZE		0x8000		/* 32KB: 2x8KWord sectors */
#define NV_RAM_SIZE     	FLASH_SIZE
#define FLASH29_REG_ADRS(reg)	(SH_CACHE_THRU + (reg * FLASH_WIDTH))

/* TrueFFS */

#define	TFFS_BASE_ADDR		(SH_CACHE_THRU + ROM_BASE_ADRS)
#define	TFFS_FLASH_SIZE		0x003E0000	/* Exclude the top four pairs
						   of sectors. The first 1M is
						   reserved for bootrom, the
						   rest is for the file system.
						   See also: sysTffsFormat() */
#define	TFFS_WIN_SIZE		0x00400000	/* Smallest power of two larger
						   than TFFS_FLASH_SIZE */


/* WindML configuration */

#undef	INCLUDE_WINDML
#undef	INCLUDE_PCI_WINDML_GRAPHICS
#undef	INCLUDE_WINDML_PS2_POINTER
#undef	INCLUDE_WINDML_PS2_KEYBOARD


/* Memory addresses */

#define LOCAL_MEM_LOCAL_ADRS	0x0c000000
#define LOCAL_MEM_SIZE		0x00800000	/* 4M, up to 64M Bytes  */
#undef  LOCAL_MEM_AUTOSIZE			/* dynamic memory sizing */

/* User reserved memory, see sysMemTop */

#define USER_RESERVED_MEM	0		/* number of reserved bytes */

/*
 * The constants ROM_TEXT_ADRS, ROM_SIZE, RAM_HIGH_ADRS, and RAM_LOW_ADRS
 * are defined in config.h and Makefile.
 * All definitions for these constants must be identical.
 */

#define ROM_BASE_ADRS		0
#define ROM_TEXT_ADRS		ROM_BASE_ADRS
#define ROM_WARM_ADRS		(((ROM_TEXT_ADRS + 8) & SH7751_PHYS_MASK) | SH7751_P2_BASE)
#define ROM_SIZE		0x00100000	/* 1M Bytes in Flash */

#define RAM_LOW_ADRS		(LOCAL_MEM_LOCAL_ADRS + 0x2000)
#define RAM_HIGH_ADRS		(LOCAL_MEM_LOCAL_ADRS + 0x400000)
						/* RAM address for ROM boot */

/*
 * Default power management mode - selected via vxPowerModeSet() in
 * sysHwInit().
 * DEFAULT_POWER_MGT_MODE can be VX_POWER_MODE_DISABLE, VX_POWER_MODE_SLEEP,
 * VX_POWER_MODE_DEEP_SLEEP (SH4 only) or
 * VX_POWER_MODE_USER | ([0x00 - 0xff] << 8) (8-bit value passed to STBCR) |
 * ([0x00 - 0xff] << 16) (8-bit value passed to STBCR2).
 * VX_POWER_MODE_xxx are defined in vxShLib.h.
 */

#define DEFAULT_POWER_MGT_MODE	VX_POWER_MODE_DISABLE

/* Exclude unsupported libraries */

#undef  INCLUDE_SM_NET


#if defined(PRJ_BUILD)
#include "prjParams.h"
#endif

/* target include */

#include "ms7751se.h"

#ifdef __cplusplus
}
#endif

#endif /* INCconfigh */
