/* romInit.s - Hitachi ULSI MS7751RSE01 ROM initialization module */

/* Copyright 2003      Wind River Systems, Inc. */

	.data
	.globl	_copyright_wind_river
	.long	_copyright_wind_river

/*
modification history
--------------------
01a,16jun03,h_k  written based on ms7751se/romInit.s 01e. (SPR #87000)
*/

/*
DESCRIPTION
This module contains the entry code for the VxWorks bootrom.
The entry point romInit, is the first code executed on power-up.
It sets the BOOT_COLD parameter to be passed to the generic
romStart() routine.

The routine sysToMonitor() jumps to the location 8 bytes
past the beginning of romInit, to perform a "warm boot".
This entry point allows a parameter on the stack to be passed
to romStart().

Hardware that requires special register setting or memory
mapping to be done immediately, may do so here.
*/

#define _ASMLANGUAGE
#include "vxWorks.h"
#include "sysLib.h"
#include "asm.h"
#include "config.h"

/* Lower 8 bits of on-chip register addresses. Upper bits will be set in GBR */
 
#define _BCR1_          0x00    /* 0xff800000: Bus Control Register 1         */
#define _BCR2_          0x04    /* 0xff800004: Bus Control Register 2         */
#define _BCR3_          0x50    /* 0xff800050: Bus Control Register 3         */
#define _WCR1_          0x08    /* 0xff800008: Wait Control Register 1        */
#define _WCR2_          0x0c    /* 0xff80000c: Wait Control Register 2        */
#define _WCR3_          0x10    /* 0xff800010: Wait Control Register 3        */
#define _MCR_           0x14    /* 0xff800014: Memory Control Register        */
#define _PCR_           0x18    /* 0xff800018: PCMCIA Control Register        */
#define _RTCSR_         0x1c    /* 0xff80001c: Refresh Timer Control/Status   */
#define _RTCNT_         0x20    /* 0xff800020: Refresh Timer CouNTer          */
#define _RTCOR_         0x24    /* 0xff800024: Refresh Time COnstant Register */
#define _RFCR_          0x28    /* 0xff800028: Refresh Count Register         */
#define _PCTRA_		0x2c    /* 0xff80002c: Port control Register A        */
#define _PDTRA_		0x30    /* 0xff800030: Port data Register A           */
#define _PCTRB_		0x40    /* 0xff800040: Port control Register B        */
#define _PDTRB_		0x44    /* 0xff800044: Port data Register B           */
#define _GPIOIC_	0x48	/* 0xff800048: GPIO Interrupt Control Reg     */
 
#define _FRQCR_         0x00    /* 0xffc00000: FReQuency Control Register     */
 
#define _MMUCR_         0x10    /* 0xff000010: MMU Control Register           */
#define _CCR_           0x1c    /* 0xff00001c: Cache Control Register         */
 
#define _RT_PASSWD_     0xa500
#define _RFCR_PASSWD_   0xa400

	/* internals */

	.global	_romInit	/* start of system code          */
	.global	_sdata		/* start of data                 */

	/* externals */

	.extern	_romStart	/* system initialization routine */

		.type	_sdata,@object
_sdata:		.asciz	"start of data"


/*******************************************************************************
*
* romInit - entry point for VxWorks in ROM
*

* romInit
*     (
*     int startType		/@ only used by 2nd entry point @/
*     )

*/
	.text
	.align	2
	.type	_romInit,@function

_romInit:


	mov.l	InitialSR,r0
	ldc	r0,sr		 /* disable interrupts */
	
	bra	cold;
	nop

_romInit2:			/* subsequent ROM starts (2nd entry point) */
	mov.l	InitialSR,r0
	ldc	r0,sr		/* disable interrupts */
	bra	warm;
	nop

	/* copyright notice appears at beginning of ROM (in TEXT segment) */

	.ascii	"Copyright 1984-2001 Wind River Systems, Inc."

	.align	2
InitialSR:	.long	0x400000f0	/* MD=1, RB=0, BL=0, FD=0, IM=0xf */


cold:
	mov.l	_XFF800000,r1
	ldc	r1,gbr			/* gbr: 0xff800000 */

	/* initialize BSC (Bus State Controller) for MS7751RSE */

	mov.l	_BCR1_DATA,r0
	mov.l	r0,@(_BCR1_,gbr)	/* BCR1 (Bus Control Register 1) */

	mov.w	_BCR2_DATA,r0
	mov.w	r0,@(_BCR2_,gbr)	/* BCR2 (Bus Control Register 2) */

	mov.w	_BCR3_DATA,r0
	mov.w	r0,@(_BCR3_,gbr)	/* BCR3 (Bus Control Register 3) */

	mov.l 	_BCR4_ADRS,r1
	mov.l	_BCR4_DATA,r0
	mov.l  	r0,@r1			/* BCR4 (Bus Control Register 4) */

	mov.l	_WCR1_DATA,r0
	mov.l	r0,@(_WCR1_,gbr)	/* WCR1 (Wait Control Register 1) */

	mov.l	_WCR2_DATA,r0
	mov.l	r0,@(_WCR2_,gbr)	/* WCR2 (Wait Control Register 2) */

	mov.l	_WCR3_DATA,r0
	mov.l	r0,@(_WCR3_,gbr)	/* WCR3 (Wait Control Register 3) */

	mov.w	_PCR_DATA,r0
	mov.w	r0,@(_PCR_,gbr)		/* PCR (PCMCIA Control Register) */

	mov.w	_RTCNT_DATA,r0
	mov.w	r0,@(_RTCNT_,gbr)	/* reset SDRAM refresh time counter */

	mov.w	_RTCOR_DATA,r0
	mov.w	r0,@(_RTCOR_,gbr)	/* set SDRAM refresh time constant */

	mov.w	_RFCR_DATA,r0
	mov.w	r0,@(_RFCR_,gbr)	/* reset SDRAM refresh count */

	mov.l	_MCR_DATA0,r0
	mov.l	r0,@(_MCR_,gbr)		/* MCR.MRSET */

	mov.l 	_SDMR3_ADRS,r1
	mov.l	_X00000000, r0
	mov.b  	r0,@r1			/* precharge SDRAM, any data is ok. */

	mov.w	_RTCSR_DATA0,r0
	mov.w	r0,@(_RTCSR_,gbr)	/* start SDRAM refresh */

	mov	#0, r0 			/* Repeat 128 */
repeat:
	add	#1, r0
	tst	#128,r0
	bt	repeat 

	mov.l	_MCR_DATA1,r0
	mov.l	r0,@(_MCR_,gbr)		/* MCR.MRSET */

	mov.l 	_SDMR3_ADRS,r1
	mov.l	_X00000000, r0
	mov.b  	r0,@r1			/* precharge SDRAM, any data is ok. */

	mov.w	_RTCSR_DATA1,r0
	mov.w	r0,@(_RTCSR_,gbr)	/* start SDRAM refresh */

        mov.l   _PCTRA_D, r0            /* PCTRA data    -> R0 */
        mov.l   r0, @(_PCTRA_,gbr)      /* PCTRA set */
 
        mov.w   _PCTRB_D, r0            /* PCTRB data    -> R0 */
        mov.w   r0, @(_PCTRB_,gbr)      /* PCTRB set */
 
        mov.w   _GPIOIC_D, r0           /* GPIOIC data   -> R0 */
        mov.w   r0, @(_GPIOIC_,gbr)     /* GPIOIC set */

	mov	#BOOT_COLD,r4		/* force startType as cold boot */
warm:					/* startType kept in r4         */
	mov.l	_XFFC00000,r1
	ldc	r1,gbr			/* gbr: 0xffc00000 */

	/* initialize FRQCR (FReQuency Control Register) */

	mov.w	_FRQCR_DATA,r0
	mov.w	r0,@(_FRQCR_,gbr)

	shll2	r1
	ldc	r1,gbr			/* gbr: 0xff000000 */

	/* initialize CCR (Cache Control Register) */

	mov.w	_X0808,r0		/* r0: 0x00000808 */
	mov.l	r0,@(_CCR_,gbr)		/* invalidate & disable cache */

	/* initialize MMU (Memory Management Unit) */

	mov	#0x00,r0		/* r0: 0x00000000 */
	mov.l	r0,@(_MMUCR_,gbr)	/* disable mmu */

        /* turn off SYSFAIL LED */

        mov.l   _LED_REG_ADRS,  r1
        mov.w   r0, @r1                 /* r0: 0x00000000 */

	/* initialize sp */

	mov.l	_STACKINIT_ADRS,sp	/* stack grows down from code start */

        /* calculate C entry point: routine - entry point + ROM base */

	mov.l	_ROMSTART_ADRS, r0
	mov.l	_ROMINIT_ADRS,  r1
	mov.l	_ROMTEXT_ADRS,  r2
	sub	r1,r0
	add	r2,r0
	jsr 	@r0			/* call romStart(startType) in ROM */
	nop


			.align	2
_STACKINIT_ADRS:	.long	STACK_ADRS	/* defined in configAll.h    */
_ROMSTART_ADRS:		.long 	_romStart	/* entry point of bootInit.c */
_ROMINIT_ADRS:		.long 	_romInit	/* entry point of this file  */
_ROMTEXT_ADRS:		.long 	ROM_TEXT_ADRS	/* defined in config.h       */
_BCR4_ADRS:		.long	0xFE0A00F0	/* BCR4 address              */
_SDMR3_ADRS:		.long 	0xFF940088	/* SDRAM(CS3) mode register  */
_XFF800000:		.long	0xff800000
_XFFC00000:		.long	0xffc00000
_LED_REG_ADRS:          .long   0xba000000 	/* LED register address */
_X00000000:		.long	0x00000000
_XLED:			.short  0xc1ba
_X0808:			.short	0x0808

		.align	2
_BCR1_DATA:	.long	0x00180008  /* set area 4 to Byte Control mode to mapping ks884x ISA interface */
_BCR2_DATA:	.short	0x2ffc
_BCR3_DATA:	.short	0x0001		/* area 3 SDRAM burst length */
		.align	2
_BCR4_DATA:	.long	0x00000000
_WCR1_DATA:	.long	0x02710771  /* set area4 to 1 idle cycle between read and write access (0x02770771) */
_WCR2_DATA:	.long	0x7ffe4fe7
_WCR3_DATA:	.long	0x01777771
_MCR_DATA0:	.long	0x080901b4
_MCR_DATA1:	.long	0x480901b4
_PCR_DATA:	.short	0x0000
_RTCSR_DATA0:	.short	0xa508
_RTCSR_DATA1:	.short	0xa518
_RTCNT_DATA:	.short 	0xa500		/* RTCNT clear data */
_RTCOR_DATA:	.short	0xa50d		/* Refresh cycle(867nS/13.87uS) */
_RFCR_DATA:	.short	0xa400		/* RFCR clear data */
		.align	2
_PCTRA_D:       .long   0x00000000      /* bit31 : PB15PUP 0 PORT15 pull-up */
_PCTRB_D:       .long   0x00000000      /* bit31 : PB15PUP 0 PORT31 pull-up */
_GPIOIC_D:      .short  0x0000          /* bit15 : PTIREN15  0 PORT15 I/O port */
_FRQCR_DATA:	.short	0x0e1a          /* CPU:Bus:Peripheral=240Mhz:60Mhz:60Mhz */

