'\" t
.so wrs.an
.\" ms7751Rse/target.nr - Hitachi MS7751RSE01 target specific documentation
.\"
.\" Copyright 2003      Wind River Systems, Inc.
.\"
.\" modification history
.\" --------------------
.\" 01c,22jul04,pcd  added supported ks864x End driver.
.\" 01c,13oct03,h_k  fixed supported End driver name.
.\" 01b,19jun03,h_k  added known problem for preliminary revision.
.\" 01a,16jun03,h_k  written based on ms7751se/target.nr 01h. (SPR #87000)
.\"
.\"
.TH ms7751Rse T "Hitachi MS7751RSE01" "Rev: 16 Jun 03" "TORNADO REFERENCE: VXWORKS"

.SH "NAME"
.aX "Hitachi MS7751RSE01"

.SH "INTRODUCTION"
This reference entry provides board-specific information necessary to run
VxWorks for the ms7751Rse BSP. Before using a board with VxWorks,
verify that the board runs in the factory configuration by using
vendor-supplied ROMs and jumper settings and checking the RS-232
connection. Also make sure to use the big endian version of the ROMs.
Hitchi usually provides both the little endian and the big endian EPROMs.

.SS "Boot ROMs"
The VxWorks boot ROM for the Hitachi MS7751RSE01 targets is programmed in
the on-board flash memory (a pair of MBM29LV160T chips). The socketed
EPROMS supplied by the manufacturer contain the ROM monitor program. This
program can be used to program the flash as described below in the "ROM
considerations" section.

This BSP provides NVRAM functionality by reserving the last pair of sectors
of the on-chip flash. To use this feature, make sure INCLUDE_NVRAM_TO_FLASH
is defined.

To load VxWorks, and for more information, follow the instructions in the
.I "Tornado User's Guide: Getting Started."

.SS "Jumpers"
The following jumpers and switches are relevant to VxWorks configuration:

.TS E
expand;
cf3 s s
lf3 lf3 lf3
l l lw(3i) .
.ne 5
Vendor and Model
.sp .25
Jumper	Function	Description
_
SW4-6	Endian Select	T{
Sets the byte order to be used
T}
SW5-1,2	SCIF Baud Rate	T{
Sets the SCIF baud rate used by the ROM monitor
T}
SW5-3,4	Boot Select	T{
Selects the boot device
T}
.TE
.PP
For details of jumper configuration, see the board diagram at the end
of this entry.  Also see the board's hardware manual for other jumpers
not directly related to VxWorks operation.

.SH "FEATURES"
This section describes all features of the board, supported or not.
It documents all configurations of the board and the interaction between
features and configuration items.

.SS "Supported Features"
The following features of the MS7751RSE are supported:

.TS E
expand;
lf3 lf3
lw13 lw(3.7i) .
.ne 6
.sp .5
Feature	Description
_
Processors	T{
SH7751R; 240MHz internal clock, 60MHz bus clock, 60MHz peripheral clock.
T}
FLASH	T{
On board flash used for VxWorks boot image, NVRAM emulation and TrueFFS.
T}
DRAM	T{
up to 64MBytes, no auto-sizing is supported
T}
Peripherals	T{
serial port SCIF;
serial port SCI (not tested);
two sync/async serial ports of M1543C SuperIO (one tested);
10/100baseT Ethernet interface;
Primary and secondary ATA/EIDE ports;
M1543C PCI-ISA Bus Controller (PIBC)
M1543C integrated programmable interrupt controller (PIC)
M1543C integrated mouse controller (requires WindML)
M1543C integrated keyboard controller (requires WindML)
USB port (requires USB Developers Kit)
T}
Miscellaneous	T{
RESET switch (hard reset);
ABORT switch (causes soft reboot);
T}
.TE

.SS "Unsupported Features"
The following board features are not supported:

.TS E
expand;
lf3 lf3
lw13 lw(3.7i) .
.ne 6
.sp .5
Feature	Description
_
Peripherals	T{
PCMCIA port;
Parallel port;
Floppy disk port;
Real Time Clock
T}
.TE

.SS "Feature Interactions"
Not Applicable

.SH "HARDWARE DETAILS"
This section documents the details of the device drivers and board
hardware elements.

.SS "Devices"
The chip drivers included are:

    shScifSio - SH7751R On-chip tty driver
    ns16552Sio - M1543C UltraIO tty driver
    sh7700Timer - SH77xx On-chip Timer library
    nvRamToFlash - MS7751R On-board flash ROM library
    aliM1543CPibc.c - M1543C PIBC/PIC/IDE/USB/PMU/SuperIO configuration library
    ln97xEnd - AMD Am79C97X PCnet-PCI END driver
    ataDrv - IDE/ATA hard disk controller
    sysTffs - includes MTD routines for TrueFFS for MBM29LV160T

.SS "Memory Maps"
On-board RAM for the MS7751RSE boards always appears at address 0xC000000
locally. When the flash memory selected as the boot device (SW5-3,4), it 
appears at local address 0. For more information about the VxWorks memory
map used on the SuperH architecture, see the "Architecture Supplement."
For more information about the MS7751RSE specific address map see the
"SH7751R SolutionEngine (MS7751RSE01E) User's Manual".

.SS "Interrupts"
The SH7751R CPU has an on-chip interrupt controller (INTC). The INTC can accept
15 levels of external interrupts through 4 input lines (IRL0-IRL3), and each
interrupt is identified by INTEVT code. Also, the INTC accepts non-maskable
interrupt (NMI) through a dedicated input line. The NMI input of SH7751R CPU 
is connected to the ABORT switch. 

In addition, the M1543C has an integrated interrupt controller consisting
of two 8259 compatible PICs. All PCI interrupts as well as the M1543C on-chip
device interrupts are handled through the PICs. The PICs are connected
to IRL12 of the SH7751R. The PIC has it's own interrupt vector table, but
the exception vector table also has 16 corresponding entries reserved starting
at the value defined with PIC_INUM_BASE.

Exception Vector Table

.TS
center tab (|);
lf3 lf3 lf3 lf3
r l l l .
.ne 6
.sp .5
Event|      |Vector
Code |Source|Address
_
0x000|TRAPA #0 (Software Breakpoint)|VBR + 0x800
0x020|TRAPA #1 (Integer Zero Divide)|VBR + 0x804
0x040|TLB Miss/Invalid (Load)|VBR + 0x808
0x060|TLB Miss/Invalid (Store)|VBR + 0x80c
0x080|Initial Page Write|VBR + 0x810
0x0a0|TLB Protection Violation (Load)|VBR + 0x814
0x0c0|TLB Protection Violation (Store)|VBR + 0x818
0x0e0|Address Error (Load)|VBR + 0x81c
0x100|Address Error (Store)|VBR + 0x820
0x120|FPU exception|VBR + 0x824
0x140|TLB Multiple Hit|VBR + 0x828
0x160|TrapA Instruction|VBR + 0x82c
0x180|Illegal General Instruction|VBR + 0x830
0x1a0|Illegal Slot Instruction|VBR + 0x834
0x1c0|NMI (VME Bus Timeout Error)|VBR + 0x838
0x1e0|UBC (Hardware Breakpoint)|VBR + 0x83c
0x200|IRL15 (Expansion slot IRQ8)|VBR + 0x840
0x220|IRL14 (Expansion slot IRQ7)|VBR + 0x844
0x240|IRL13 (MR-SHPC IRQ2)|VBR + 0x848
0x260|IRL12 (M1543C)|VBR + 0x84c
0x280|IRL11 (Expansion slot IRQ6)|VBR + 0x850
0x2a0|IRL10 (Expansion slot IRQ5)|VBR + 0x854
0x2c0|IRL9 (MR-SHPC IRQ1)|VBR + 0x858
0x2e0|IRL8 (Not Used)|VBR + 0x85c
0x300|IRL7 (Expansion slot IRQ4)|VBR + 0x860
0x320|IRL6 (Expansion slot IRQ3)|VBR + 0x864
0x340|IRL5 (Not Used)|VBR + 0x868
0x360|IRL4 (Expansion slot IRQ2)|VBR + 0x86c
0x380|IRL3 (Not Used)|VBR + 0x870
0x3a0|IRL2 (Expansion slot IRQ1)|VBR + 0x874
0x3c0|IRL1 (MR-SHPC IRQ0)|VBR + 0x878
0x3e0|(Reserved)|VBR + 0x87c
0x400|TMU0 underflow|VBR + 0x880
0x420|TMU1 underflow|VBR + 0x884
0x440|TMU2 underflow|VBR + 0x888
0x460|TMU2 input capture|VBR + 0x88c
0x480|RTC alarm|VBR + 0x890
0x4a0|RTC periodic|VBR + 0x894
0x4c0|RTC carry|VBR + 0x898
0x4e0|SCI receive error|VBR + 0x89c
0x500|SCI receive interrupt|VBR + 0x8a0
0x520|SCI transmit interrupt|VBR + 0x8a4
0x540|SCI transmit end|VBR + 0x8a8
0x560|WDT interval timer|VBR + 0x8ac
0x580|BSC refresh CMI|VBR + 0x8b0
0x5a0|BSC refresh OVF|VBR + 0x8b4
0x5c0|(Not Used)|VBR + 0x8b8
0x5e0|(Not Used)|VBR + 0x8bc
0x600|JTAG|VBR + 0x8c0
0x620|(Reserved)|VBR + 0x8c4
0x640|DMAC DMTE0|VBR + 0x8c8
0x660|DMAC DMTE1|VBR + 0x8cc
0x680|DMAC DMTE2|VBR + 0x8d0
0x6a0|DMAC DMTE3|VBR + 0x8d4
0x6c0|DMAC DMTER|VBR + 0x8d8
0x6e0|(Not Used)|VBR + 0x8dc
0x700|SCIF receive error|VBR + 0x8e0
0x720|SCIF receive interrupt|VBR + 0x8e4
0x740|SCIF break interrupt|VBR + 0x8e8
0x760|SCIF transmit interrupt|VBR + 0x8ec
0x780-0x7e0|(Not Used)|
0x800|FPU Disable|VBR + 0x900
0x820|Slot FPU disable|VBR + 0x904
0x840-0x9e0|(Reserved)|
0xa00|PCI SERR|VBR + 0x940
0xa20|PCIC DMA3|VBR + 0x944
0xa40|PCIC DMA2|VBR + 0x948
0xa60|PCIC DMA1|VBR + 0x94c
0xa80|PCIC DMA0|VBR + 0x950
0xaa0|PCIC Pwr. ON|VBR + 0x954
0xac0|PCIC Pwr. down|VBR + 0x958
0xae0|PCI Error|VBR + 0x95c
0xb00|TMU3 underflow|VBR + 0x960
0xb20-0xb60|(Not used)|
0xb80|TMU4 underflow|VBR + 0x970
0xba0-0xde0|(Not used)|
n/a|M1543C PIC sources|VBR + (0x9c0 to 0x9fc)
0x1000-0x1fe0|TRAPA #IMM (Not Used)|VBR + 0x800 + IMM*4
.TE
.sp 2

.SS "Flash Configuration"
The flash chips (2 x MBM29LV160T) on board can be used for three 
different functions: boot ROM storage, non-volatile storage (boot 
parameters and MAC address), and for TruesFFS. The config.h file 
contains cofiguration macros to set the size of reserved for these
functions. Make sure to use the provided sysTffsFormat() routine 
instead of tffsDevFormat() in order to reserve the space needed for
the boot ROM image.

The multiple use of the flash chips can cause problems when using
ROM-resident images (e.g. bootrom_res, vxVorks.res_rom). While
erasing or programming, the flash enters a special state. In this 
state data cannot be read from it and code stored in it cannot be
executed. For this reason ROM-resident images crash when a write 
to the boot parameters or TrueFFS is attempted.

.SS "IDE Configuration"
The IDE controllers on the M1543C chip can be used with ataDrv. 
To include the ATA controller driver, define INCLUDE_ATA in config.h.
In order to use a file system you also have to define it's 
corresponding macro (e.g. INCLUDE_DOSFS).

.SS "Serial Configuration"
No special configuration needed.

.SS "Network Configuration"
The Ethernet (END) driver is called "lnPci" and should be specified as the 
boot device to the boot ROM. Before the first use, use the bootrom "N"
command to set a unique Ethernet address, stored in the Serial EEPROM
controlled by the AM79C973 (only if the wendor has not programmed it 
already). After the N command make sure to push the reset button.

.SS "PCI Bus Configuration"
The SH7751R on-chip PCI controller is used on this board for PCI bus
control. For a description of the PCI memory map and I/O map please see
the ms7751se.h file.

PCI interrupts are mapped through the interrupt controller of the M1543C
PCI/ISA bridge. For details please see the intLine table in the sysBusPci.c
file.

The SH7751R on-chip PCI controller is limited to a 16 MB window for
non-prefetched memory accesses, and a 256 KB window for PCI I/O accesses.
Depending on your application, this may not be enough. This BSP can be 
configured to include code that will use the MMU mis-hit exceptions to 
move the PCI window when accessing addresses outside of the currently 
active PCI window. This is called a virtual PCI meomory and I/O space. It 
is possible to enable the virtual PCI extension for non-prefetch memory
space only (INCLUDE_SH7751_VIRTUAL_PCI_MEMIO), or for both non-prefetch
memory space and I/O space (INCLUDE_SH7751_VIRTUAL_PCI_IO). 

In order to use the virtual PCI memory and I/O features, either the bundled
basic MMU support or the optional full MMU support (requires VxVMI) must
be included. As bootrom images never include MMU support, bootroms must 
ensure that the 16 MB PCI memory and 256 KB PCI I/O space is sufficient.

.SS "WindML Configuration"
This BSP may use the optional multimedia library, WindML.  WindML devices 
for the MS7751RSE are defined and enabled using the WindML device section 
within the project facility.  The graphics devices suported for this BSP
are the generic PCI graphics cards supported in WindML. Note, for graphics
devices that require 16MByte or more PCI memory make sure to enable virtual 
PCI extension.

For command line builds, define the selected devices in the config.h file:

.CS
    #define INCLUDE_WINDML
    #define INCLUDE_PCI_WINDML_GRAPHICS
    #define INCLUDE_WINDML_PS2_POINTER
    #define INCLUDE_WINDML_PS2_KEYBOARD
.CE


.SS "Boot Devices"
The supported boot device is:

    \f3lnPci\f1 - Ethernet (10/100baseT)

.SS "Boot Methods"
The boot methods are affected by the boot parameters.  If no password is
specified, RSH (remote shell) protocol is used.  If a password is specified,
FTP protocol is used, or, if the flag is set, TFTP protocol is used.

.SS "ROM Considerations"
This section describes the procedure for programming the VxWorks boot image
into the on-board flash using the Hitachi ROM monitor. Some older boards
were shipped with ROM monitors that did not implement the "fl" command
used to program the flash. You need to contact Hitachi, Ltd. and obtain
a new set of EPROMs containing updated ROM monitor.

.IP "1)" 
Make sure to install the correct ROM monitor depending on what byte order 
will be used (big endian). Set the SW4.6 to ON.

.IP "2)" 
Set the baud rate using the SW5.1 and SW5.2 as highest possible supported 
by the host terminal program. For the Windows HyperTerminal set to 115200 
(SW5.1 and SW5.2 both ON). Configure the host terminal program to use the 
same baud rate. Connect the serial cable to the MS7751RSE01 CN2 connector.

.IP "3)" 
Set the boot device to the socketed EPROM (SW5.3 and SW5.4 both ON)

.IP "4)" 
Power on the target. The host teminal program should display:

.CS
   =============================================================
               SH7751R Self Debugger Ver 0.0
   -------------------------------------------------------------
     (C) Copyright 1995-2000. Hitachi.Ltd. All rights reserved.
   =============================================================
                                     H[elp] for help messages...
   Ready >
.CE

.IP "5)" 
Issue the command to program the flash:

.CS
   Ready >fl 0
   Flash ROM data copy to RAM
   Please Send A S-format Record
.CE

.IP "6)" 
Make sure that the <bootimage>.hex file is in PC text format (with CR/LF). 
Send it using the terminal program as a text file. In the Windows HyperTerminal
select the `Transfer' menu, then `Send text file...'. Downloading the image can
take several minutes depending on the file size and baud rate. Unfortunately
there is no progress indication. Once the download is complete, the ROM monitor
automatically erases and reprograms the flash. 

.CS
   End Addrs = 000542B4

   Transfer complete
   Flash chip erase: complete
   Program :complete
   Flash write complete
   Ready >
.CE

.IP "7)" 
Turn power OFF, change the SW5.3 and SW5.4 setting (OFF and ON respectively) 
to select the flash as th boot device. The baud rate setting does not aply to
VxWorks, therefore the status of SW5.1 and SW5.2 does not have to be changed. 
Set the host communication software baud rate to 9600 (the default VxWorks 
console baud rate, which can be changed in config.h) and power on the target. 
The console shoud display the vxWorks boot screen.


.SH "SPECIAL CONSIDERATIONS"
This section describes miscellaneous information that the user needs
to know about the BSP.

.SS "LITTLE ENDIAN SUPPORT"
This board can operate using either big endian or little endian byte
order. The BSP is shipped with binaries built for big endian.
For information about 
building the BSP please see the SuperH architecture supplement.


.SS "Delivered Objects"
The delivered objects are: `bootrom_uncmp', `vxWorks', `vxWorks.sym', 
and `vxWorks.st'.

.SS "Make Targets"
The make targets are listed as the names of object-format files.  Append `.hex'
to each to derive a hex-format file name. In order to program the flash using
the Hitachi ROM monitor, the hex files must be saved in PC text format (CR/LF).

.SS "Running the BSP validation tests"
The Solution Engine boards do not have bus error detection hardware, therefore
tests that rely on bus errors will fail. On processors with MMU, it is possible
to pass tests by enabling MMU support and modifying the test configuration 
files to use MMU exceptions ("TLB Miss") instead of bus errors. Bootroms do 
not include MMU support, therefore the bootline tests still fail.

.SS "Special Routines"
Not Aplicable

.SH "KNOWN PROBLEMS"

.SS "PCI Initialization Failure"
Setting J3 to [1-2] closed and power up the board without hitting SW1, POWER
switch, on the board will cause PCI initialization error. 
You have to keep J3 [2-3] closed, that is the default setting, and use the
POWER switch. If you need remote control the power on/off for the board, you
can change the J3 to [1-2] closed but it's still required to hit the SW1 button
in order to send the power on signal to PCI-ISA controller, then reset the
board (hit SW2 button) and boot up the ROM image.

.SS "MS7751RSE01 Preliminary Revison"
The preliminary revison of the MS7751RSE01 board has 162MHz CPU that is not
supported by this BSP as default. In that case you can take one of the two
solutions described below..
a. Modify romInit.s and change _FRQCR_DATA to 0x0e13, modify config.h and
   change SYS_CPU_FREQ/SYS_BUS_FREQ/SYS_PCLK_FREQ to
   162000000/54000000/27000000 to set up for 162MHz CPU, 54MHz bus, 27MHz
   peripheral clocks.
b. Use the ms7751se BSP. The preliminary MS7751RSE01 is upper compatible with
   the MS7751SE01. If 2-way associative cache feature is required in your
   system, add CACHE_2WAY_MODE to the USER_D_CACHE_MODE defined in config.h.

.SH "BOARD LAYOUT"
For the board layout and jumper locations please consult the "SH7751R 
SolutionEngine (MS7751RSE01E) User's Manual"

.SH "SEE ALSO"
.tG "Getting Started,"
.pG "Configuration," 
.pG "SuperH Architecture Supplement (Appendix)"

.SH "BIBLIOGRAPHY"
.iB "SH7751R SolutionEngine (MS7751RSE01E) User's Manual,"
.iB "SuperH (SH) 32-Bit RISC MCU/MPU Series SH7751R High-Performance RISC Engine Hardware Manual,"
.iB "M1543C B1 South Bridge Socket 7, 370/Slot1, A Chip Set with Super I/O & FIR,"
.iB "Am79C973/Am79C975 PCnet- FAST III Single-Chip 10/100 Mbps PCI Ethernet Controllerwith Integrated PHY"
