/* usrAppInit.c - stub application initialization routine */

/* Copyright 1984-1998 Wind River Systems, Inc. */

/*
modification history
--------------------
01a,02jun98,ms   written
*/

#include "end.h"
#include "endLib.h"
#include "muxLib.h"
#include "muxTkLib.h"
#include "configNet.h"
#include "m2IfLib.h"
#include "config.h"


/*
DESCRIPTION
Initialize user application code.
*/ 

#if defined (INCLUDE_KS884X_END) || defined (INCLUDE_KS884X_SHBUS_END)

#include "bootLib.h"	/* for boot parameters */
#include "ks884xEnd.h"
#include "ks884xEnd_shBus.h"

IMPORT END_TBL_ENTRY ks_endDevTbl[];
IMPORT BOOT_PARAMS	 sysBootParams;


/******************************************************************************
*
* ksNetIfConfig - configure a network interface
*	- subnetmask is extracted from inetAdrs and, if present,
*	  set for interface
*	- inet address is set for interface
*	- if present, inet name for interface is added to host table
*/
LOCAL STATUS ksNetIfConfig 
    (
    char *	devName,		/* device name */
    int     unitNum,        /* unit number */
    char *	inetAdrs,		/* inet address */
    char *	inetName,		/* host name */
    int 	netmask			/* subnet mask */
    )
{
    char 	ifname [20];
    char    addrStr[20];
    UINT32  addr;



    /* check for empty inet address */
    if (inetAdrs[0] == EOS)
	{
		printf ("No inet address specified for interface %s.\n", devName);
		return (ERROR);
	}

    /* build interface name */
    sprintf(ifname, "%s%d", devName, unitNum);

    /* set subnet mask, if any specified */
    if (netmask != 0)
		ifMaskSet(ifname, netmask);

    /* set inet address */
    if (ifAddrSet(ifname, inetAdrs) != OK)
    {
        printf("Error setting inet address of %s to %s, errno = %#x\n",
                ifname, inetAdrs, errno);
        return (ERROR);
    }

    /* add host name to host table */
    if ((inetName != NULL) && (*inetName != EOS))
    {
        /* delete the host name from host table if it exist */
        if ( (addr = hostGetByName (inetName)) != ERROR )
        {
             memset (addrStr, 0, sizeof(addrStr) );
             sprintf(addrStr, "%d.%d.%d.%d", 
                              ((addr >> 24) & 0x000000ff), 
                              ((addr >> 16) & 0x000000ff), 
                              ((addr >> 8 ) & 0x000000ff), 
                              (addr & 0x000000ff) 
                     );
		     hostDelete (inetName, addrStr);
        }

		hostAdd (inetName, inetAdrs);
    }

    return (OK);
}


/*
 * setDefaultRoute
 *	This function is used to set default routing path for the END
 *
 * Argument(s)
 *	add		flag indicates add or delete the route
 *
 * Return(s)
 *	OK		if successful
 *  ERROR	otherwise
 */
LOCAL STATUS setDefaultRoute(int add)
{


	/* if gateway is set, use it */
	if (sysBootParams.gad && EOS != sysBootParams.gad[0]) 
    {
		if (add)
			routeAdd("0.0.0.0", sysBootParams.gad);
		else
			routeDelete("0.0.0.0", sysBootParams.gad);
	}
	/* if host is set, use it */
	else if (sysBootParams.had && EOS != sysBootParams.had[0]) 
    {
		if (add) 
        {
			/* if use LAN IF for downloading vxWorks image */
			if (sysBootParams.unitNum != 0) 
				printf("Warning: ip address is used as default routing gateway\n", __FUNCTION__);
        
			routeAdd("0.0.0.0", sysBootParams.had);
		}
		else
			routeDelete("0.0.0.0", sysBootParams.had);
	}
	/* if none of above is set, use its owe ethernet */
	else if (sysBootParams.had && EOS != sysBootParams.ead[0]) 
    {
		char buf[16], *p;

		/* search for netmask deliminate */
		p = strstr(sysBootParams.ead, ":");

		if (p) 
			memcpy(buf, sysBootParams.ead, (int)(p - sysBootParams.ead));
		else 
			strcpy(buf, sysBootParams.ead);

		if (add) 
        {
			/* if use LAN IF for downloading vxWorks image */
			if (sysBootParams.unitNum != 0) 
				printf("Warning: ip address is used as default routing gateway\n", __FUNCTION__);
			
			routeAdd("0.0.0.0", buf);
		}
		else
			routeDelete("0.0.0.0", buf);
	}
	else 
    {
		printf("Invalid sysBootParams\n");
		return ERROR;
	}

	return OK;
}

/*
 * ksSetEndIf
 *	This function is used to set END interface for KS884x
 *
 * Argument(s) 
 *  netIf   network unit number (0: ks0, 1: ks1)
 *	ipAddr	pointer to ipaddress to set.
 *	mask	netmask
 *
 * Return(s)
 *	OK		if successful
 *  ERROR	otherwise
 */
STATUS ksSetEndIf
(
    char * ipAddr
)
{
    char  hostName[32];
	/*char *dev="ks";*/	/* set default boot device first */
    int   maxIf=1;
    int   netIf=0; 
    int   mask=0xffffff00;



    if (netIf >= maxIf)
    {
		printf("Exceed network interface that ks884x can support for device %s%d\n", KS_DEV_NAME, netIf);
        return (ERROR);
    } 

    memset(hostName, 0, sizeof(hostName));

    /* detach routine from the TCP/IP network stack in case it already attached */
    ipDetach(netIf, KS_DEV_NAME);

    /* attach routine for the TCP/IP network stack */
    if (ipAttach(netIf, KS_DEV_NAME) != OK)
	{
		printf("Failed to attach TCP/IP to device %s%d", KS_DEV_NAME, netIf);
		return ERROR;
	}
	taskDelay(10);

    /* add gatways to the network routing table */ 
	setDefaultRoute(TRUE);

    /* set an interface address for a network interface, and a host to the host table */
    sprintf(hostName, "ks884%d", (netIf+1) );

	return ksNetIfConfig(KS_DEV_NAME, netIf, ipAddr, hostName, mask);
}
#endif /* INCLUDE_KS884X_END */



/******************************************************************************
*
* usrAppInit - initialize the users application
*/ 

void usrAppInit (void)
{
#ifdef	USER_APPL_INIT
	USER_APPL_INIT;		/* for backwards compatibility */
#endif

    /* add application specific code here */

#ifdef INCLUDE_END
    int            count;
    char           muxDevName[8];
    END_TBL_ENTRY* pDevTbl;
    void         * pCookie = NULL;



#if (1)
#if defined (INCLUDE_KS884X_END) || defined (INCLUDE_KS884X_SHBUS_END)

    /* Add in mux ks884x ENDs. */
    for (count = 0, pDevTbl = ks_endDevTbl; pDevTbl->endLoadFunc != END_TBL_END;
         pDevTbl++, count++)
    {

        /* Make sure that WDB has not already installed the device. */
        if (!pDevTbl->processed)
        {
            /* printf ("usrAppInit() is calling muxDevLoad. \n"); */
            pCookie = muxDevLoad (pDevTbl->unit,
                                  pDevTbl->endLoadFunc,
                                  pDevTbl->endLoadString,
                                  pDevTbl->endLoan, pDevTbl->pBSP);
            if (pCookie == NULL)
            {
                printf("muxDevLoad failed for device entry %d!\n", count);
            }
            else
            {
                pDevTbl->processed = TRUE;
                /* printf ("usrAppInit() is calling muxDevStart. \n"); */
                if (muxDevStart(pCookie) == ERROR)
                {
                    printf("muxDevStart failed for device entry %d!\n", count);
                }
#ifdef END_POLL_STATS_ROUTINE_HOOK
		        endPollStatsInit (pCookie, END_POLL_STATS_ROUTINE_HOOK);
#endif /* END_POLL_STATS_ROUTINE_HOOK */
            }
        }
    }
#endif /* INCLUDE_KS884X_END */

#endif /* #if (0) */

    if ( strcmp(sysBootParams.ead, "192.168.1.1") !=0 )
        ksSetEndIf ( "192.168.1.1" ); 
    else
        ksSetEndIf ( "192.168.0.1" ); 

    /* Display Device Driver info on UART */
    showKsBspInfo (); 

    /*	Display KS884x Link Status */
    showKsLinkStatus ();

#endif /* INCLUDE_END */


#ifdef	INCLUDE_DOSFS
    hashLibInit ();			/* initialize hash table package */
    dosFsInit (NUM_DOSFS_FILES); 	/* init dosFs filesystem */
#endif	/* INCLUDE_DOSFS */

#ifdef	INCLUDE_RAWFS
    rawFsInit (NUM_RAWFS_FILES); 	/* init rawFs filesystem */
#endif	/* INCLUDE_RAWFS */

#ifdef	INCLUDE_FLASH
    /* Initializes flash files system for /rfa0, /rfa1, /rfa2  */
    flashFsLibInit(); 
#endif /* INCLUDE_FLASH */


}


