/* ---------------------------------------------------------------------------
          Copyright (c) 2004-2008 Micrel, Inc.  All rights reserved.
   ---------------------------------------------------------------------------

    device.c - Linux network device driver.

    Author      Date        Description
    THa         01/05/04    Created file.
    THa         10/14/04    Updated with latest specs.
    THa         05/10/05    Updated for two port devices.
    THa         06/27/05    Updated for version 0.1.5.
    THa         06/29/05    Updated for Linux 2.6.
    THa         07/07/05    Changed /proc entries.
    THa         10/18/05    Provide proc entries to change duplex and speed.
    THa         01/13/06    Test SH generic bus transfer rate.
    THa         03/14/06    Release 1.0.0.
    THa         04/03/06    Use TWO_NETWORK_INTERFACE instead of TWO_PORTS.
    THa         04/06/06    Implement MII IO calls.
    THa         05/31/06    Add ethtool support.
    THa         06/05/06    Make sure interrupts are enabled before processing
                            them.
    THa         06/29/06    Change statistics reporting to provide meaningful
                            data.
    THa         07/14/06    Fix two network interfaces driver issues.
    THa         07/26/06    Add preliminary DMA support.
    THa         10/03/06    Use tasklet to receive.
    THa         10/16/06    Add ST7100 DIM_0_x_SG FDMA support.
    THa         10/24/06    Add KS884X Load Board support.
    THa         12/05/06    Version 1.0.8 adds revision A6 turbo mode support.
    THa         12/05/07    Update for 64-bit Linux kernels.
    THa         02/15/08    Add force link support.
    THa         03/28/08    Update for Linux 2.6.24.
    THa         07/02/08    Fix continual backpressure problem in KSZ8841P.
   ---------------------------------------------------------------------------
*/


#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/ioport.h>
#include <linux/proc_fs.h>
#include <asm/uaccess.h>

MODULE_LICENSE("GPL");
MODULE_AUTHOR("Micrel, Inc. <www.micrel.com>");
MODULE_DESCRIPTION("Micrel KSZ884XM Ethernet driver");

#include "target.h"
#include "hardware.h"
#include "device.h"

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2, 4, 21))
#include <linux/ethtool.h>
#endif
#include <asm/irq.h>

#ifdef CONFIG_SH_7751_SOLUTION_ENGINE
#include <asm/pci-sh7751.h>
#endif

#ifdef SH_BUS_DMA
#include <asm/dma.h>
#include <linux/irq.h>
#include <linux/delay.h>

#ifdef CONFIG_STM_DMA
#include <linux/bigphysarea.h>

#if 1
#define STM_DMA_TEST
#define DBG
#endif
#endif
#endif
#if 1
#include "ks_ioctl.h"


int hwread
(
   unsigned char   BankNum,
   unsigned long   RegAddr,
   unsigned char   Width
);
#endif


#if 0
#define STP_SUPPORT
#endif


#ifdef SH_BUS
#ifdef DBG
#if 1
#define SH_BUS_TEST
#endif
#endif

#if defined( CONFIG_SH_STB7100_REF ) || defined( CONFIG_SH_STB7100_MBOARD )
#define BASE_IO_ADDR  0xA2000300

#elif defined( CONFIG_SH_7751_SOLUTION_ENGINE )
#define BASE_IO_ADDR  0xB0000000

#elif defined( CONFIG_KS884X_LOAD_BOARD )
#define BASE_IO_ADDR  0x03000000

#elif defined( CONFIG_MACH_SMDK2410KBUS )
#define BASE_IO_ADDR  VA_KSZ8842_BASE

#else
#define BASE_IO_ADDR  0
#endif

#else
#define BASE_IO_ADDR  0
#endif

#define BASE_IRQ  -1

#if 1
#define IOREMAP_RANGE  0x10000
#endif
#if defined( CONFIG_SH_7751_SOLUTION_ENGINE )
#undef IOREMAP_RANGE
#endif

#ifdef STP_SUPPORT
#ifdef KBUILD_BASENAME
#include <../net/bridge/br_private.h>

#else
/* From Linux 2.4 */

/* net/bridge/br_private_timer.h */
struct br_timer
{
        int running;
        unsigned long expires;
};

/* net/bridge/br_private.h */
typedef struct bridge_id bridge_id;
typedef struct mac_addr mac_addr;
typedef __u16 port_id;

struct bridge_id
{
        unsigned char   prio[2];
        unsigned char   addr[6];
};

struct mac_addr
{
        unsigned char   addr[6];
        unsigned char   pad[2];
};

struct net_bridge;

struct net_bridge_port
{
        struct net_bridge_port  *next;
        struct net_bridge       *br;
        struct net_device       *dev;
        int                     port_no;

        /* STP */
        port_id                 port_id;
        int                     state;
        int                     path_cost;
        bridge_id               designated_root;
        int                     designated_cost;
        bridge_id               designated_bridge;
        port_id                 designated_port;
        unsigned                topology_change_ack:1;
        unsigned                config_pending:1;
        int                     priority;

        struct br_timer         forward_delay_timer;
        struct br_timer         hold_timer;
        struct br_timer         message_age_timer;
};
#endif
#endif


#ifdef DBG
#define NET_MSG_ENABLE  ( NETIF_MSG_DRV | NETIF_MSG_PROBE | NETIF_MSG_LINK )
#else
#define NET_MSG_ENABLE  0
#endif


#if defined( DEF_KS8842 )
#define DRV_NAME     "KSZ8842"
#else
#define DRV_NAME     "KSZ8841"
#endif
#ifdef SH_BUS
#ifdef SH_32BIT_ACCESS_ONLY
#define BUS_NAME     "32-bit only"
#elif defined( SH_BUS_DMA )
#define BUS_NAME     "DMA"
#elif defined( SH_16BIT )
#define BUS_NAME     "16-bit"
#else
#define BUS_NAME     "32-bit"
#endif
#else
#define BUS_NAME     "ISA"
#endif
#define DRV_VERSION  "1.0.12"
#define DRV_RELDATE  "Jul 2, 2008"

static char version[] __devinitdata =
    "Micrel " DRV_NAME " " BUS_NAME " " DRV_VERSION " (" DRV_RELDATE ")";

PHARDWARE phw;

void RxProcessTask ( unsigned long data );

#ifdef CONFIG_SH_7751_SOLUTION_ENGINE
static int led_count = 0;
#endif


#ifdef DBG
int ks_rx_overrun = 0;

#ifdef DEBUG_COUNTER
extern int rx_cnt1, rx_cnt2, rx_cnt3;
static unsigned long query_jiffies = 0;
#endif

/* 2 lines buffer. */
#define DEBUG_MSG_BUF   ( 80 * 2 )

#define PRINT_MSG_SIZE  ( 80 * 20 )
#define PRINT_INT_SIZE  ( 80 * 10 )

#define DEBUG_MSG_SIZE  ( PRINT_MSG_SIZE + PRINT_INT_SIZE + \
    DEBUG_MSG_BUF * 2 )


static char* strDebug = NULL;
static char* strIntBuf = NULL;
static char* strMsg = NULL;
static char* strIntMsg = NULL;
static int   cnDebug = 0;
static int   cnIntBuf = 0;
static int   fLastDebugLine = 1;
static int   fLastIntBufLine = 1;

static unsigned long lockDebug = 0;
#endif


#define DBG_CH  '-'


void DbgMsg (
    char *fmt, ... )
{
#ifdef DBG
    va_list args;
    char**  pstrMsg;
    int*    pcnDebug;
    int     nLeft;
    int     in_intr = in_interrupt();

    pcnDebug = &cnDebug;
    pstrMsg = &strMsg;
    nLeft = PRINT_MSG_SIZE - cnDebug - 1;

    /* Called within interrupt routines. */
    if ( in_intr ) {

        /* If not able to get lock then put in the interrupt message buffer.
        */
        if ( test_bit( 1, &lockDebug ) ) {
            pcnDebug = &cnIntBuf;
            pstrMsg = &strIntMsg;
            nLeft = PRINT_INT_SIZE - cnIntBuf - 1;
            in_intr = 0;
        }
    }
    else {
        set_bit( 1, &lockDebug );
    }
    va_start( args, fmt );
    nLeft = vsnprintf( *pstrMsg, nLeft, fmt, args );
    va_end( args );
    if ( nLeft ) {
        *pcnDebug += nLeft;
        *pstrMsg += nLeft;
    }
    if ( !in_intr ) {
        clear_bit( 1, &lockDebug );
    }
#endif
}  /* DbgMsg */

/* -------------------------------------------------------------------------- */

/*
    StartTimer

    Description:
        This routine starts the kernel timer after the specified time tick.

    Parameters:
        PTTimerInfo pInfo
            Pointer to kernel timer information.

        int nTime
            The time tick.

    Return (None):
*/

static void StartTimer (
    PTTimerInfo pInfo,
    int         nTime )
{
    pInfo->cnCount = 0;
    pInfo->pTimer->expires = jiffies + nTime;
    add_timer( pInfo->pTimer );

    /* infinity */
    pInfo->nCount = -1;
}  /* StartTimer */


/*
    StopTimer

    Description:
        This routine stops the kernel timer.

    Parameters:
        PTTimerInfo pInfo
            Pointer to timer information.

    Return (None):
*/

static void StopTimer (
    PTTimerInfo pInfo )
{
    pInfo->nCount = 0;
    del_timer_sync( pInfo->pTimer );
}  /* StopTimer */

/* -------------------------------------------------------------------------- */

/*
    InitHardware

    Description:
        This routine initializes system variables required to acquire the
        hardware.

    Parameters:
        struct dev_info* priv
            Pointer to real hardware device private data.

    Return (None):
*/

static void InitHardware (
    struct dev_info* priv )
{
    ASSERT( priv->hw.m_ulVIoAddr );

    spin_lock_init( &priv->lockHardware );
    init_waitqueue_head( &priv->wqhHardware );
}  /* InitHardware */


/*
    AcquireHardware

    Description:
        This function is used to acquire the hardware so that only one process
        has access to the hardware.

    Parameters:
        struct dev_info* priv
            Pointer to real hardware device private data.

        int in_intr
            Indicate calling from interrupt routines.

        int wait
            Indicate whether to wait or not.  User functions should wait, while
            periodic timer routine should not.

    Return (int):
        Zero if successful; otherwise an error code indicating failure.
*/

int AcquireHardware (
    struct dev_info* priv,
    int              in_intr,
    int              wait )
{
    int rc;

    ASSERT( priv->hw.m_ulVIoAddr );
    do {

        /* Acquire access to the hardware acquire count variable. */
        if ( in_intr )
            spin_lock( &priv->lockHardware );
        else
            spin_lock_irq( &priv->lockHardware );
        if ( 0 == priv->hw.m_bAcquire ) {

#ifdef SH_BUS_DMA
            priv->hw.m_InterruptBlocked =
                HardwareBlockInterrupt( &priv->hw );
#endif

            /* Increase hardware acquire count. */
            priv->hw.m_bAcquire++;

            /* Release access to the hardware acquire count variable. */
            if ( in_intr )
                spin_unlock( &priv->lockHardware );
            else
                spin_unlock_irq( &priv->lockHardware );

            /* Hardware acquired. */
            return 0;
        }

        /* Release access to the hardware acquire count variable. */
        if ( in_intr )
            spin_unlock( &priv->lockHardware );
        else
            spin_unlock_irq( &priv->lockHardware );

        /* Willing to wait. */
        if ( wait ) {
            if ( ( rc = wait_event_interruptible( priv->wqhHardware,
                    !priv->hw.m_bAcquire )) ) {

                /* System failure. */
                return( rc );
            }
        }
    } while ( wait );

    /* Hardware not acquired. */
    return -EBUSY;
}  /* AcquireHardware */


/*
    ReleaseHardware

    Description:
        This routine is used to release the hardware so that other process can
        have access to the hardware.

    Parameters:
        struct dev_info* priv
            Pointer to real hardware device private data.

        int in_intr
            Indicate calling from interrupt routines.

    Return (None):
*/

void ReleaseHardware (
    struct dev_info* priv,
    int              in_intr )
{
    ASSERT( priv->hw.m_ulVIoAddr );

    /* Acquire access to the hardware acquire count variable. */
    if ( in_intr )
        spin_lock( &priv->lockHardware );
    else
        spin_lock_irq( &priv->lockHardware );

    /* Decrease hardware acquire count. */
    priv->hw.m_bAcquire--;

#ifdef SH_BUS_DMA
    HardwareRestoreInterrupt( &priv->hw, priv->hw.m_InterruptBlocked );
#endif

    /* Wake up other processes waiting in the queue. */
    if ( 0 == priv->hw.m_bAcquire )
        wake_up_interruptible( &priv->wqhHardware );

    /* Release access to the hardware acquire count variable. */
    if ( in_intr )
        spin_unlock( &priv->lockHardware );
    else
        spin_unlock_irq( &priv->lockHardware );
}  /* ReleaseHardware */

/* -------------------------------------------------------------------------- */

#ifndef HardwareDisableInterruptSync
/*
    HardwareDisableInterruptSync

    Description:
        This procedure makes sure the interrupt routine is not entered after
        the hardware interrupts are disabled.

    Parameters:
        void* ptr
            Pointer to hardware information structure.

    Return (None):
*/

void HardwareDisableInterruptSync (
    void* ptr )
{
    PHARDWARE          pHardware = ( PHARDWARE ) ptr;
    struct net_device* dev = ( struct net_device* ) pHardware->m_pDevice;
    struct dev_priv*   priv = ( struct dev_priv* ) dev->priv;
    unsigned long      flags;

    spin_lock_irqsave( &priv->lock, flags );
    HardwareDisableInterrupt( pHardware );

#ifdef BLOCK_STATUS
    HardwareReadInterrupt( pHardware, &pHardware->m_InterruptStatus );
    if ( pHardware->m_InterruptStatus ) {
        HardwareAcknowledgeInterrupt( pHardware, pHardware->m_wInterruptMask );
    }
#endif
    spin_unlock_irqrestore( &priv->lock, flags );
}  /* HardwareDisableInterruptSync */
#endif

/* -------------------------------------------------------------------------- */

static void display_mib_counters (
    PHARDWARE pHardware )
{
    PPORT_CONFIG pPort;
    int          port;

    pPort = &pHardware->m_Port[ MAIN_PORT ];
    for ( port = 0; port < TOTAL_PORT_COUNTER_NUM; port++ ) {
        printk( "%2x = %-10lu  ", port,
            ( unsigned long )( pPort->cnCounter[ port ]));
        if ( ( port % 4 ) == 3 )
            printk( "\n" );
    }
    printk( "\n" );
#ifdef DEF_KS8842
    pPort = &pHardware->m_Port[ OTHER_PORT ];
    for ( port = 0; port < TOTAL_PORT_COUNTER_NUM; port++ ) {
        printk( "%2x = %-10lu  ", port,
            ( unsigned long )( pPort->cnCounter[ port ]));
        if ( ( port % 4 ) == 3 )
            printk( "\n" );
    }
    printk( "\n" );
    pPort = &pHardware->m_Port[ HOST_PORT ];
    for ( port = 0; port < TOTAL_PORT_COUNTER_NUM; port++ ) {
        printk( "%2x = %-10lu  ", port,
            ( unsigned long )( pPort->cnCounter[ port ]));
        if ( ( port % 4 ) == 3 )
            printk( "\n" );
    }
    printk( "\n" );
#endif
}  /* display_mib_counter */

/* -------------------------------------------------------------------------- */

#define proc_dir_name  "driver/ksz8842_isa"


typedef enum {

    /* Read-only entries. */
    PROC_INFO,

    /* Read-write entries. */
    PROC_SET_DUPLEX,
    PROC_SET_SPEED,
    PROC_SET_FORCE,

#ifdef DEF_KS8842
    PROC_SET_BROADCAST_STORM,
    PROC_SET_DIFFSERV,
    PROC_SET_802_1P,
#endif

    /* Port specific read-only entries. */
    PROC_GET_CABLE_STATUS,
    PROC_GET_LINK_STATUS,

    /* Port specific read-write entries. */
    PROC_SET_CAPABILITIES,

#ifdef DEF_KS8842
    PROC_ENABLE_BROADCAST_STORM,
    PROC_ENABLE_DIFFSERV,
    PROC_ENABLE_802_1P,
    PROC_ENABLE_PRIORITY_QUEUE,

    PROC_SET_PORT_BASED,

    PROC_ENABLE_PRIORITY_RATE,
    PROC_SET_RX_P0_RATE,
    PROC_SET_RX_P1_RATE,
    PROC_SET_RX_P2_RATE,
    PROC_SET_RX_P3_RATE,
    PROC_SET_TX_P0_RATE,
    PROC_SET_TX_P1_RATE,
    PROC_SET_TX_P2_RATE,
    PROC_SET_TX_P3_RATE,
#endif

    PROC_LAST
} TProcNum;


typedef struct {
    TProcNum proc_num;
    char*    proc_name;
} TProcInfo, *PTProcInfo;


static TProcInfo ProcInfo[ PROC_LAST ] = {

    /* Read-only entries. */
    { PROC_INFO,                    "info",                 },

    /* Read-write entries. */
    { PROC_SET_DUPLEX,              "duplex",               },
    { PROC_SET_SPEED,               "speed",                },
    { PROC_SET_FORCE,               "force",                },

#ifdef DEF_KS8842
    { PROC_SET_BROADCAST_STORM,     "broadcast_percent",    },
    { PROC_SET_DIFFSERV,            "v_diffserv",           },
    { PROC_SET_802_1P,              "v_802_1p",             },
#endif

    /* Port specific read-only entries. */
    { PROC_GET_CABLE_STATUS,        "cable_status",         },
    { PROC_GET_LINK_STATUS,         "link_status",          },

    /* Port specific read-write entries. */
    { PROC_SET_CAPABILITIES,        "set_capabilities",     },

#ifdef DEF_KS8842
    { PROC_ENABLE_BROADCAST_STORM,  "broadcast_storm",      },
    { PROC_ENABLE_DIFFSERV,         "diffserv",             },
    { PROC_ENABLE_802_1P,           "p_802_1p",             },
    { PROC_ENABLE_PRIORITY_QUEUE,   "priority_queue",       },

    { PROC_SET_PORT_BASED,          "port_based",           },

    { PROC_ENABLE_PRIORITY_RATE,    "priority_rate",        },
    { PROC_SET_RX_P0_RATE,          "rx_p0_rate",           },
    { PROC_SET_RX_P1_RATE,          "rx_p1_rate",           },
    { PROC_SET_RX_P2_RATE,          "rx_p2_rate",           },
    { PROC_SET_RX_P3_RATE,          "rx_p3_rate",           },
    { PROC_SET_TX_P0_RATE,          "tx_p0_rate",           },
    { PROC_SET_TX_P1_RATE,          "tx_p1_rate",           },
    { PROC_SET_TX_P2_RATE,          "tx_p2_rate",           },
    { PROC_SET_TX_P3_RATE,          "tx_p3_rate",           },
#endif
};


typedef struct {
    struct dev_info* priv;
    PTProcInfo       info;
    int              port;
} TProcPortInfo, *PTProcPortInfo;


TProcPortInfo ProcPortInfo[ 3 ][ PROC_LAST ];


static struct proc_dir_entry* proc_info;
static struct proc_dir_entry* proc_set;


/*
    read_proc

    Description:
        This function process the read operation of /proc files.

    Parameters:

    Return (int):
        The length of buffer data returned.
*/

static
int read_proc (
    char*  buf,
    char** start,
    off_t  offset,
    int    count,
    int*   eof,
    void*  data )
{
#ifdef DEF_KS8842
    int              i;
#endif
    int              len = 0;
    int              processed = TRUE;
    int              rc;
    PTProcPortInfo   proc = ( PTProcPortInfo ) data;
    PTProcInfo       info = proc->info;
    struct dev_info* priv = proc->priv;
    PHARDWARE        pHardware = &priv->hw;
    DWORD            dwData[ 5 * 2 ];

    /* Why want more? */
    if ( offset ) {
        *eof = 1;
        return( 0 );
    }

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0))
    MOD_INC_USE_COUNT;
#endif

    if ( down_interruptible( &priv->proc_sem ) ) {

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0))
        MOD_DEC_USE_COUNT;
#endif
        return -ERESTARTSYS;
    }

#if 0
    len += sprintf( buf + len, "%s:%d %d\n", info->proc_name, info->proc_num,
        proc->port );
#endif
    switch ( info->proc_num ) {
        case PROC_SET_DUPLEX:
            len += sprintf( buf + len, "%u; ",
                pHardware->m_bDuplex );
            if ( MediaStateConnected == pHardware->m_ulHardwareState ) {
                if ( 1 == pHardware->m_ulDuplex )
                    len += sprintf( buf + len, "half-duplex\n" );
                else if ( 2 == pHardware->m_ulDuplex )
                    len += sprintf( buf + len, "full-duplex\n" );
            }
            else
                len += sprintf( buf + len, "unlinked\n" );
            break;
        case PROC_SET_SPEED:
            len += sprintf( buf + len, "%u; ",
                pHardware->m_bSpeed );
            if ( MediaStateConnected == pHardware->m_ulHardwareState ) {
                len += sprintf( buf + len, "%u\n",
                    pHardware->m_ulTransmitRate / 10000 );
            }
            else
                len += sprintf( buf + len, "unlinked\n" );
            break;
        case PROC_SET_FORCE:
            len += sprintf( buf + len, "%u\n",
                pHardware->m_bForceLink );
            break;
#ifdef DEF_KS8842
        case PROC_SET_BROADCAST_STORM:
            len += sprintf( buf + len, "%u%%\n",
                pHardware->m_bBroadcastPercent );
            break;
        case PROC_SET_DIFFSERV:
            for ( i = 0; i < 64; i++ ) {
                len += sprintf( buf + len, "%2u=0x%02X ", i, ( int )
                    pHardware->m_wDiffServ[ i ]);
                if ( ( i % 8 ) == 7 )
                    len += sprintf( buf + len, "\n" );
            }
            break;
        case PROC_SET_802_1P:
            for ( i = 0; i < 8; i++ )
                len += sprintf( buf + len, "%u=%u ", i,
                    pHardware->m_b802_1P_Priority[ i ]);
            len += sprintf( buf + len, "\n" );
            break;
        case PROC_SET_PORT_BASED:
            len += sprintf( buf + len, "%u\n",
                pHardware->m_Port[ proc->port ].bPortPriority );
            break;
        case PROC_SET_RX_P0_RATE:
            len += sprintf( buf + len, "%u\n", ( int )
                pHardware->m_Port[ proc->port ].dwRxRate[ 0 ]);
            break;
        case PROC_SET_RX_P1_RATE:
            len += sprintf( buf + len, "%u\n", ( int )
                pHardware->m_Port[ proc->port ].dwRxRate[ 1 ]);
            break;
        case PROC_SET_RX_P2_RATE:
            len += sprintf( buf + len, "%u\n", ( int )
                pHardware->m_Port[ proc->port ].dwRxRate[ 2 ]);
            break;
        case PROC_SET_RX_P3_RATE:
            len += sprintf( buf + len, "%u\n", ( int )
                pHardware->m_Port[ proc->port ].dwRxRate[ 3 ]);
            break;
        case PROC_SET_TX_P0_RATE:
            len += sprintf( buf + len, "%u\n", ( int )
                pHardware->m_Port[ proc->port ].dwTxRate[ 0 ]);
            break;
        case PROC_SET_TX_P1_RATE:
            len += sprintf( buf + len, "%u\n", ( int )
                pHardware->m_Port[ proc->port ].dwTxRate[ 1 ]);
            break;
        case PROC_SET_TX_P2_RATE:
            len += sprintf( buf + len, "%u\n", ( int )
                pHardware->m_Port[ proc->port ].dwTxRate[ 2 ]);
            break;
        case PROC_SET_TX_P3_RATE:
            len += sprintf( buf + len, "%u\n", ( int )
                pHardware->m_Port[ proc->port ].dwTxRate[ 3 ]);
            break;
#endif
        default:
            processed = FALSE;
    }

    /* Require hardware to be acquired first. */
    if ( !processed ) {
        if ( ( rc = AcquireHardware( priv, FALSE, TRUE )) ) {
            up( &priv->proc_sem );
            return( rc );
        }
        switch ( info->proc_num ) {
#ifdef DEF_KS8842
            case PROC_ENABLE_BROADCAST_STORM:
                len += sprintf( buf + len, "%d\n",
                    PortGetBroadcastStorm( pHardware, proc->port ));
                break;
            case PROC_ENABLE_DIFFSERV:
                len += sprintf( buf + len, "%d\n",
                    PortGetDiffServ( pHardware, proc->port ));
                break;
            case PROC_ENABLE_802_1P:
                len += sprintf( buf + len, "%d\n",
                    PortGet802_1P( pHardware, proc->port ));
                break;
            case PROC_ENABLE_PRIORITY_QUEUE:
                len += sprintf( buf + len, "%d\n",
                    PortGetPriority( pHardware, proc->port ));
                break;
            case PROC_ENABLE_PRIORITY_RATE:
                len += sprintf( buf + len, "%d\n",
                    SwitchGetPriorityRate( pHardware, proc->port ));
                break;
#endif
            case PROC_GET_CABLE_STATUS:
                HardwareGetCableStatus( pHardware, proc->port, dwData );
                len += sprintf( buf + len,
                    "^[%u:%u, %u:%u, %u:%u, %u:%u, %u:%u]$\n",
                    dwData[ 0 ], dwData[ 1 ],
                    dwData[ 2 ], dwData[ 3 ],
                    dwData[ 4 ], dwData[ 5 ],
                    dwData[ 6 ], dwData[ 7 ],
                    dwData[ 8 ], dwData[ 9 ]);
                break;
            case PROC_GET_LINK_STATUS:
                HardwareGetLinkStatus( pHardware, proc->port, dwData );
                len += sprintf( buf + len,
                    "^[%08X, %X, %08X, %08X, %08X, %08X]$\n",
                    dwData[ 0 ], dwData[ 1 ], dwData[ 2 ],
                    dwData[ 3 ], dwData[ 4 ], dwData[ 5 ]);
                break;
            case PROC_INFO:
                display_mib_counters( pHardware );
                HardwareReadRegDWord( pHardware, REG_INT_STATUS_BANK,
                    0, &dwData[ 0 ]);
                HardwareReadRegDWord( pHardware, REG_INT_STATUS_BANK,
                    4, &dwData[ 1 ]);
                HardwareReadRegDWord( pHardware, REG_INT_STATUS_BANK,
                    8, &dwData[ 2 ]);
                printk( "%08X %08X %08X\n", dwData[ 0 ], dwData[ 1 ],
                    dwData[ 2 ]);
                printk( "%x %x\n", pHardware->m_InterruptBlocked,
                    pHardware->m_bBlocked );
#ifdef DBG
                ks_rx_overrun = 0;
#endif
                break;
            default:
                break;
        }
        ReleaseHardware( priv, FALSE );
    }
    up( &priv->proc_sem );
    *eof = 1;

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0))
    MOD_DEC_USE_COUNT;
#endif

    return len;
}  /* read_proc */


/*
    write_proc

    Description:
        This function process the write operation of /proc files.

    Parameters:

    Return (int):
        The length of buffer data accepted.
*/

static
int write_proc (
    struct file*  file,
    const char*   buffer,
    unsigned long count,
    void*         data )
{
    int              len;
    int              num;
    int              rc;
    char             str[ 22 ];
    PTProcPortInfo   proc = ( PTProcPortInfo ) data;
    PTProcInfo       info = proc->info;
    struct dev_info* priv = proc->priv;
    PHARDWARE        pHardware = &priv->hw;

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0))
    MOD_INC_USE_COUNT;
#endif

    if ( count > 20 )
        len = 20;
    else
        len = count;
    if ( copy_from_user( str, buffer, len ) ) {

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0))
        MOD_DEC_USE_COUNT;
#endif
        return -EFAULT;
    }
    if ( down_interruptible( &priv->proc_sem ) ) {

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0))
        MOD_DEC_USE_COUNT;
#endif
        return -ERESTARTSYS;
    }
    if ( ( rc = AcquireHardware( priv, FALSE, TRUE )) ) {
        up( &priv->proc_sem );
        return( rc );
    }
    if ( '0' == str[ 0 ]  &&  'x' == str[ 1 ] ) {
        sscanf( &str[ 2 ], "%x", ( unsigned int* ) &num );
    }
    else if ( '0' == str[ 0 ]  &&  'b' == str[ 1 ] ) {
        int i = 2;

        num = 0;
        while ( str[ i ] ) {
            num <<= 1;
            num |= str[ i ] - '0';
            i++;
        }
    }
    else if ( '0' == str[ 0 ]  &&  'd' == str[ 1 ] ) {
        sscanf( &str[ 2 ], "%u", &num );
    }
    else
        sscanf( str, "%d", &num );
    switch ( info->proc_num ) {
        case PROC_SET_DUPLEX:
            if ( num <= 2 )
                pHardware->m_bDuplex = ( BYTE ) num;

#ifdef CHECK_OVERRUN
            if ( 3 == num ) {
                printk( "watermark off\n" );
                HardwareWriteRegWord( pHardware, REG_RX_WATERMARK_BANK,
                    REG_RX_WATERMARK_OFFSET, 0 );
            }
            else if ( 4 == num ) {
                printk( "watermark on\n" );
                HardwareWriteRegWord( pHardware, REG_RX_WATERMARK_BANK,
                    REG_RX_WATERMARK_OFFSET, RX_HIGH_WATERMARK_2KB );
            }
            else if ( 5 == num ) {
                USHORT data;

                HardwareReadRegWord( pHardware, REG_TX_CTRL_BANK,
                    REG_TX_CTRL_OFFSET, &data );
                printk( "flow off: %x\n", data );
                pHardware->m_wTransmitConfig &= ~TX_CTRL_FLOW_ENABLE;
                HardwareStopTransmit( pHardware );
                HardwareStartTransmit( pHardware );
            }
            else if ( 6 == num ) {
                USHORT data;

                HardwareReadRegWord( pHardware, REG_TX_CTRL_BANK,
                    REG_TX_CTRL_OFFSET, &data );
                printk( "flow on: %x\n", data );
                pHardware->m_wTransmitConfig |= TX_CTRL_FLOW_ENABLE;
                HardwareStopTransmit( pHardware );
                HardwareStartTransmit( pHardware );
            }
#endif
            break;
        case PROC_SET_SPEED:
            if ( 0 == num  ||  10 == num  ||  100 == num )
                pHardware->m_bSpeed = ( BYTE ) num;
            break;
        case PROC_SET_FORCE:
            pHardware->m_bForceLink = ( BYTE ) num;
            break;
        case PROC_SET_CAPABILITIES:
            HardwareSetCapabilities( pHardware, proc->port, num );
            break;
#ifdef DEF_KS8842
        case PROC_SET_BROADCAST_STORM:
            HardwareConfigBroadcastStorm( pHardware, num );
            break;
        case PROC_ENABLE_BROADCAST_STORM:
            if ( !num )
                SwitchDisableBroadcastStorm( pHardware, proc->port );
            else
                SwitchEnableBroadcastStorm( pHardware, proc->port );
            break;
        case PROC_ENABLE_DIFFSERV:
            if ( !num )
                SwitchDisableDiffServ( pHardware, proc->port );
            else
                SwitchEnableDiffServ( pHardware, proc->port );
            break;
        case PROC_ENABLE_802_1P:
            if ( !num )
                SwitchDisable802_1P( pHardware, proc->port );
            else
                SwitchEnable802_1P( pHardware, proc->port );
            break;
        case PROC_ENABLE_PRIORITY_QUEUE:
            if ( !num )
                SwitchDisableMultiQueue( pHardware, proc->port );
            else
                SwitchEnableMultiQueue( pHardware, proc->port );
            break;
#if 0
        case PROC_SET_DIFFSERV:
            pHardware->m_wDiffServ[ 0 ] = ( ULONG ) num;
            break;
        case PROC_SET_802_1P:
            HardwareConfig802_1P_Priority( pHardware, ( UCHAR ) num );
            break;
#endif
        case PROC_SET_PORT_BASED:
            SwitchConfigPortBased( pHardware, proc->port, num );
            break;
        case PROC_ENABLE_PRIORITY_RATE:
            if ( !num )
                SwitchDisablePriorityRate( pHardware, proc->port );
            else
                SwitchEnablePriorityRate( pHardware, proc->port );
            break;
        case PROC_SET_RX_P0_RATE:
            HardwareConfigRxPriorityRate( pHardware, proc->port, 0, num );
            break;
        case PROC_SET_RX_P1_RATE:
            HardwareConfigRxPriorityRate( pHardware, proc->port, 1, num );
            break;
        case PROC_SET_RX_P2_RATE:
            HardwareConfigRxPriorityRate( pHardware, proc->port, 2, num );
            break;
        case PROC_SET_RX_P3_RATE:
            HardwareConfigRxPriorityRate( pHardware, proc->port, 3, num );
            break;
        case PROC_SET_TX_P0_RATE:
            HardwareConfigTxPriorityRate( pHardware, proc->port, 0, num );
            break;
        case PROC_SET_TX_P1_RATE:
            HardwareConfigTxPriorityRate( pHardware, proc->port, 1, num );
            break;
        case PROC_SET_TX_P2_RATE:
            HardwareConfigTxPriorityRate( pHardware, proc->port, 2, num );
            break;
        case PROC_SET_TX_P3_RATE:
            HardwareConfigTxPriorityRate( pHardware, proc->port, 3, num );
            break;
#endif
        default:
            printk( KERN_ALERT "write_proc:%d\n", info->proc_num );
    }
    ReleaseHardware( priv, FALSE );
    up( &priv->proc_sem );

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0))
    MOD_DEC_USE_COUNT;
#endif

    return len;
}  /* write_proc */

/* -------------------------------------------------------------------------- */

#ifdef SH_BUS_DMA
BOOLEAN TestLoopback_ISA ( PHARDWARE );

unsigned char* dma_src = NULL;
unsigned char* dma_dst = NULL;


#ifdef CONFIG_SH_7751_SOLUTION_ENGINE
BOOLEAN Platform_DmaInitialize ( PHARDWARE );
BOOLEAN Platform_DmaStartXfer ( PHARDWARE, UCHAR, ULONG, ULONG, ULONG, BOOLEAN );
BOOLEAN Platform_IsDmaComplete ( PHARDWARE, UCHAR );
void    Platform_DmaTxISR ( void * );
void    Platform_DmaRxISR ( void * );


void dma_rcv_packet_done (
    struct net_device* dev );
void dma_send_packet_done (
    struct net_device* dev );

static void dma_tei (
    int             irq, 
    void           *dev_id, 
    struct pt_regs *regs )
{
    int chan = irq - DMTE_IRQ[ 0 ];

#if 0
printk( "tei:%d\n", chan );
#endif
    if ( chan ) {
        Platform_DmaTxISR( dev_id );
        dma_send_packet_done(( struct net_device* ) dev_id );
    }
    else {
        Platform_DmaRxISR( dev_id );
        dma_rcv_packet_done(( struct net_device* ) dev_id );
    }
}  /* dma_tei */
#endif


#ifdef CONFIG_STM_DMA
int stm_dma_chan[ 2 ] = { 0 };
struct dma_channel* stm_dma[ 2 ];

void dma_rcv_packet_done (
    void* dev );
void dma_send_packet_done (
    void* dev );


void init_stb7100_dma (
    struct net_device* dev )
{
    int i;
    int num = 1;

    if ( !dma_src ) {
        printk( "not init\n" );
        return;
    }

#ifdef SH_BUS_DMA_WRITE
    num = 2;
#endif
    for ( i = 0; i < num; i++ ) {
        if ( 0 == i ) {
            stm_dma_chan[ i ] = request_dma( STB7100_HIGH_BANDWIDTH_CHANNEL,
                "STB7100_TEST" );
            if ( stm_dma_chan[ i ] < 0 )
                stm_dma_chan[ i ] = request_dma( ANY_CHANNEL, "STB7100_TEST" );
        }
        else
            stm_dma_chan[ i ] = request_dma( ANY_CHANNEL, "STB7100_TEST" );
        if ( stm_dma_chan[ i ] < 0 ) {
            printk( "no dma channel: %d\n", i );
            return;
        }

        stm_dma[ i ] = get_dma_channel( stm_dma_chan[ i ]);
        if ( !stm_dma[ i ] ) {
            printk( "no dma: %d\n", i );
            return;
        }

        stm_dma[ i ]->err_callback = NULL;

        if ( i )
            stm_dma[ i ]->comp_callback = dma_send_packet_done;
        else
            stm_dma[ i ]->comp_callback = dma_rcv_packet_done;
        stm_dma[ i ]->comp_callback_param = dev;

#ifdef SH_BUS_DMA_PACED
        stm_dma[ i ]->mode = MODE_PACED;
#else
        stm_dma[ i ]->mode = MODE_FREERUNNING;
#endif

        stm_dma[ i ]->src_sz = 4;
        stm_dma[ i ]->dst_sz = 4;

        stm_dma[ i ]->dstride = 0;
        stm_dma[ i ]->sstride = 0;

        stm_dma[ i ]->list_len = 1;
    }
}  /* init_stb7100_dma */


void enable_dma (
    PHARDWARE pHardware,
    int       on )
{
    USHORT tmp;

    if ( on ) {
        pHardware->m_bSavedBank = pHardware->m_bBank;
        HardwareWriteRegByte( pHardware, REG_EEPROM_CTRL_BANK,
            REG_EEPROM_CTRL_OFFSET, EEPROM_ACCESS_ENABLE | EEPROM_DATA_OUT );
        if ( pHardware->m_bSavedBank != pHardware->m_bBank ) {
            pHardware->m_bBank = pHardware->m_bSavedBank;
            HW_READ_WORD( pHardware, 0, &tmp );
            HW_WRITE_WORD( pHardware, 12, pHardware->m_bBank );
        }
    }
    else {
        HW_READ_WORD( pHardware, 0, &tmp );
        HW_WRITE_WORD( pHardware, 12, REG_EEPROM_CTRL_BANK );
        HW_READ_WORD( pHardware, 0, &tmp );
        HW_WRITE_WORD( pHardware, REG_EEPROM_CTRL_OFFSET,
            EEPROM_ACCESS_ENABLE );
        if ( pHardware->m_bBank != REG_EEPROM_CTRL_BANK ) {
            HW_WRITE_WORD( pHardware, 14, pHardware->m_bBank );
        }
    }
}  /* enable_dma */


#ifdef STM_DMA_TEST
/*
    Aug 9, 2006

    The following routines are used to test the capabilities of the ST7100
    FDMA.  The DMA transfers work but do not behave as expected.  There are
    8 tests.

    Memory to memory:
    
    1. The source and destination memory addresses are both increasing.
       Freerunning mode is used and no dimension is specified.  It works.
    2. The source memory address is fixed, and the destination memory address
       is increasing.  Freerunning mode is used and DIM_0_x_1 is specified.
       It works.
    3. The source memory address is increasing, and the destination memory
       address is fixed.  Freerunning mode is used and DIM_1_x_0 is specified.
       It works.

    Memory to IO:

    4. The source memory address is increasing, and the destination IO address
       is increasing.  Freerunning mode is used and no dimension is specified.
       It works but somehow the 4-byte location after the destination address
       is overwritten.  This causes the KSZ884X chip not able to send packet
       out.
    5. The source IO address is fixed, and the destination memory address is
       increasing.  Freerunning mode is used and DIM_0_x_1 is specified.
       It works.  As this is the setup to read data from the chip, using DMA
       to read packets in the KSZ884X is doable.
    6. The source memory address is increasing, and the destination IO address
       is fixed.  Freerunning mode is used and DIM_1_x_0 is specified.  It
       works but somehow the 4-byte location after the destination address is
       overwritten.  This causes the KSZ884X chip not able to send packets
       out.  As this is the setup to write data to the chip, using DMA to send
       packets in the KSZ884X is not possible.

    Memory to IO using paced mode:

    7. The source IO address is fixed, and the destination memory address is
       increasing.  Paced mode is used and DIM_0_x_1 is specified.  It seems
       to work but behaves like DIM_1_x_0 is specified.  The source address
       is moving, and only the specified destination memory gets updated with
       the data.  Also the write seems to be done in 2 16-bit operations, so
       only the high word gets written to the chip.  It definitely does not
       work to receive packets from the KSZ884X.
    8. The source memory address is increasing, and the destination IO address
       is fixed.  Paced mode is used and DIM_1_x_0 is specified.  It works but
       the write seems to be done in 2 16-bit operations to the same address,
       and for KSZ884X this does not work for writing data to the chip.
       Sometimes the DMA transfer is not completed when the lenght is not in
       multiple of 8 bytes.

    In conclusion, to use DMA to transfer data between host and KSZ884X, only
    reading can be done for now using freerunning mode.

    There are other performance issues related to DMA.  For DMA simulation the
    driver transfers data using PIO but setups the DMA to measure DMA
    performance.  Using freerunning mode with increasing source and
    destination memory addresses the driver has a throughput of 29 Mbps
    running nuttcp TCP transmit.  Switch to fixed address and specify a
    dimension like DIM_0_x_1 or DIM_1_x_0, and the throughput drops to 9 Mbps.
    Using real DMA to receive packets actually drops TCP receive throughput
    from 20 Mbps to 6 Mbps.  Somehow the DMA seems to have problem resetting
    the address to the starting position for next DMA transfers.

    DMA speed test:

    Memory to memory, no dimension:             64 Mbps *
    Memory to memory, DIM_1_x_0 or DIM_0_x_1:   10 Mbps
    IO to memory, DIM_0_x_1:                     6 Mbps
    Memory to IO, DIM_1_x_0;                     9 Mbps
    Paced IO to memory, DIM_0_x_1:              16 Mbps
    Memory to paced IO, DIM_1_x_0:              15 Mbps

    * 141 Mbps when DMA channel 4 is used.  The other tests stay the same.

    KSZ8842 network test:

    nuttcp                  -t              -r              -t -u
    PIO                     44.7 98%        28.7 79%        59.0 97%
    Mem to mem              28.7 54%        20.4 63%        31.5 38%
    Mem to mem, dim          8.6 10%         7.8 12%         9.1  7%
    IO, dim                  7.4  9%         5.1 11%         7.9  6%
    Paced IO, dim           11.6  2%        10.5 16%        12.6  9%

    If the DMA I/O transfer rate can achieve the maximum PIO transfer rate of
    95 Mbps, then it really helps sustaining a 30 Mbps throughput while
    reducing CPU time.


    Aug 14, 2006

    After modifying the DMA channels 12 and 13 to use OPCODE_4 and 1 transfer,
    the paced mode works for receive and transmit packets.  However the
    transfer speeds drop to 11 and 13 Mbps.  As a consequence the network
    speed also drops.

    12, WRITE, OPCODE_4, 1
    13, READ,  OPCODE_4, 1

    Paced IO to memory, DIM_0_x_1:              11 Mbps
    Memory to paced IO, DIM_1_x_0:              13 Mbps

    Paced IO, dim           11.6  2%        10.0 11%        12.5  9%
*/

static int dma_chan;
static int dma_xfer_count;
static int dma_xfer_start;
static int dma_xfer_mode;
static int dma_mode;
static wait_queue_head_t wqhDMA;


void report_test ( void* ptr )
{
    int i = 0;
    int j;

    for ( j = 0; j < dma_xfer_start; j++ ) {
        if ( dma_dst[ j ] != 0x5A ) {
            printk( "modified: %d=%02X\n", j, dma_dst[ j ]);
            break;
        }
    }

    if ( 1 <= dma_mode ) {
        if ( dma_xfer_mode != 1  &&  dma_xfer_mode != 3 ) {
            ULONG data;

            /* Read hardware register for comparison. */
            HW_READ_DWORD(( PHARDWARE ) ptr, 0, &data );
            memcpy( &dma_dst[ 0 ], &data, 4 );

            /* Read next hardware register to verify overwrite. */
            if ( 1 == dma_mode ) {
                HW_READ_DWORD(( PHARDWARE ) ptr, 4, &data );
                memcpy( &dma_dst[ 4 ], &data, 4 );
            }
        }
    }
    if ( 0 == dma_xfer_mode ) {
        for ( i = 0; i < dma_xfer_count; i++ ) {
            if ( dma_src[ i ] != dma_dst[ dma_xfer_start + i ] ) {
                printk( "dma: %d=%02X %02X\n", i, dma_src[ i ],
                    dma_dst[ dma_xfer_start + i ]);
                break;
            }
        }
        j = dma_xfer_start + dma_xfer_count;
    }
    else if ( 1 == dma_xfer_mode  ||  3 == dma_xfer_mode ) {
        for ( i = 0; i < dma_xfer_count; i += 4 ) {
            for ( j = 0; j < 4; j++ ) {
                if ( i + j == dma_xfer_count ) {
                    i = dma_xfer_count;
                    break;
                }
                if ( dma_src[ j ] != dma_dst[ dma_xfer_start + i + j ] ) {
                    if ( 2 == dma_mode ) {
                        ULONG* src = ( ULONG* ) &dma_src[ j ];
                        ULONG* dst = ( ULONG* )
                            &dma_dst[ dma_xfer_start + i + j ];

                        printk( "dma: %d=%08lx %08lx\n", j, *src, *dst );
                    }
                    else
                        printk( "dma: %d=%02X %02X\n", j,
                            dma_src[ j ], dma_dst[ dma_xfer_start + i + j ]);
                    break;
                }
            }
            if ( j < 4 )
                break;
        }
        j = dma_xfer_start + dma_xfer_count;
    }
    else if ( 2 == dma_xfer_mode ) {
        i = dma_xfer_count;
        if ( i >= 4 )
        for ( j = 0; j < 4; j++ ) {
            if ( dma_src[ i - 4 + j ] != dma_dst[ dma_xfer_start + j ] ) {
                if ( 2 == dma_mode ) {
                    ULONG* src = ( ULONG* ) &dma_src[ i - 4 + j ];
                    ULONG* dst = ( ULONG* ) &dma_dst[ dma_xfer_start + j ];

                    printk( "dma: %d=%08lx %08lx\n", i - 4 + j, *src, *dst );
                }
                else
                    printk( "dma: %d=%02X %02X\n", i - 4 + j,
                        dma_src[ i - 4 + j ], dma_dst[ dma_xfer_start + j ]);
                i = 0;
                break;
            }
        }
        j = dma_xfer_start + 4;
    }
    if ( i == dma_xfer_count )
        printk( "transferred: %d\n", dma_xfer_count );
    if ( 0 == dma_mode  ||  2 == dma_mode  ||
            ( 1 == dma_mode  &&  1 == dma_xfer_mode ) ) {
        for ( ; j < 2048; j++ ) {
            if ( dma_dst[ j ] != 0x5A ) {
                printk( "modified: %d=%02X\n", j, dma_dst[ j ]);
                break;
            }
        }
    }
    else {
        for ( i = 0; i < 4; i++ ) {
            if ( dma_dst[ j + i ] != dma_src[ i ] ) {
                printk( "modified: %d=%02X%02X%02X%02X\n", j + i,
                    dma_dst[ j + i ], dma_dst[ j + i + 1 ],
                    dma_dst[ j + i + 2 ], dma_dst[ j + i + 3 ]);
                break;
            }
        }
    }
    free_dma( dma_chan );
    wake_up_interruptible( &wqhDMA );
}  /* report_test */


void test_stb7100_dma (
    PHARDWARE pHardware,
    char* msg,
    int mode,
    int xfer )
{
    struct dma_channel* dma_channel;
    ULONG               i;
    ULONG               mac0, mac1;
    int                 rc;

    dma_chan = request_dma( ANY_CHANNEL, "STB7100_TEST" );
    if ( dma_chan < 0 ) {
        printk( "no dma channel\n" );
        return;
    }

    dma_channel = get_dma_channel( dma_chan );
    if ( !dma_channel ) {
        printk( "no dma\n" );
        return;
    }

    printk( "%s - ", msg );
    for ( i = 0; i < 2048; i++ )
        dma_src[ i ] = 0x12 + i;
    memset(( void* ) dma_dst, 0x5a, 2048 );
    dma_cache_wback( dma_src, 2048 );
    dma_cache_wback_inv( dma_dst, 2048 );

    dma_mode = mode;
    dma_xfer_mode = xfer;
    if ( 0 == dma_mode )
        dma_xfer_count = FDMA_MIN_SIZE * 10;
    else if ( 2 == dma_mode ) {
        if ( 2 == dma_xfer_mode )
            dma_xfer_count = FDMA_WR_SIZE * 32;
        else
            dma_xfer_count = FDMA_MIN_SIZE * 10;
    }
    else if ( 0 == dma_xfer_mode  ||
            ( 2 == dma_mode  &&  1 == dma_xfer_mode ) )
        dma_xfer_count = 4;
    else
        dma_xfer_count = 8;
    dma_xfer_start = 0;

    dma_channel->err_callback = NULL;
    dma_channel->comp_callback = report_test;
    dma_channel->comp_callback_param = pHardware;

    if ( 2 == dma_mode )
        dma_channel->mode = MODE_PACED;
    else
        dma_channel->mode = MODE_FREERUNNING;
    dma_channel->count = dma_xfer_count;

    dma_channel->sar = virt_to_bus(( void* ) dma_src );
    dma_channel->dar = virt_to_bus(( void* )( dma_dst + dma_xfer_start ));
    if ( dma_mode >= 1 ) {
        if ( 1 == dma_xfer_mode )
            dma_channel->sar = virt_to_bus(( void* )(
                pHardware->m_ulVIoAddr + 0 ));
        else
            dma_channel->dar = virt_to_bus(( void* )(
                pHardware->m_ulVIoAddr + 0 ));
    }

    dma_channel->src_sz = 4;
    dma_channel->dst_sz = 4;

    dma_channel->dstride = 0;
    dma_channel->sstride = 0;

    dma_channel->list_len = 1;

    if ( 0 == dma_xfer_mode )
        dma_channel->flags = NODE_DONE_ISR | 12 | LIST_TYPE_UNLINKED;
    else if ( 1 == dma_xfer_mode )
        dma_channel->flags = NODE_DONE_ISR | DIM_0_x_1 | LIST_TYPE_UNLINKED;
    else if ( 2 == dma_xfer_mode )
        dma_channel->flags = NODE_DONE_ISR | DIM_1_x_0 | LIST_TYPE_UNLINKED;
    if ( dma_mode >= 1 ) {
        if ( 2 == dma_mode ) {
            if ( 2 == dma_xfer_mode )
            dma_channel->flags |= 13;
            else
            dma_channel->flags |= 12;
        }

        /* Use MAC address bank for register read/write. */
        HardwareSelectBank( pHardware, 2 );
        HW_READ_DWORD( pHardware, 0, &mac0 );
        HW_READ_DWORD( pHardware, 4, &mac1 );
        if ( 1 == dma_xfer_mode ) {
            memcpy( &dma_src[ 0 ], &mac0, 4 );
        }
    }

    if ( !dma_configure_channel( dma_channel ) ) {
        dma_xfer( dma_chan );
        rc = interruptible_sleep_on_timeout( &wqhDMA, 5 );
    }

    /* Restore MAC address. */
    if ( dma_mode >= 1 ) {
        HW_WRITE_DWORD( pHardware, 0, mac0 );
        HW_WRITE_DWORD( pHardware, 4, mac1 );
        HW_WRITE_BYTE( pHardware, 14, pHardware->m_bBank );
    }
}  /* test_stb7100_dma */


#ifdef SH_BUS_DMA_USE_SG
void test_stb7100_sg (
    PHARDWARE pHardware )
{
    struct dma_channel* dma_channel;
    ULONG               i;
    ULONG               mac0, mac1;
    int                 rc;
    struct scatterlist  old_sg;
    struct scatterlist  new_sg[ 2 ];
    struct scatterlist* new_sg_ptr;
    struct scatterlist* old_sg_ptr;
    int                 old_sg_len;
    int                 new_sg_len;
    int                 sg_count;

    dma_chan = request_dma( ANY_CHANNEL, "STB7100_TEST" );
    if ( dma_chan < 0 ) {
        printk( "no dma channel\n" );
        return;
    }

    dma_channel = get_dma_channel( dma_chan );
    if ( !dma_channel ) {
        printk( "no dma\n" );
        return;
    }

    for ( i = 0; i < 2048; i++ )
        dma_src[ i ] = 0x12 + i;
    memset(( void* ) dma_dst, 0x5a, 2048 );
    dma_cache_wback( dma_src, 2048 );
    dma_cache_wback_inv( dma_dst, 2048 );

    dma_mode = 2;
    dma_xfer_mode = 3;
    dma_xfer_count = FDMA_MIN_SIZE * 8 + 10;
    dma_xfer_start = 0;

    old_sg.page = virt_to_page( dma_dst );
    old_sg.offset = ( long ) dma_dst & ~PAGE_MASK;
    old_sg.length = dma_xfer_count;

    new_sg_ptr = new_sg;
    old_sg_ptr = ( struct scatterlist* ) &old_sg;
    new_sg_len = 1;
    old_sg_len = new_sg_len;
    while ( old_sg_len-- ) {
        *new_sg_ptr = *old_sg_ptr;
        if ( new_sg_ptr->length < FDMA_MIN_SIZE ) {
        }
        else if ( new_sg_ptr->length % FDMA_MIN_SIZE ) {
            int biglen = new_sg_ptr->length -
                ( new_sg_ptr->length % FDMA_MIN_SIZE );

            new_sg_ptr->length = biglen;
            new_sg_ptr++;
            *new_sg_ptr = *old_sg_ptr;
            new_sg_ptr->offset += biglen;
            new_sg_ptr->length -= biglen;
            new_sg_len++;
        }
        new_sg_ptr++;
        old_sg_ptr++;
    }
    sg_count = dma_map_sg( NULL, new_sg, new_sg_len, DMA_FROM_DEVICE );

    dma_channel->err_callback = NULL;
    dma_channel->comp_callback = report_test;
    dma_channel->comp_callback_param = pHardware;

    if ( 2 == dma_mode )
        dma_channel->mode = MODE_PACED;
    else
        dma_channel->mode = MODE_FREERUNNING;
    dma_channel->count = dma_xfer_count;

    dma_channel->sar = virt_to_bus(( void* ) dma_src );
    dma_channel->dar = virt_to_bus(( void* )( dma_dst + dma_xfer_start ));
    if ( dma_mode >= 1 ) {
        if ( 2 == dma_xfer_mode )
            dma_channel->dar = virt_to_bus(( void* )(
                pHardware->m_ulVIoAddr + 0 ));
        else
            dma_channel->sar = virt_to_bus(( void* )(
                pHardware->m_ulVIoAddr + 0 ));
    }

    dma_channel->dstride = 0;
    dma_channel->sstride = 0;

    dma_channel->list_len = sg_count;
    dma_channel->dst_sg = new_sg;
    dma_channel->dst_sg_len = sg_count;
    dma_channel->src_sz = sg_count;

    dma_channel->flags = CALLBACK_CONTEXT_INTERRUPT | DIM_0_x_SG | 12 |
        LIST_TYPE_UNLINKED | DMA_SETUP_NOBLOCK;
    dma_channel->flags |= HORRIBLE_SMSC911X_HACK;
    dma_channel->smsc911x_short_chan = 13;
    if ( dma_mode >= 1 ) {

        /* Use MAC address bank for register read/write. */
        HardwareSelectBank( pHardware, 2 );
        HW_READ_DWORD( pHardware, 0, &mac0 );
        HW_READ_DWORD( pHardware, 4, &mac1 );
        if ( 1 == dma_xfer_mode  ||  3 == dma_xfer_mode ) {
            memcpy( &dma_src[ 0 ], &mac0, 4 );
        }
    }

    if ( !dma_configure_channel( dma_channel ) ) {
        dma_xfer( dma_chan );
        rc = interruptible_sleep_on_timeout( &wqhDMA, 5 );
    }

    /* Restore MAC address. */
    if ( dma_mode >= 1 ) {
        HW_WRITE_DWORD( pHardware, 0, mac0 );
        HW_WRITE_DWORD( pHardware, 4, mac1 );
        HW_WRITE_BYTE( pHardware, 14, pHardware->m_bBank );
    }
}  /* test_stb7100_sg */
#endif


void dma_done ( void* ptr )
{
    wake_up_interruptible( &wqhDMA );
}  /* dma_done */


void test_dma_xfer (
    PHARDWARE pHardware,
    char*     message,
    int       mode,
    int       xfer )
{
    struct dma_channel* dma_channel;
    unsigned long       start;
    unsigned long       stop;
    int                 j;
    ULONG               i;
    int                 rc;
#ifdef SH_BUS_DMA_USE_SG
    struct scatterlist  old_sg;
    struct scatterlist  new_sg[ 2 ];
    struct scatterlist* new_sg_ptr;
    struct scatterlist* old_sg_ptr;
    int                 old_sg_len;
    int                 new_sg_len;
    int                 sg_count;
#endif

    dma_channel = get_dma_channel( dma_chan );
    if ( !dma_channel ) {
        printk( "no dma\n" );
        return;
    }

    for ( i = 0; i < 2048; i++ )
        dma_src[ i ] = 0x12 + i;
    memset(( void* ) dma_dst, 0x5a, 2048 );
    dma_cache_wback( dma_src, 2048 );
    dma_cache_wback_inv( dma_dst, 2048 );

    dma_mode = mode;
    dma_xfer_mode = xfer;
    if ( 1 >= dma_mode )
        dma_xfer_count = 1024;
    else if ( 3 == dma_xfer_mode )
        dma_xfer_count = FDMA_MIN_SIZE * 10;
    else if ( 2 == dma_xfer_mode )
        dma_xfer_count = FDMA_WR_SIZE * 32;
    else
        dma_xfer_count = FDMA_MIN_SIZE * 10;
    dma_xfer_start = 0;

    dma_channel->err_callback = NULL;
    dma_channel->comp_callback = dma_done;
    dma_channel->comp_callback_param = pHardware;

    if ( 2 == dma_mode )
        dma_channel->mode = MODE_PACED;
    else
        dma_channel->mode = MODE_FREERUNNING;

    dma_channel->src_sz = 4;
    dma_channel->dst_sz = 4;

    dma_channel->dstride = 0;
    dma_channel->sstride = 0;

    dma_channel->list_len = 1;

#ifdef SH_BUS_DMA_USE_SG
    if ( 3 == dma_xfer_mode ) {
        old_sg.page = virt_to_page( dma_dst );
        old_sg.offset = ( long ) dma_dst & ~PAGE_MASK;
        old_sg.length = dma_xfer_count;

        new_sg_ptr = new_sg;
        old_sg_ptr = ( struct scatterlist* ) &old_sg;
        new_sg_len = 1;
        old_sg_len = new_sg_len;
        while ( old_sg_len-- ) {
            *new_sg_ptr = *old_sg_ptr;
            if ( new_sg_ptr->length < FDMA_MIN_SIZE ) {
            }
            else if ( new_sg_ptr->length % FDMA_MIN_SIZE ) {
                int biglen = new_sg_ptr->length -
                    ( new_sg_ptr->length % FDMA_MIN_SIZE );

                new_sg_ptr->length = biglen;
                new_sg_ptr++;
                *new_sg_ptr = *old_sg_ptr;
                new_sg_ptr->offset += biglen;
                new_sg_ptr->length -= biglen;
                new_sg_len++;
            }
            new_sg_ptr++;
            old_sg_ptr++;
        }
        sg_count = dma_map_sg( NULL, new_sg, new_sg_len, DMA_FROM_DEVICE );

        dma_channel->list_len = sg_count;
        dma_channel->dst_sg = new_sg;
        dma_channel->dst_sg_len = sg_count;
        dma_channel->src_sz = sg_count;
        dma_channel->flags = CALLBACK_CONTEXT_INTERRUPT | DIM_0_x_SG | 12 |
            LIST_TYPE_UNLINKED | DMA_SETUP_NOBLOCK;
    }
#endif

    start = jiffies;
    for ( j = 0; j < 900; j++ ) {
        dma_channel->count = dma_xfer_count;

        dma_channel->sar = virt_to_bus(( void* ) dma_src );
        dma_channel->dar = virt_to_bus(( void* )( dma_dst + dma_xfer_start ));
        if ( dma_mode >= 1 ) {
            if ( 2 == dma_xfer_mode )
                dma_channel->dar = virt_to_bus(( void* )(
                    pHardware->m_ulVIoAddr + 8 ));
            else
                dma_channel->sar = virt_to_bus(( void* )(
                    pHardware->m_ulVIoAddr + 8 ));
        }

        dma_channel->flags = NODE_DONE_ISR | LIST_TYPE_UNLINKED;
        if ( 0 == dma_xfer_mode )
            dma_channel->flags |= 12;
        else if ( 1 == dma_xfer_mode )
            dma_channel->flags |= DIM_0_x_1;
        else if ( 2 == dma_xfer_mode )
            dma_channel->flags |= DIM_1_x_0;
        else if ( 3 == dma_xfer_mode )
            dma_channel->flags |= DIM_0_x_SG;
        if ( dma_mode >= 1 ) {
            if ( 2 == dma_mode ) {
                if ( 3 == dma_xfer_mode )
                dma_channel->flags |= 12;
                else
                if ( 2 == dma_xfer_mode )
                dma_channel->flags |= 13;
                else
                dma_channel->flags |= 12;
            }
        }

        rc = 1;
        if ( !dma_configure_channel( dma_channel ) ) {
            dma_xfer( dma_chan );
            rc = interruptible_sleep_on_timeout( &wqhDMA, 8 );
        }
        if ( !rc ) {
            printk( "dma not completed!\n" );
            return;
        }
    }
    stop = jiffies;
    printk( "%s: ", message );
    printk( "%lu %lu Mbps\n", stop - start, j * dma_xfer_count * 8 /
        (( stop - start ) * ( 1000 / HZ )) / 1000 );
}  /* test_dma_xfer */


void test_dma_speed (
    PHARDWARE pHardware )
{
    ULONG mac0, mac1;

    dma_chan = request_dma( dma_chan, "STB7100_TEST" );
    if ( dma_chan < 0 ) {
        printk( "no dma channel\n" );
        return;
    }
    printk( "dma channel: %d\n", dma_chan );

    /* Use MAC address bank for register read/write. */
    HW_WRITE_BYTE( pHardware, 14, 2 );
    HW_READ_DWORD( pHardware, 0, &mac0 );
    HW_READ_DWORD( pHardware, 4, &mac1 );

#if 0
    test_dma_xfer( pHardware, "inc mem to inc mem", 0, 0 );
    test_dma_xfer( pHardware, "fixed mem to inc mem", 0, 1 );
    test_dma_xfer( pHardware, "inc mem to fixed mem", 0, 2 );
#endif
#if 0
    test_dma_xfer( pHardware, "fixed io to inc mem", 1, 1 );
    test_dma_xfer( pHardware, "inc mem to fixed io", 1, 2 );
#endif

#ifdef SH_BUS_DMA_USE_0
    test_dma_xfer( pHardware, "paced io to inc mem", 2, 1 );
    test_dma_xfer( pHardware, "inc mem to paced io", 2, 2 );
#endif

#ifdef SH_BUS_DMA_USE_SG
    test_dma_xfer( pHardware, "paced io to sg mem", 2, 3 );
#endif

    free_dma( dma_chan );

    /* Restore MAC address. */
    HW_WRITE_BYTE( pHardware, 14, 2 );
    HW_WRITE_DWORD( pHardware, 0, mac0 );
    HW_WRITE_DWORD( pHardware, 4, mac1 );
    HW_WRITE_BYTE( pHardware, 14, pHardware->m_bBank );
}  /* test_dma_speed */
#endif
#endif
#endif


/*
    netdev_close

    Description:
        This function process the close operation of network device.  This is
        caused by the user command "ifconfig ethX down."

    Parameters:
        struct net_device* dev
            Pointer to network device.

    Return (int):
        Zero if successful; otherwise an error code indicating failure.
*/

static int netdev_close (
    struct net_device* dev )
{
    struct dev_priv* priv = ( struct dev_priv* ) dev->priv;
    struct dev_info* hw_priv = priv->pDevInfo;
    PHARDWARE        pHardware = &hw_priv->hw;
    int              rc;

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0))
    MOD_DEC_USE_COUNT;
#endif
    priv->opened--;

    if ( !( priv->opened ) ) {
        if ( priv->skb_waiting ) {
            priv->stats.tx_dropped++;
            dev_kfree_skb( priv->skb_waiting );
            priv->skb_waiting = NULL;
        }
        netif_stop_queue( dev );

        if ( ( rc = AcquireHardware( hw_priv, FALSE, TRUE )) ) {
            return( rc );
        }

#ifdef TWO_NETWORK_INTERFACE
        PortSet_STP_State( pHardware, priv->port, STP_STATE_BLOCKED );
        pHardware->m_bStarted--;
        if ( !pHardware->m_bStarted )
        {
            dev = hw_priv->pDev[ MAIN_PORT ];
#endif

            if ( hw_priv->MonitorTimerInfo.nCount ) {
                StopTimer( &hw_priv->MonitorTimerInfo );
            }

            HardwareDisableInterrupt( pHardware );
            HardwareDisable( pHardware );

            /* Delay for receive task to stop scheduling itself. */
            DelayMillisec( 2000 / HZ );
            pHardware->m_InterruptBlocked = 0;

#ifdef SH_BUS_DMA
#ifdef CONFIG_SH_7751_SOLUTION_ENGINE
            free_irq( DMTE0_IRQ, dev );
            free_irq( DMTE1_IRQ, dev );
            kfree( dma_src );
            kfree( dma_dst );
#endif

#ifdef CONFIG_STM_DMA
            bigphysarea_free( dma_src, 2048 );
            bigphysarea_free( dma_dst, 2048 );
#endif
#endif
            tasklet_disable( &hw_priv->rx_tasklet );
            free_irq( dev->irq, dev );

#ifdef TWO_NETWORK_INTERFACE
            pHardware->m_bPort = MAIN_PORT;
        }
        else {

            /* Whichever port left running becomes the main port. */
            if ( MAIN_PORT == priv->port )
                pHardware->m_bPort = OTHER_PORT;
            else
                pHardware->m_bPort = MAIN_PORT;
        }
#endif

        ReleaseHardware( hw_priv, FALSE );
        pHardware->m_bBlocked = 0;

#ifdef DBG
        ks_rx_overrun = 0;
#endif
    }

#ifdef DBG_
{
    int i;

    DBG_PRINT( "counters:\n" );
    for ( i = OID_COUNTER_FIRST; i < OID_COUNTER_LAST; i++ )
    {
        DBG_PRINT( "%u = %u, %u\n", i,
            ( int ) pHardware->m_cnCounter[ 0 ][ i ],
            ( int ) pHardware->m_cnCounter[ 1 ][ i ]);
    }
    DBG_PRINT( "wait delays:\n" );
    for ( i = WAIT_DELAY_FIRST; i < WAIT_DELAY_LAST; i++ )
    {
        DBG_PRINT( "%u = %u\n", i, pHardware->m_nWaitDelay[ i ]);
    }
    DBG_PRINT( "bad:\n" );
    for ( i = COUNT_BAD_FIRST; i < COUNT_BAD_LAST; i++ )
    {
        DBG_PRINT( "%u = %u\n", i, ( int ) pHardware->m_nBad[ i ]);
    }
    DBG_PRINT( "good:\n" );
    for ( i = COUNT_GOOD_FIRST; i < COUNT_GOOD_LAST; i++ )
    {
        DBG_PRINT( "%u = %u\n", i, ( int ) pHardware->m_nGood[ i ]);
    }
}
#endif

#ifdef CONFIG_SH_7751_SOLUTION_ENGINE
    mach_led( 0x80, 0 );
#endif

    return 0;
}  /* netdev_close */


/*
    netdev_open

    Description:
        This function process the open operation of network device.  This is
        caused by the user command "ifconfig ethX up."

    Parameters:
        struct net_device* dev
            Pointer to network device.

    Return (int):
        Zero if successful; otherwise an error code indicating failure.
*/

static int netdev_open (
    struct net_device* dev )
{
    struct dev_priv* priv = ( struct dev_priv* ) dev->priv;
    struct dev_info* hw_priv = priv->pDevInfo;
    PHARDWARE        pHardware = &hw_priv->hw;
    int              port = MAIN_PORT;
    int              rc = 0;

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0))
    MOD_INC_USE_COUNT;
#endif

    if ( !( priv->opened ) ) {

        /* Reset device statistics. */
        memset( &priv->stats, 0, sizeof( struct net_device_stats ));

        if ( ( rc = AcquireHardware( hw_priv, FALSE, TRUE )) ) {

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0))
            MOD_DEC_USE_COUNT;
#endif
            return( rc );
        }

#ifdef TWO_NETWORK_INTERFACE
        port = priv->port;
#endif

#ifdef TWO_NETWORK_INTERFACE
        if ( !pHardware->m_bStarted )
#endif
        {
            struct net_device* hw_dev = dev;

#ifdef TWO_NETWORK_INTERFACE
            hw_dev = hw_priv->pDev[ MAIN_PORT ];
#endif
            if ( ( rc = request_irq( hw_dev->irq, DEV_INTERRUPT_PTR,
                    DEV_INTERRUPT_FLAG,
                    hw_dev->name, hw_dev )) ) {
                ReleaseHardware( hw_priv, FALSE );

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0))
                MOD_DEC_USE_COUNT;
#endif
                return( rc );
            }
            tasklet_enable( &hw_priv->rx_tasklet );

#ifdef SH_BUS_DMA
#ifdef CONFIG_SH_7751_SOLUTION_ENGINE
            if ( ( rc = request_irq( DMTE0_IRQ, dma_tei, SA_INTERRUPT,
                    "dma_tei", hw_dev )) ) {
            }
            if ( ( rc = request_irq( DMTE1_IRQ, dma_tei, SA_INTERRUPT,
                    "dma_tei", hw_dev )) ) {
            }
            dma_src = kmalloc( 64, GFP_KERNEL | GFP_DMA );
            dma_dst = kmalloc( 64, GFP_KERNEL | GFP_DMA );
#endif

#ifdef CONFIG_STM_DMA
            dma_src = ( unsigned char* ) bigphysarea_alloc( 2048 );
            dma_dst = ( unsigned char* ) bigphysarea_alloc( 2048 );
#endif
#endif

            pHardware->m_bPromiscuous = FALSE;
            pHardware->m_bAllMulticast = FALSE;
            pHardware->m_bMulticastListSize = 0;
            pHardware->m_ulHardwareState = MediaStateDisconnected;

            /* Initialize to invalid value so that link detection is done. */
            pHardware->m_PortInfo[ MAIN_PORT ].bLinkPartner = 0xFF;
            pHardware->m_PortInfo[ MAIN_PORT ].ulHardwareState =
                MediaStateDisconnected;

#ifdef DEF_KS8842
            pHardware->m_PortInfo[ OTHER_PORT ].bLinkPartner = 0xFF;
            pHardware->m_PortInfo[ OTHER_PORT ].ulHardwareState =
                MediaStateDisconnected;
#endif

            hw_priv->Counter[ MAIN_PORT ].time = jiffies + HZ * 4;

#ifdef DEF_KS8842
            hw_priv->Counter[ OTHER_PORT ].time =
                hw_priv->Counter[ MAIN_PORT ].time + HZ * 2;
            hw_priv->Counter[ HOST_PORT ].time =
                hw_priv->Counter[ OTHER_PORT ].time + HZ * 2;
#else
            hw_priv->Counter[ HOST_PORT ].time =
                hw_priv->Counter[ MAIN_PORT ].time + HZ * 2;
#endif

            HardwareReset( pHardware );
            HardwareSetup( pHardware );
            HardwareSwitchSetup( pHardware );

            if ( pHardware->m_bMacOverrideAddr )
                HardwareSetAddress( pHardware );

            PortInitCounters( pHardware, HOST_PORT );

            HardwareSetupInterrupt( pHardware );
            HardwareEnable( pHardware );
            HardwareEnableInterrupt( pHardware );

#ifdef TWO_NETWORK_INTERFACE
            PortSet_STP_State( pHardware, MAIN_PORT, STP_STATE_BLOCKED );
            PortSet_STP_State( pHardware, OTHER_PORT, STP_STATE_BLOCKED );

            /* Whichever port starts first becomes the primary port. */
            pHardware->m_bPort = priv->port;
            pHardware->m_bPortRX = pHardware->m_bPortTX = priv->port;
#endif

            StartTimer( &hw_priv->MonitorTimerInfo,
                hw_priv->MonitorTimerInfo.nTime );
        }
        memset(( void* ) pHardware->m_cnCounter[ port ], 0,
            ( sizeof( ULONGLONG ) * OID_COUNTER_LAST ) );
        PortInitCounters( pHardware, port );

#if defined( DEF_KS8842 )  &&  !defined( TWO_NETWORK_INTERFACE )
        PortInitCounters( pHardware, OTHER_PORT );

        /* In case TWO_NETWORK_INTERFACE driver is used before so the two
           ports are blocked.
        */
        PortSet_STP_State( pHardware, MAIN_PORT, STP_STATE_FORWARDING );
        PortSet_STP_State( pHardware, OTHER_PORT, STP_STATE_FORWARDING );
#endif

#ifdef TWO_NETWORK_INTERFACE
        PortSet_STP_State( pHardware, priv->port, STP_STATE_FORWARDING );
        pHardware->m_bStarted++;
        pHardware->m_ulHardwareState =
            pHardware->m_PortInfo[ priv->port ].ulHardwareState;
#endif
        priv->media_state = pHardware->m_ulHardwareState;

        ReleaseHardware( hw_priv, FALSE );

        if ( MediaStateConnected == priv->media_state )
            netif_carrier_on( dev );
        else
            netif_carrier_off( dev );
        if ( netif_msg_link( priv ) ) {
            printk(KERN_INFO "%s link %s\n", dev->name,
                ( MediaStateConnected == priv->media_state ? "on" : "off" ));
        }

        netif_start_queue( dev );
    }
    priv->opened++;

#ifdef CONFIG_SH_7751_SOLUTION_ENGINE
    mach_led( 0x80, 1 );
#endif

#ifdef SH_BUS_DMA
#ifdef CONFIG_STM_DMA

#ifdef STM_DMA_TEST
    AcquireHardware( hw_priv, FALSE, TRUE );
    HardwareDisable( pHardware );
    init_waitqueue_head( &wqhDMA );
#if 0
    test_stb7100_dma( pHardware, "inc mem to inc mem", 0, 0 );
    test_stb7100_dma( pHardware, "fixed mem to inc mem", 0, 1 );
    test_stb7100_dma( pHardware, "inc mem to fixed mem", 0, 2 );
#endif
#if 0
    test_stb7100_dma( pHardware, "inc mem to inc io", 1, 0 );
    test_stb7100_dma( pHardware, "fixed io to inc mem", 1, 1 );
    test_stb7100_dma( pHardware, "inc mem to fixed io", 1, 2 );
#endif

#ifdef SH_BUS_DMA_USE_0
#if 1
    test_stb7100_dma( pHardware, "paced io to inc mem", 2, 1 );
    test_stb7100_dma( pHardware, "inc mem to paced io", 2, 2 );
#endif
#endif

#ifdef SH_BUS_DMA_USE_SG
#if 1
    test_stb7100_sg( pHardware );
#endif
#endif
#if 1
    dma_chan = ANY_CHANNEL;
    test_dma_speed( pHardware );
    dma_chan = STB7100_HIGH_BANDWIDTH_CHANNEL;
    test_dma_speed( pHardware );
#endif
    HardwareEnable( pHardware );
    ReleaseHardware( hw_priv, FALSE );
#endif
    init_stb7100_dma( dev );
#endif
#endif

    return 0;
}  /* netdev_open */


/* RX errors = rx_errors */
/* RX dropped = rx_dropped */
/* RX overruns = rx_fifo_errors */
/* RX frame = rx_crc_errors + rx_frame_errors + rx_length_errors */
/* TX errors = tx_errors */
/* TX dropped = tx_dropped */
/* TX overruns = tx_fifo_errors */
/* TX carrier = tx_aborted_errors + tx_carrier_errors + tx_window_errors */
/* collisions = collisions */

/*
    dev_query_statistics

    Description:
        This function returns the statistics of the network device.  The device
        needs not be opened.

    Parameters:
        struct net_device* dev
            Pointer to network device.

    Return (struct net_device_stat*):
        Network device statistics.
*/

static struct net_device_stats* dev_query_statistics (
    struct net_device* dev )
{
    struct dev_priv* priv = ( struct dev_priv* ) dev->priv;
    PHARDWARE        pHardware = &priv->pDevInfo->hw;
    PPORT_CONFIG     pPort;
    int              port = MAIN_PORT;

#ifdef TWO_NETWORK_INTERFACE
    port = priv->port;
#endif
    pPort = &pHardware->m_Port[ port ];

#if defined( DEF_KS8841 )  ||  defined( TWO_NETWORK_INTERFACE )
#if 0
    priv->stats.rx_bytes = ( unsigned long )( pPort->cnCounter[
        MIB_COUNTER_RX_LO_PRIORITY ] + pPort->cnCounter[
        MIB_COUNTER_RX_HI_PRIORITY ]);
    priv->stats.rx_packets = ( unsigned long )( pPort->cnCounter[
        MIB_COUNTER_RX_BROADCAST ] + pPort->cnCounter[
        MIB_COUNTER_RX_MULTICAST ] + pPort->cnCounter[
        MIB_COUNTER_RX_UNICAST ]);
    priv->stats.tx_bytes = ( unsigned long )( pPort->cnCounter[
        MIB_COUNTER_TX_LO_PRIORITY ] + pPort->cnCounter[
        MIB_COUNTER_TX_HI_PRIORITY ]);
    priv->stats.tx_packets = ( unsigned long )( pPort->cnCounter[
        MIB_COUNTER_TX_BROADCAST ] + pPort->cnCounter[
        MIB_COUNTER_TX_MULTICAST ] + pPort->cnCounter[
        MIB_COUNTER_TX_UNICAST ]);
#endif
#endif

    priv->stats.rx_errors = pHardware->m_cnCounter[ port ]
        [ OID_COUNTER_RCV_ERROR ];
    priv->stats.tx_errors = pHardware->m_cnCounter[ port ]
        [ OID_COUNTER_XMIT_ERROR ];
#if 0
    priv->stats.tx_carrier_errors =
        pHardware->m_cnCounter[ port ][ OID_COUNTER_XMIT_LOST_CARRIER ];
#endif

    priv->stats.multicast = ( unsigned long ) pPort->cnCounter[
        MIB_COUNTER_RX_MULTICAST ];

    priv->stats.collisions = ( unsigned long ) pPort->cnCounter[
        MIB_COUNTER_TX_TOTAL_COLLISION ];

    priv->stats.rx_length_errors = ( unsigned long )( pPort->cnCounter[
        MIB_COUNTER_RX_UNDERSIZE ] + pPort->cnCounter[
        MIB_COUNTER_RX_FRAGMENT ] + pPort->cnCounter[
        MIB_COUNTER_RX_OVERSIZE ] + pPort->cnCounter[
        MIB_COUNTER_RX_JABBER ]);
    priv->stats.rx_crc_errors = ( unsigned long ) pPort->cnCounter[
        MIB_COUNTER_RX_CRC_ERR ];
    priv->stats.rx_frame_errors = ( unsigned long )( pPort->cnCounter[
        MIB_COUNTER_RX_ALIGNMENT_ERR ] + pPort->cnCounter[
        MIB_COUNTER_RX_SYMBOL_ERR ]);

    priv->stats.tx_window_errors = ( unsigned long ) pPort->cnCounter[
        MIB_COUNTER_TX_LATE_COLLISION ];

#if defined( DEF_KS8842 )  &&  !defined( TWO_NETWORK_INTERFACE )
    pPort = &pHardware->m_Port[ OTHER_PORT ];

    port = OTHER_PORT;
    priv->stats.rx_errors += pHardware->m_cnCounter[ port ]
        [ OID_COUNTER_RCV_ERROR ];
    priv->stats.tx_errors += pHardware->m_cnCounter[ port ]
        [ OID_COUNTER_XMIT_ERROR ];

    priv->stats.multicast += ( unsigned long ) pPort->cnCounter[
        MIB_COUNTER_RX_MULTICAST ];

    priv->stats.collisions += ( unsigned long ) pPort->cnCounter[
        MIB_COUNTER_TX_TOTAL_COLLISION ];

    priv->stats.rx_length_errors += ( unsigned long )( pPort->cnCounter[
        MIB_COUNTER_RX_UNDERSIZE ] + pPort->cnCounter[
        MIB_COUNTER_RX_FRAGMENT ] + pPort->cnCounter[
        MIB_COUNTER_RX_OVERSIZE ] + pPort->cnCounter[
        MIB_COUNTER_RX_JABBER ]);
    priv->stats.rx_crc_errors += ( unsigned long ) pPort->cnCounter[
        MIB_COUNTER_RX_CRC_ERR ];
    priv->stats.rx_frame_errors += ( unsigned long )( pPort->cnCounter[
        MIB_COUNTER_RX_ALIGNMENT_ERR ] + pPort->cnCounter[
        MIB_COUNTER_RX_SYMBOL_ERR ]);

    priv->stats.tx_window_errors += ( unsigned long ) pPort->cnCounter[
        MIB_COUNTER_TX_LATE_COLLISION ];

#if 0
    pPort = &pHardware->m_Port[ HOST_PORT ];
    priv->stats.rx_bytes = ( unsigned long )( pPort->cnCounter[
        MIB_COUNTER_RX_LO_PRIORITY ] + pPort->cnCounter[
        MIB_COUNTER_RX_HI_PRIORITY ]);
    priv->stats.rx_packets = ( unsigned long )( pPort->cnCounter[
        MIB_COUNTER_RX_BROADCAST ] + pPort->cnCounter[
        MIB_COUNTER_RX_MULTICAST ] + pPort->cnCounter[
        MIB_COUNTER_RX_UNICAST ]);
    priv->stats.tx_bytes = ( unsigned long )( pPort->cnCounter[
        MIB_COUNTER_TX_LO_PRIORITY ] + pPort->cnCounter[
        MIB_COUNTER_TX_HI_PRIORITY ]);
    priv->stats.tx_packets = ( unsigned long )( pPort->cnCounter[
        MIB_COUNTER_TX_BROADCAST ] + pPort->cnCounter[
        MIB_COUNTER_TX_MULTICAST ] + pPort->cnCounter[
        MIB_COUNTER_TX_UNICAST ]);
#endif
#endif

#ifdef DEBUG_COUNTER
    if ( jiffies - query_jiffies >= 1000 * HZ / 1000 ) {
        if ( pHardware->m_nGood[ COUNT_GOOD_INT ] ) {
            display_mib_counters( pHardware );
        }
        printk( "%d %d %d; ", rx_cnt1, rx_cnt2, rx_cnt3 );
        rx_cnt1 = rx_cnt2 = rx_cnt3 = 0;
        printk( "I=%u L=%u R=%u; N=%u C=%u; Z=%u, T=%u\n",
            pHardware->m_nGood[ COUNT_GOOD_INT ],
            pHardware->m_nGood[ COUNT_GOOD_INT_LOOP ],
            pHardware->m_nGood[ COUNT_GOOD_INT_RX ],
            pHardware->m_nGood[ COUNT_GOOD_NEXT_PACKET ],
            pHardware->m_nGood[ COUNT_GOOD_RCV_COMPLETE ],
            pHardware->m_nGood[ COUNT_GOOD_NO_NEXT_PACKET ],
            pHardware->m_nGood[ COUNT_GOOD_RCV_NOT_DISCARD ]);
        pHardware->m_nGood[ COUNT_GOOD_INT ] =
        pHardware->m_nGood[ COUNT_GOOD_INT_LOOP ] =
        pHardware->m_nGood[ COUNT_GOOD_INT_RX ] =
        pHardware->m_nGood[ COUNT_GOOD_NEXT_PACKET ] =
        pHardware->m_nGood[ COUNT_GOOD_RCV_COMPLETE ] =
        pHardware->m_nGood[ COUNT_GOOD_NO_NEXT_PACKET ] =
        pHardware->m_nGood[ COUNT_GOOD_RCV_NOT_DISCARD ] =
            0;
    }
    query_jiffies = jiffies;
#endif

    return( &priv->stats );
}  /* dev_query_statistics */


/*
    device_set_mac_address

    Description:
        This function is used to set the MAC address of the network device.

    Parameters:
        struct net_device* dev
            Pointer to network device.

        void* addr
            Buffer of MAC address.

    Return (int):
        Zero to indicate success.
*/

static int device_set_mac_address (
    struct net_device* dev,
    void*              addr )
{
    struct dev_priv* priv = ( struct dev_priv* ) dev->priv;
    struct dev_info* hw_priv = priv->pDevInfo;
    PHARDWARE        pHardware = &hw_priv->hw;
    struct sockaddr* mac = addr;
    int              rc;
    UINT             InterruptMask;

#ifdef DBG
    printk( "set mac address\n" );
#endif

    memcpy( dev->dev_addr, mac->sa_data, MAX_ADDR_LEN );

#ifdef TWO_NETWORK_INTERFACE
    if ( pHardware->m_bPort != priv->port ) {
        return 0;
    }
#endif

    if ( ( rc = AcquireHardware( hw_priv, FALSE, TRUE )) ) {
        return( rc );
    }

    pHardware->m_bMacOverrideAddr = TRUE;
    memcpy( pHardware->m_bOverrideAddress, mac->sa_data, MAC_ADDRESS_LENGTH );

    InterruptMask = HardwareBlockInterrupt( pHardware );
    HardwareSetAddress( pHardware );
    HardwareRestoreInterrupt( pHardware, InterruptMask );

    ReleaseHardware( hw_priv, FALSE );

    return 0;
}  /* device_set_mac_address */


/*
    dev_set_multicast_list

    Description:
        This routine is used to set multicast addresses or put the network
        device into promiscuous mode.

    Parameters:
        struct net_device* dev
            Pointer to network device.

        void* addr
            Buffer of MAC address.

    Return (None):
*/

static void dev_set_multicast_list (
    struct net_device* dev )
{
    struct dev_priv*    priv = ( struct dev_priv* ) dev->priv;
    struct dev_info*    hw_priv = priv->pDevInfo;
    PHARDWARE           pHardware = &hw_priv->hw;
    struct dev_mc_list* mc_ptr;
    int                 rc;

#ifdef DBG
#ifdef TWO_NETWORK_INTERFACE
    printk( "set multicast list: %d\n", priv->port );
#endif
#endif

#ifdef TWO_NETWORK_INTERFACE
    if ( pHardware->m_bPort != priv->port ) {
        return;
    }
#endif

    /* Called in interrupt context. */
    if ( ( rc = AcquireHardware( hw_priv, FALSE, FALSE )) ) {
        return;
    }

    /* Turn on/off promiscuous mode. */
    HardwareSetPromiscuous( pHardware,
        ( BYTE )(( dev->flags & IFF_PROMISC ) == IFF_PROMISC ));

    /* Turn on/off all multicast mode. */
    HardwareSetMulticast( pHardware,
        ( BYTE )(( dev->flags & IFF_ALLMULTI ) == IFF_ALLMULTI ));

    if ( ( dev->flags & IFF_MULTICAST )  &&  dev->mc_count ) {
        if ( dev->mc_count <= MAX_MULTICAST_LIST ) {
            int i = 0;

            for ( mc_ptr = dev->mc_list; mc_ptr; mc_ptr = mc_ptr->next ) {
                if ( !( *mc_ptr->dmi_addr & 1 ) )
                    continue;
                if ( i >= MAX_MULTICAST_LIST )
                    break;
                memcpy( pHardware->m_bMulticastList[ i++ ], mc_ptr->dmi_addr,
                    MAC_ADDRESS_LENGTH );
            }
            pHardware->m_bMulticastListSize = ( BYTE ) i;
            HardwareSetGroupAddress( pHardware );
        }

        /* List too big to support so turn on all multicast mode. */
        else {
            pHardware->m_bMulticastListSize = 255;
            HardwareSetMulticast( pHardware, TRUE );
        }
    }
    else {
        pHardware->m_bMulticastListSize = 0;
        HardwareClearMulticast( pHardware );
    }

    ReleaseHardware( hw_priv, FALSE );
}  /* dev_set_multicast_list */


#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2, 4, 21))
/*
    mdio_read

    Description:
        This function returns the PHY register value.

    Parameters:
        struct net_device* dev
            Pointer to network device.

        int phy_id
            The PHY id.

        int reg_num
            The register number.

    Return (int):
        The register value.
*/

static int mdio_read (
    struct net_device *dev,
    int               phy_id,
    int               reg_num )
{
    struct dev_priv* priv = ( struct dev_priv* ) dev->priv;
    struct dev_info* hw_priv = priv->pDevInfo;
    PHARDWARE        pHardware = &hw_priv->hw;
    int              port = MAIN_PORT;
    u16              val_out;

    if ( phy_id <= OTHER_PORT )
        port = phy_id;
    HardwareReadPhy( pHardware, port, reg_num << 1, &val_out );

    return val_out;
}  /* mdio_read */


/*
    mdio_write

    Description:
        This procedure sets the PHY register value.

    Parameters:
        struct net_device* dev
            Pointer to network device.

        int phy_id
            The PHY id.

        int reg_num
            The register number.

        int val
            The register value.

    Return (None):
*/

static void mdio_write (
    struct net_device *dev,
    int               phy_id,
    int               reg_num,
    int               val )
{
    struct dev_priv* priv = ( struct dev_priv* ) dev->priv;
    struct dev_info* hw_priv = priv->pDevInfo;
    PHARDWARE        pHardware = &hw_priv->hw;
    int              port = MAIN_PORT;

    if ( phy_id <= OTHER_PORT )
        port = phy_id;
    HardwareWritePhy( pHardware, port, reg_num << 1, val );
}  /* mdio_write */


#include "device_ethtool.c"
#endif


/*
    netdev_ioctl

    Description:
        This function is used to process I/O control calls.

    Parameters:
        struct net_device* dev
            Pointer to network device.

        struct ifreq* ifr
            Pointer to interface request structure.

        int cmd
            I/O control code.

    Return (int):
        Zero to indicate success.
*/

#define SIOCDEVDEBUG  ( SIOCDEVPRIVATE + 10 )

static int netdev_ioctl (
    struct net_device* dev,
    struct ifreq*      ifr,
    int                cmd )
{
    struct dev_priv* priv = ( struct dev_priv* ) dev->priv;
    struct dev_info* hw_priv = priv->pDevInfo;
    PHARDWARE        pHardware = &hw_priv->hw;
    PTRequest        req = ( PTRequest ) ifr->ifr_data;
    int              nResult;
    int              rc;
    int              result = 0;
    int              port = MAIN_PORT;

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0))
    struct mii_ioctl_data *data = ( struct mii_ioctl_data* ) &ifr->ifr_data;

#else
    struct mii_ioctl_data *data = if_mii( ifr );
#endif

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0))
    MOD_INC_USE_COUNT;
#endif

    if ( down_interruptible( &hw_priv->proc_sem ) ) {

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0))
        MOD_DEC_USE_COUNT;
#endif
        return -ERESTARTSYS;
    }

    // assume success
    nResult = DEV_IOC_OK;
    switch ( cmd ) {
    	case SIOCGMIIPHY:               /* Get address of MII PHY in use. */
        case SIOCDEVPRIVATE:            /* for binary compat, remove in 2.5 */

#ifdef TWO_NETWORK_INTERFACE
            data->phy_id = priv->port;
            port = priv->port;

#else
            data->phy_id = 0;
#endif

            /* Fallthrough... */

	case SIOCGMIIREG:               /* Read MII PHY register. */
        case SIOCDEVPRIVATE+1:          /* for binary compat, remove in 2.5 */

#ifdef TWO_NETWORK_INTERFACE
            if ( data->phy_id != priv->port  ||

#else
            if ( data->phy_id != 0  ||
#endif
                    data->reg_num >= 6 )
                result = -EIO;
            else {

#ifdef TWO_NETWORK_INTERFACE
                port = priv->port;
#endif
                HardwareReadPhy( pHardware, port, data->reg_num << 1,
                    &data->val_out );
            }
            break;

        case SIOCSMIIREG:               /* Write MII PHY register. */
        case SIOCDEVPRIVATE+2:          /* for binary compat, remove in 2.5 */
            if ( !capable( CAP_NET_ADMIN ) )
                result = -EPERM;

#ifdef TWO_NETWORK_INTERFACE
            else if ( data->phy_id != priv->port  ||

#else
            else if ( data->phy_id != 0  ||
#endif
                    data->reg_num >= 6 )
                result = -EIO;
            else {

#ifdef TWO_NETWORK_INTERFACE
                port = priv->port;
#endif
                HardwareWritePhy( pHardware, port, data->reg_num << 1,
                    data->val_in );

#if defined( DEF_KS8842 )  &&  !defined( TWO_NETWORK_INTERFACE )
                HardwareWritePhy( pHardware, OTHER_PORT, data->reg_num << 1,
                    data->val_in );
#endif
            }
            break;

        case SIOCDEVDEBUG:
            if ( req ) {
                switch ( req->nCmd ) {
                    case DEV_CMD_INIT:
                        req->param.bData[ 0 ] = 'M';
                        req->param.bData[ 1 ] = 'i';
                        req->param.bData[ 2 ] = 'c';
                        req->param.bData[ 3 ] = 'r';
                        req->nSize = 8 + 4;
                        break;
                    case DEV_CMD_GET:
                        switch ( req->nSubCmd ) {
                            case DEV_READ_REG:
                                hwread( req->param.nData[ 0 ],
                                    req->param.nData[ 1 ],
                                    req->param.nData[ 2 ]);
                                break;
                            case DEV_LINK_STATUS:
                                HardwareGetLinkStatus( pHardware, 0,
                                    &req->param.LinkStatus );
                                break;
                        }
                        break;
                    case DEV_CMD_PUT:
                        if ( ( rc = AcquireHardware( hw_priv, FALSE, TRUE )) ) {
                            result = -EAGAIN;
                            break;
                        }
                        switch ( req->nSubCmd ) {
                            case DEV_CAPABILITIES:
                                HardwareSetCapabilities( pHardware, 0,
                                    req->param.nData[ 0 ]);
                                break;
                            case DEV_CABLE_STATUS:
                                HardwareGetCableStatus( pHardware, 0,
                                    req->param.CableStatus );
                                break;
                        }
                        ReleaseHardware( hw_priv, FALSE );
                        break;
                    default:
                        nResult = DEV_IOC_INVALID_CMD;
                        break;
                }
            }
            if ( req ) {
                req->nResult = nResult;
            }
            else if ( !result )
                result = -EFAULT;
            break;
        default:
            result = -EOPNOTSUPP;
    }

    up( &hw_priv->proc_sem );

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0))
    MOD_DEC_USE_COUNT;
#endif

    return( result );
}  /* netdev_ioctl */


/*
    dev_monitor

    Description:
        This routine is run in a kernel timer to monitor the network device.

    Parameters:
        unsigned long ptr
            Pointer value to supplied data.

    Return (None):
*/

static void dev_monitor (
    unsigned long ptr )
{
    struct net_device* dev = ( struct net_device* ) ptr;
    struct dev_priv*   priv = ( struct dev_priv* ) dev->priv;
    struct dev_info*   hw_priv = priv->pDevInfo;
    PHARDWARE          pHardware = &hw_priv->hw;
    PTTimerInfo        pInfo = &hw_priv->MonitorTimerInfo;
    int                port;
    int                rc;

    if ( !( rc = AcquireHardware( hw_priv, FALSE, FALSE )) ) {

/* PHY change interrupt is working for KS8841. */
#ifdef DEF_KS8842
        if ( !pHardware->m_bLinkIntWorking )
            SwitchGetLinkStatus( pHardware );
#endif

#ifdef DEF_KS8841
        for ( port = 0; port < 1; port++ ) {

#else
        for ( port = 0; port < TOTAL_PORT_NUM; port++ ) {
#endif

            /* Reading MIB counters or requested to read. */
            if ( pHardware->m_Port[ port ].bCurrentCounter  ||
                    1 == hw_priv->Counter[ port ].fRead ) {

                /* Need to process receive interrupt. */
                if ( PortReadCounters( pHardware, port ) )
                    goto dev_monitor_release;
                hw_priv->Counter[ port ].fRead = 0;

                /* Finish reading counters. */
                if ( 0 == pHardware->m_Port[ port ].bCurrentCounter ) {
                    hw_priv->Counter[ port ].fRead = 2;
                    wake_up_interruptible(
                        &hw_priv->Counter[ port ].wqhCounter );
                }
            }
            else if ( jiffies >= hw_priv->Counter[ port ].time ) {
                hw_priv->Counter[ port ].fRead = 1;
                if ( port == MAIN_PORT )
                    hw_priv->Counter[ MAIN_PORT ].time = jiffies + HZ * 6;
                else
                    hw_priv->Counter[ port ].time =

#ifdef DEF_KS8842
                        hw_priv->Counter[ port - 1 ].time + HZ * 2;

#else
                        hw_priv->Counter[ MAIN_PORT ].time + HZ * 2;
#endif
            }
        }

#ifdef CONFIG_SH_7751_SOLUTION_ENGINE
        mach_led( led_count, 0 );
        led_count++;
        led_count &= 0xF;
        mach_led( led_count, 1 );
#endif

#ifdef STP_SUPPORT
#ifdef DBG
        if ( dev->br_port  &&
                dev->br_port->state != pHardware->m_PortInfo[ 0 ].nSTP_State ) {
printk( "STP state: %d\n", dev->br_port->state );
pHardware->m_PortInfo[ 0 ].nSTP_State = dev->br_port->state;
        }
#endif
#endif

#ifdef TWO_NETWORK_INTERFACE
        pHardware->m_ulHardwareState =
            pHardware->m_PortInfo[ MAIN_PORT ].ulHardwareState;
        if ( priv->opened  &&
                priv->media_state != pHardware->m_ulHardwareState ) {
            if ( MediaStateConnected == pHardware->m_ulHardwareState )
                netif_carrier_on( dev );
            else
                netif_carrier_off( dev );
            priv->media_state = pHardware->m_ulHardwareState;
            if ( netif_msg_link( priv ) ) {
                printk(KERN_INFO "%s link %s\n", dev->name,
                    ( MediaStateConnected == priv->media_state ?
                    "on" : "off" ));
            }
        }

#ifdef STP_SUPPORT
        if ( dev->br_port  &&  dev->br_port->state !=
                pHardware->m_PortInfo[ MAIN_PORT ].nSTP_State ) {
            PortSet_STP_State( pHardware, MAIN_PORT, dev->br_port->state );
        }
#endif
        dev = hw_priv->pDev[ OTHER_PORT ];
        if ( !dev )
            goto dev_monitor_release;
        pHardware->m_ulHardwareState =
            pHardware->m_PortInfo[ OTHER_PORT ].ulHardwareState;
        priv = ( struct dev_priv* ) dev->priv;

#ifdef STP_SUPPORT
        if ( dev->br_port  &&  dev->br_port->state !=
                pHardware->m_PortInfo[ OTHER_PORT ].nSTP_State ) {
            PortSet_STP_State( pHardware, OTHER_PORT, dev->br_port->state );
        }
#endif
#endif
        if ( priv->opened  &&
                priv->media_state != pHardware->m_ulHardwareState ) {
            if ( MediaStateConnected == pHardware->m_ulHardwareState )
                netif_carrier_on( dev );
            else
                netif_carrier_off( dev );
            priv->media_state = pHardware->m_ulHardwareState;
            if ( netif_msg_link( priv ) ) {
                printk(KERN_INFO "%s link %s\n", dev->name,
                    ( MediaStateConnected == priv->media_state ?
                    "on" : "off" ));
            }
        }

dev_monitor_release:
        ReleaseHardware( hw_priv, FALSE );
    }

#ifdef DBG
    if ( strMsg != strDebug ) {
        if ( fLastDebugLine )
            printk( "%c", DBG_CH );
        fLastDebugLine = 0;
        if ( '\n' == strDebug[ cnDebug - 2 ]  &&
                DBG_CH == strDebug[ cnDebug - 1 ] ) {
            strDebug[ cnDebug - 1 ] = '\0';
            fLastDebugLine = 1;
        }
        strMsg = strDebug;
        cnDebug = 0;
        printk( strDebug );
    }
    if ( strIntMsg != strIntBuf ) {
        printk( "---\n" );
        if ( fLastIntBufLine )
            printk( "%c", DBG_CH );
        fLastIntBufLine = 0;
        if ( '\n' == strIntBuf[ cnIntBuf - 2 ]  &&
                DBG_CH == strIntBuf[ cnIntBuf - 1 ] ) {
            strIntBuf[ cnIntBuf - 1 ] = '\0';
            fLastIntBufLine = 1;
        }
        strIntMsg = strIntBuf;
        cnIntBuf = 0;
        printk( strIntBuf );
    }
#endif

    ++pInfo->cnCount;
    if ( pInfo->nCount > 0 ) {
        if ( pInfo->cnCount < pInfo->nCount ) {
            pInfo->pTimer->expires = jiffies + pInfo->nTime;
            add_timer( pInfo->pTimer );
        }
        else
            pInfo->nCount = 0;
    }
    else if ( pInfo->nCount < 0 ) {
        pInfo->pTimer->expires = jiffies + pInfo->nTime;
        add_timer( pInfo->pTimer );
    }
}  /* dev_monitor */


#ifdef SH_BUS_TEST
void test_memory_copy (
    void* dest,
    void* src,
    int   len,
    char* message )
{
    unsigned long start;
    unsigned long stop;
    int           j;

    start = jiffies;
    for ( j = 0; j < 10000; j++ ) {

#ifdef CONFIG_CPU_SH4_
        dma_cache_wback( src, len );
        dma_cache_inv( dest, len );
#endif
        memcpy( dest, src, len );
    }
    stop = jiffies;
    printk( "%s: ", message );
    printk( "%lu %lu Mbps\n", stop - start, j * len * 8 /
        (( stop - start ) * ( 1000 / HZ )) / 1000 );
}  /* test_memory_copy */


void test_data_read (
    PHARDWARE pHardware,
    void*     dest,
    char*     message )
{
    unsigned long start;
    unsigned long stop;
    int           j;
    PULONG        pdwData;

    pdwData = ( PULONG ) dest;
    start = jiffies;
    for ( j = 0; j < 900; j++ ) {

        /* Setup read address pointer. */
        HardwareWriteRegWord( pHardware, REG_RX_ADDR_PTR_BANK,
            REG_RX_ADDR_PTR_OFFSET, ADDR_PTR_AUTO_INC | 0 );

        HW_READ_BUFFER_DWORD( pHardware, REG_DATA_OFFSET, pdwData, 1024 );
    }
    stop = jiffies;
    printk( "%s: ", message );
    printk( "%lu %lu Mbps\n", stop - start, j * 1024 * 8 /
        (( stop - start ) * ( 1000 / HZ )) / 1000 );
}  /* test_data_read */


void test_data_write (
    PHARDWARE pHardware,
    void*     dest,
    void*     src,
    char*     message )
{
    unsigned long start;
    unsigned long stop;
    int           j;
    PULONG        pdwData;

    pdwData = ( PULONG ) dest;
    start = jiffies;
    for ( j = 0; j < 900; j++ ) {
        memcpy( dest, src, 1024 );

        /* Setup read address pointer. */
        HardwareWriteRegWord( pHardware, REG_RX_ADDR_PTR_BANK,
            REG_RX_ADDR_PTR_OFFSET, ADDR_PTR_AUTO_INC | 0 );

        HW_WRITE_BUFFER_DWORD( pHardware, REG_DATA_OFFSET, pdwData, 1024 );
    }
    stop = jiffies;
    printk( "%s: ", message );
    printk( "%lu %lu Mbps\n", stop - start, j * 1024 * 8 /
        (( stop - start ) * ( 1000 / HZ )) / 1000 );
}  /* test_data_write */


void test_data_read_word (
    PHARDWARE pHardware,
    void*     dest,
    char*     message )
{
    unsigned long start;
    unsigned long stop;
    int           j;
    PULONG        pdwData;

    pdwData = ( PULONG ) dest;
    start = jiffies;
    for ( j = 0; j < 900; j++ ) {

        /* Setup read address pointer. */
        HardwareWriteRegWord( pHardware, REG_RX_ADDR_PTR_BANK,
            REG_RX_ADDR_PTR_OFFSET, ADDR_PTR_AUTO_INC | 0 );

        HW_READ_BUFFER_WORD( pHardware, REG_DATA_OFFSET, pdwData, 1024 );
    }
    stop = jiffies;
    printk( "%s: ", message );
    printk( "%lu %lu Mbps\n", stop - start, j * 1024 * 8 /
        (( stop - start ) * ( 1000 / HZ )) / 1000 );
}  /* test_data_read_word */


void test_data_write_word (
    PHARDWARE pHardware,
    void*     dest,
    void*     src,
    char*     message )
{
    unsigned long start;
    unsigned long stop;
    int           j;
    PULONG        pdwData;

    pdwData = ( PULONG ) dest;
    start = jiffies;
    for ( j = 0; j < 900; j++ ) {
        memcpy( dest, src, 1024 );

        /* Setup read address pointer. */
        HardwareWriteRegWord( pHardware, REG_RX_ADDR_PTR_BANK,
            REG_RX_ADDR_PTR_OFFSET, ADDR_PTR_AUTO_INC | 0 );

        HW_WRITE_BUFFER_WORD( pHardware, REG_DATA_OFFSET, pdwData, 1024 );
    }
    stop = jiffies;
    printk( "%s: ", message );
    printk( "%lu %lu Mbps\n", stop - start, j * 1024 * 8 /
        (( stop - start ) * ( 1000 / HZ )) / 1000 );
}  /* test_data_write_word */


#ifdef SH_BUS_DMA
#ifdef CONFIG_SH_7751_SOLUTION_ENGINE
void test_dma_read (
    PHARDWARE pHardware,
    void*     dest,
    char*     message )
{
    unsigned long start;
    unsigned long stop;
    int           i;
    int           j;
    PULONG        pdwData;

    pdwData = ( PULONG ) dest;
    start = jiffies;
    for ( j = 0; j < 500; j++ ) {

        /* Setup read address pointer. */
        HardwareWriteRegWord( pHardware, REG_RX_ADDR_PTR_BANK,
            REG_RX_ADDR_PTR_OFFSET, ADDR_PTR_AUTO_INC | 0 );

        Platform_DmaStartXfer( pHardware, 0,
            pHardware->m_ulVIoAddr + REG_DATA_OFFSET, 
            virt_to_bus( pdwData ),
            512, FALSE );
        i = 0;
        while ( !Platform_IsDmaComplete( pHardware, 0 ) ) {
            mdelay( 1 );
            if ( ++i > 10 ) {
                printk( "dma timeout!\n" );
                return;
            }
        }
    }
    stop = jiffies;
    printk( "%s: ", message );
    printk( "%lu %lu Mbps\n", stop - start, j * 512 * 8 /
        (( stop - start ) * ( 1000 / HZ )) / 1000 );
}  /* test_dma_read */


void test_dma_write (
    PHARDWARE pHardware,
    void*     dest,
    char*     message )
{
    unsigned long start;
    unsigned long stop;
    int           i;
    int           j;
    PULONG        pdwData;

    pdwData = ( PULONG ) dest;
    start = jiffies;
    for ( j = 0; j < 500; j++ ) {

        /* Setup read address pointer. */
        HardwareWriteRegWord( pHardware, REG_RX_ADDR_PTR_BANK,
            REG_RX_ADDR_PTR_OFFSET, ADDR_PTR_AUTO_INC | 0 );

        Platform_DmaStartXfer( pHardware, 1,
            virt_to_bus( pdwData ),
            pHardware->m_ulVIoAddr + REG_DATA_OFFSET, 
            1024, TRUE );
        i = 0;
        while ( !Platform_IsDmaComplete( pHardware, 1 ) ) {
            mdelay( 1 );
            if ( ++i > 10 ) {
                printk( "dma timeout!\n" );
                return;
            }
        }
    }
    stop = jiffies;
    printk( "%s: ", message );
    printk( "%lu %lu Mbps\n", stop - start, j * 1024 * 8 /
        (( stop - start ) * ( 1000 / HZ )) / 1000 );
}  /* test_dma_write */
#endif
#endif


void test_data_bus (
    PHARDWARE pHardware )
{
    DWORD*  dwData;
    PUCHAR  pbData;
    PUSHORT pwData;

    dwData = kmalloc( 2048, GFP_KERNEL | GFP_DMA );
    pbData = ( PUCHAR ) dwData;
    pwData = ( PUSHORT ) dwData;

#if 1
    test_memory_copy( pwData, pbData + 1024, 1024, "dword-aligned copy" );
    test_memory_copy( pwData, pbData + 1022, 1024, " word-aligned copy" );
    test_memory_copy( pwData, pbData + 1021, 1024, " byte-aligned copy" );
#endif

    test_data_read( pHardware, pbData + 1024,
        "dword-aligned read buffer dword" );
    test_data_write( pHardware, pwData, pbData + 1024,
        "dword-aligned write buffer dword with copy" );

    test_data_read( pHardware, pbData + 1022,
        " word-aligned read buffer dword" );
    test_data_write( pHardware, pwData, pbData + 1022,
        " word-aligned write buffer dword with copy" );

    test_data_read( pHardware, pbData + 1021,
        " byte-aligned read buffer dword" );
    test_data_write( pHardware, pwData, pbData + 1021,
        " byte-aligned write buffer dword with copy" );

#ifndef SH_32BIT_ACCESS_ONLY
#if 0
    test_data_read_word( pHardware, pbData + 1022,
        " word-aligned read buffer word" );
    test_data_write_word( pHardware, pwData, pbData + 1022,
        " word-aligned write buffer word with copy" );

    test_data_read_word( pHardware, pbData + 1021,
        " byte-aligned read buffer word" );
    test_data_write_word( pHardware, pwData, pbData + 1021,
        " byte-aligned write buffer word with copy" );
#endif
#endif

#ifdef SH_BUS_DMA
#ifdef CONFIG_SH_7751_SOLUTION_ENGINE
    test_dma_read( pHardware, pbData,
        "dma read 512" );
    test_dma_write( pHardware, pbData,
        "dma write 1024" );
#endif
#endif

    kfree( dwData );
}  /* test_data_bus */
#endif


/*
    dev_probe

    Description:
        This function is used to probe the device with the given I/O address.

    Parameters:
        struct net_device* dev
            Pointer to network device.

        unsigned long io_addr
            I/O address.

    Return (int):
        Zero if device is detected; otherwise an error code indicating failure.
*/

static int dev_probe (
    struct net_device* dev,
    unsigned long      io_addr )
{
    HARDWARE hw;
    int      rc = 0;

    if ( !request_mem_region( io_addr, BASE_IO_RANGE, dev->name ) )
        return -EBUSY;

    memset( &hw, 0, sizeof( HARDWARE ));

#ifdef IOREMAP_RANGE
    if ( io_addr < 0x80000000 )
        hw.m_pVirtualMemory = ( PUCHAR ) ioremap( io_addr, IOREMAP_RANGE );
    else
#endif
        hw.m_pVirtualMemory = ( PUCHAR ) io_addr;
    hw.m_ulVIoAddr = io_addr;
    hw.m_wPhyAddr = 0;
    if ( !HardwareInitialize( &hw )  ||  !HardwareReset( &hw ) ) {
        DBG_PRINT( "chip not detected"NEWLINE );
        rc = -ENODEV;
        goto dev_probe_done;
    }

    /* Auto-detect interrupt number. */
    if ( ( int ) dev->irq < 0 ) {
        unsigned long mask;

        mask = probe_irq_on();

        HardwareGenerateInterrupt( &hw );

        dev->irq = probe_irq_off( mask );

        HardwareServiceInterrupt( &hw );
    }
    if ( ( int ) dev->irq < 0 ) {
        DBG_PRINT( "IRQ not detected"NEWLINE );
        rc = -ENODEV;
        goto dev_probe_done;
    }

    printk( "%s: I/O = %lx, IRQ = %d\n", dev->name, io_addr, dev->irq );

dev_probe_done:

#ifdef IOREMAP_RANGE
    if ( ( ULONG ) hw.m_pVirtualMemory != hw.m_ulVIoAddr )
        iounmap(( void* ) hw.m_pVirtualMemory );
#endif
    release_mem_region( io_addr, BASE_IO_RANGE );
    return( rc );
}  /* dev_probe */


static struct net_device* last_dev = NULL;
static struct hw_fn* ks8842_fn = NULL;
static int device_present = 0;


static void init_proc (
    struct dev_info* priv )
{
    int  i;
    int  port;

#ifdef DEF_KS8842
    int  port_count = 2;

#else
    int  port_count = 1;
#endif
    char proc_name[ 40 ];

    sprintf( proc_name, "%s-%d", proc_dir_name, priv->id );
    sema_init( &priv->proc_sem, 1 );
    priv->proc_main = proc_mkdir( proc_name, NULL );
    if ( priv->proc_main ) {
        priv->proc_main->owner = THIS_MODULE;
        port = 2;

        /* Read-only entries. */
        for ( i = 0; i < PROC_SET_DUPLEX; i++ ) {
            ProcPortInfo[ port ][ i ].priv = priv;
            ProcPortInfo[ port ][ i ].info = &ProcInfo[ i ];
            ProcPortInfo[ port ][ i ].port = port;
            proc_info = create_proc_read_entry( ProcInfo[ i ].proc_name,
                0444, priv->proc_main, read_proc, &ProcPortInfo[ port ][ i ]);
            if ( proc_info ) {
                proc_info->owner = THIS_MODULE;
            }
        }

        /* Can be written to. */
        for ( i = PROC_SET_DUPLEX; i < PROC_GET_CABLE_STATUS;
                i++ ) {
            ProcPortInfo[ port ][ i ].priv = priv;
            ProcPortInfo[ port ][ i ].info = &ProcInfo[ i ];
            ProcPortInfo[ port ][ i ].port = port;
            proc_set = create_proc_entry( ProcInfo[ i ].proc_name, 0644,
                priv->proc_main );
            if ( proc_set ) {
                proc_set->data = &ProcPortInfo[ port ][ i ];
                proc_set->read_proc = read_proc;
                proc_set->write_proc = write_proc;
                proc_set->owner = THIS_MODULE;
            }
        }
#ifdef DEF_KS8842
        for ( i = PROC_ENABLE_PRIORITY_RATE; i <= PROC_SET_TX_P3_RATE; i++ ) {
            ProcPortInfo[ port ][ i ].priv = priv;
            ProcPortInfo[ port ][ i ].info = &ProcInfo[ i ];
            ProcPortInfo[ port ][ i ].port = port;
            proc_set = create_proc_entry( ProcInfo[ i ].proc_name, 0644,
                priv->proc_main );
            if ( proc_set ) {
                proc_set->data = &ProcPortInfo[ port ][ i ];
                proc_set->read_proc = read_proc;
                proc_set->write_proc = write_proc;
                proc_set->owner = THIS_MODULE;
            }
        }
#endif
        for ( port = 0; port < port_count; port++ ) {
            sprintf( proc_name, "%d", port );
            priv->proc_port[ port ] = proc_mkdir( proc_name, priv->proc_main );
            priv->proc_port[ port ]->owner = THIS_MODULE;
            for ( i = PROC_GET_CABLE_STATUS; i < PROC_SET_CAPABILITIES;
                    i++ ) {
                ProcPortInfo[ port ][ i ].priv = priv;
                ProcPortInfo[ port ][ i ].info = &ProcInfo[ i ];
                ProcPortInfo[ port ][ i ].port = port;
                proc_info = create_proc_read_entry( ProcInfo[ i ].proc_name,
                    0444, priv->proc_port[ port ], read_proc,
                    &ProcPortInfo[ port ][ i ]);
                if ( proc_info ) {
                    proc_info->owner = THIS_MODULE;
                }
            }
            for ( i = PROC_SET_CAPABILITIES; i < PROC_LAST; i++ ) {
                ProcPortInfo[ port ][ i ].priv = priv;
                ProcPortInfo[ port ][ i ].info = &ProcInfo[ i ];
                ProcPortInfo[ port ][ i ].port = port;
                proc_set = create_proc_entry( ProcInfo[ i ].proc_name, 0644,
                    priv->proc_port[ port ]);
                if ( proc_set ) {
                    proc_set->data = &ProcPortInfo[ port ][ i ];
                    proc_set->read_proc = read_proc;
                    proc_set->write_proc = write_proc;
                    proc_set->owner = THIS_MODULE;
                }
            }
        }
    }
}  /* init_proc */


static char* macaddr = ":";


/*
    netdev_init

    Description:
        This function initializes the network device.

    Parameters:
        struct net_device* dev
            Pointer to network device.

    Return (int):
        Zero if successful; otherwise an error code indicating failure.
*/

int __init
ks8842i_\
netdev_init (
    struct net_device* dev )
{
    struct dev_priv* priv;
    struct dev_info* hw_priv = NULL;
    unsigned long    io_addr = dev->base_addr;
    int              rc = -1;

    /* This is a real device. */
    if ( !last_dev ) {

        printk(KERN_INFO "%s\n", version );

        /* A base address is specified. */
        if ( dev->base_addr ) {
            rc = dev_probe( dev, dev->base_addr );
        }

        /* Find base address automatically. */
        else {

#ifdef SH_BUS
            for ( io_addr = 0xB0000000; io_addr <= 0xB0000000;

#else
            for ( io_addr = 0x200; io_addr < 0x380;
#endif
                    io_addr += BASE_IO_RANGE * 4 ) {
                if ( !( rc = dev_probe( dev, io_addr )) ) {
                    dev->base_addr = io_addr;
                    break;
                }
            }
        }

        /* Device is not found. */
        if ( rc ) {
            return( rc );
        }

        ks8842_fn = kmalloc( sizeof( struct hw_fn ), GFP_KERNEL );
        if ( ks8842_fn == NULL ) {
            return -ENOMEM;
        }

        HardwareSetupFunc( ks8842_fn );

        dev->mem_start = dev->base_addr;
        dev->mem_end = dev->mem_start + BASE_IO_RANGE - 1;
        dev->base_addr = 0;

        /* Own the base address so that further probing can skip this address.
        */
        if ( !request_mem_region( dev->mem_start, BASE_IO_RANGE, dev->name ) )
            return -EBUSY;
    }

    /* Allocate private data. */
    dev->priv = kmalloc( sizeof( struct dev_priv ), GFP_KERNEL );
    if ( dev->priv == NULL ) {
        if ( !last_dev ) {
            release_mem_region( dev->mem_start, BASE_IO_RANGE );
        }
        return -ENOMEM;
    }
    memset( dev->priv, 0, sizeof( struct dev_priv ));
    priv = ( struct dev_priv* ) dev->priv;
    spin_lock_init( &priv->lock );

    if ( !last_dev ) {
        priv->pDevInfo = &priv->dev_info;
        hw_priv = priv->pDevInfo;

#ifdef IOREMAP_RANGE
        if ( dev->mem_start < 0x80000000 )
            hw_priv->hw.m_pVirtualMemory = ( PUCHAR )
                ioremap( dev->mem_start, IOREMAP_RANGE );
        else
#endif
            hw_priv->hw.m_pVirtualMemory = ( PUCHAR ) dev->mem_start;
        hw_priv->hw.m_ulVIoAddr = dev->mem_start;
        hw_priv->hw.m_pDevice = dev;

        InitHardware( hw_priv );

        hw_priv->hw.m_hwfn = ks8842_fn;

        hw_priv->MonitorTimerInfo.pTimer = &hw_priv->timerMonitor;
        hw_priv->MonitorTimerInfo.nCount = 0;

        /* 500 ms timeout */
        hw_priv->MonitorTimerInfo.nTime = 500 * HZ / 1000;
        init_timer( &hw_priv->timerMonitor );
        hw_priv->timerMonitor.function = dev_monitor;
        hw_priv->timerMonitor.data = ( unsigned long ) dev;

        /* tasklet is enabled. */
        tasklet_init( &hw_priv->rx_tasklet, RxProcessTask,
            ( unsigned long ) dev );

        /* tasklet_enable will decrement the atomic counter. */
        tasklet_disable( &hw_priv->rx_tasklet );

        init_waitqueue_head( &hw_priv->Counter[ MAIN_PORT ].wqhCounter );
        init_waitqueue_head( &hw_priv->Counter[ OTHER_PORT ].wqhCounter );
        init_waitqueue_head( &hw_priv->Counter[ HOST_PORT ].wqhCounter );
    }

    /* Fill in the fields of the device structure with default values. */
    ether_setup( dev );

    dev->open               = netdev_open;
    dev->stop               = netdev_close;
    dev->hard_start_xmit    = DEV_TRANSMIT;
    dev->tx_timeout         = DEV_TRANSMIT_TIMEOUT;

    /* 500 ms timeout */
    dev->watchdog_timeo     = HZ / 2;
    dev->get_stats          = dev_query_statistics;
    dev->set_mac_address    = device_set_mac_address;
    dev->set_multicast_list = dev_set_multicast_list;
    dev->do_ioctl           = netdev_ioctl;

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2, 4, 21))
    dev->ethtool_ops        = &netdev_ethtool_ops;
#endif

#if (LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 24))
    SET_MODULE_OWNER( dev );
#endif

    /* This is a real device. */
    if ( !last_dev ) {

#ifdef CONFIG_SH_7751_SOLUTION_ENGINE
        mach_led( 0xFF, 0 );
#endif
        if ( ( rc = AcquireHardware( hw_priv, FALSE, TRUE )) ) {
            return( rc );
        }
        HardwareReadAddress( &hw_priv->hw );
        ReleaseHardware( hw_priv, FALSE );
        if ( macaddr[ 0 ] != ':' ) {
            int i;
            int j;
            int got_num;
            int num;

            i = j = num = got_num = 0;
            while ( j < MAC_ADDRESS_LENGTH ) {
                if ( macaddr[ i ] ) {
                    got_num = 1;
                    if ( '0' <= macaddr[ i ]  &&  macaddr[ i ] <= '9' )
                        num = num * 16 + macaddr[ i ] - '0';
                    else if ( 'A' <= macaddr[ i ]  &&  macaddr[ i ] <= 'F' )
                        num = num * 16 + 10 + macaddr[ i ] - 'A';
                    else if ( 'a' <= macaddr[ i ]  &&  macaddr[ i ] <= 'f' )
                        num = num * 16 + 10 + macaddr[ i ] - 'a';
                    else if ( ':' == macaddr[ i ] )
                        got_num = 2;
                    else
                        break;
                }
                else if ( got_num )
                    got_num = 2;
                else
                    break;
                if ( 2 == got_num ) {
                    hw_priv->hw.m_bOverrideAddress[ j++ ] = ( BYTE ) num;
                    num = got_num = 0;
                }
                i++;
            }
            if ( MAC_ADDRESS_LENGTH == j ) {
                hw_priv->hw.m_bMacOverrideAddr = TRUE;
                memcpy( dev->dev_addr, hw_priv->hw.m_bOverrideAddress,
                    MAC_ADDRESS_LENGTH );
            }
        }
        if ( !hw_priv->hw.m_bMacOverrideAddr )
            memcpy( dev->dev_addr, hw_priv->hw.m_bPermanentAddress,
                MAC_ADDRESS_LENGTH );

#ifdef TWO_NETWORK_INTERFACE
        /* Save the device pointer of the first port. */
        hw_priv->pDev[ MAIN_PORT ] = dev;
        hw_priv->pDev[ OTHER_PORT ] = NULL;
        priv->port = MAIN_PORT;

        hw_priv->hw.m_bPortRX = hw_priv->hw.m_bPortTX = MAIN_PORT;
#endif
        hw_priv->hw.m_bPort = MAIN_PORT;

        hw_priv->id = device_present;
        init_proc( hw_priv );

        last_dev = dev;

        phw = &hw_priv->hw;

#ifdef SH_BUS
    /* m_bLookahead needs to be dword-aligned. */
    ASSERT( !(( unsigned long )( hw_priv->hw.m_bLookahead ) & 3 ));
#endif

#ifdef SH_BUS_DMA
#ifdef CONFIG_SH_7751_SOLUTION_ENGINE
    Platform_DmaInitialize( &hw_priv->hw );

/*
    This code needs to be in the kernel to initialize the DMA interrupts.

    for (i = 0; i < 4; i++)
	make_ipr_irq(DMTE_IRQ[i], DMA_IPR_ADDR, DMA_IPR_POS, DMA_PRIORITY);
*/
#endif
#endif

#ifdef SH_BUS_TEST
    test_data_bus( &hw_priv->hw );
#endif
    }

#ifdef TWO_NETWORK_INTERFACE
    else {
        struct dev_priv* last_priv = ( struct dev_priv* ) last_dev->priv;

        dev->base_addr = last_dev->base_addr;
        dev->irq       = last_dev->irq;
        dev->if_port   = last_dev->if_port;
        dev->mem_start = last_dev->mem_start;
        dev->mem_end   = last_dev->mem_end;

        memcpy( dev->dev_addr, last_dev->dev_addr, MAX_ADDR_LEN );

        /* Get hardware information from last device private data. */
        hw_priv = last_priv->pDevInfo;

        /* Save the device pointer of the second port. */
        hw_priv->pDev[ OTHER_PORT ] = dev;

        /* Second port points to the same hardware information as the first
           port.
        */
        priv->pDevInfo = hw_priv;
        priv->port = OTHER_PORT;
        last_dev = NULL;
    }
#endif
    ++device_present;

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2, 4, 21))
    priv->mii_if.phy_id_mask = 0x1;
    priv->mii_if.reg_num_mask = 0x7;
    priv->mii_if.dev = dev;
    priv->mii_if.mdio_read = mdio_read;
    priv->mii_if.mdio_write = mdio_write;

#ifdef DEF_KS8842
#ifdef TWO_NETWORK_INTERFACE
    priv->mii_if.phy_id = priv->port;

#else
    priv->mii_if.phy_id = 0;

    priv->mii_if_2.phy_id_mask = 0x1;
    priv->mii_if_2.reg_num_mask = 0x7;
    priv->mii_if_2.dev = dev;
    priv->mii_if_2.mdio_read = mdio_read;
    priv->mii_if_2.mdio_write = mdio_write;
    priv->mii_if_2.phy_id = 1;
#endif

#else
    priv->mii_if.phy_id = 0;
#endif
#endif
    priv->msg_enable = NET_MSG_ENABLE;

    return 0;
}  /* netdev_init */


#ifdef TWO_NETWORK_INTERFACE
#define MAX_DEVICES  2

#else
#define MAX_DEVICES  1
#endif

static struct net_device NetDevice[ MAX_DEVICES ];

static int io = BASE_IO_ADDR;
static int irq = BASE_IRQ;
static int ifport;

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 9))
module_param( io, int, 0 );
module_param( irq, int, 0 );
module_param( ifport, int, 0 );
module_param( macaddr, charp, 0 );

#else
MODULE_PARM( io, "i" );
MODULE_PARM( irq, "i" );
MODULE_PARM( ifport, "i" );
MODULE_PARM( macaddr, "s" );
#endif
MODULE_PARM_DESC( macaddr, "MAC address" );
MODULE_PARM_DESC( io, "Device I/O base address" );
MODULE_PARM_DESC( irq, "Devie IRQ number" );
MODULE_PARM_DESC( ifport, "Device interface port (0-default, 1-TP, 2-AUI)" );

/*
    init_module

    Description:
        This function starts the device driver.

    Parameters:
        None.

    Return (int):
        Zero if successful; otherwise an error code indicating failure.
*/

#ifdef MODULE
int __init init_module ( void )
#else
static
int __init netdev_init_module ( void )
#endif
{
    int i;
    int result;

#ifdef DBG
    printk( "I/O = %x; IRQ = %d\n", io, irq );
#endif

#ifdef DBG
    strDebug = ( char* ) kmalloc( DEBUG_MSG_SIZE, GFP_KERNEL );
    if ( strDebug ) {
        strMsg = strDebug;
        *strMsg = '\0';
        strIntBuf = strDebug + PRINT_MSG_SIZE + DEBUG_MSG_BUF;
        strIntMsg = strIntBuf;
        *strIntMsg = '\0';
    }
    else
        return -ENOMEM;
#endif

#if defined( CONFIG_SH_7751_SOLUTION_ENGINE )
/*
    This code needs to be in the kernel to initialize the generic bus
    interrupt.

    make_ipr_irq( 8, BCR_ILCRB, 1, 0x0f - 8 );
*/
#if 1
    {
        unsigned long  wcr3;
        unsigned short bcr2;

#if 1
        wcr3 = (*(volatile unsigned long*)(SH7751_WCR2));
        wcr3 &= ~( 0x7 << 17 );
        wcr3 |= ( 0x4 << 17 );
        (*(volatile unsigned long*)(SH7751_WCR2)) = wcr3;
#endif

        wcr3 = (*(volatile unsigned long*)(SH7751_WCR3));
        wcr3 &= ~( 0xf << 16 );
        (*(volatile unsigned long*)(SH7751_WCR3)) = wcr3;

        bcr2 = (*(volatile unsigned short*)(SH7751_BCR2));

#ifdef SH_16BIT
        /* 16-bit I/O */
        bcr2 &= ~( 1 << 8 );
#else
        /* 32-bit I/O */
        bcr2 |= ( 1 << 8 );
#endif
        (*(volatile unsigned short*)(SH7751_BCR2)) = bcr2;
    }
#endif
#endif

#if defined( CONFIG_SH_STB7100_REF ) || defined( CONFIG_SH_STB7100_MBOARD )
#ifndef MODULE
    /* Set STB7100 SYSIRQ0 active low. */
    ilc_route_external(ILC_EXT_IRQ0, 4, 1);
#endif

#if 1
{
    int bank;

    for ( bank = 0; bank < 6; bank++ ) {
        for ( i = 0x1A100100 + 0x40 * bank;
                i < 0x1A100120 + 0x40 * bank; i += 8 )
            printk( "%08X ", ctrl_inl( P2SEGADDR( i )));
        printk( "\n" );
    }
#if 1
    bank = 2;
    ctrl_outl( 0x041086f1, P2SEGADDR( 0x1A100100 + 0x40 * bank ));
#if 0
    ctrl_outl( 0x86111100, P2SEGADDR( 0x1A100100 + 0x40 * bank + 8 ));
    ctrl_outl( 0x85112211, P2SEGADDR( 0x1A100100 + 0x40 * bank + 0x10 ));
#endif
/* For A4. */
#if 0
    ctrl_outl( 0x86001100, P2SEGADDR( 0x1A100100 + 0x40 * bank + 8 ));
    ctrl_outl( 0x85001100, P2SEGADDR( 0x1A100100 + 0x40 * bank + 0x10 ));
#endif
/* For A6. */
#if 1
#if defined( CONFIG_SH_STB7100_REF )
    ctrl_outl( 0x85000100, P2SEGADDR( 0x1A100100 + 0x40 * bank + 8 ));
    ctrl_outl( 0x84000100, P2SEGADDR( 0x1A100100 + 0x40 * bank + 0x10 ));
#elif defined( CONFIG_SH_STB7100_MBOARD )
    ctrl_outl( 0x86000100, P2SEGADDR( 0x1A100100 + 0x40 * bank + 8 ));
    ctrl_outl( 0x85000100, P2SEGADDR( 0x1A100100 + 0x40 * bank + 0x10 ));
#endif
#endif
    printk( "new EMI settings for KSZ884X: " );
    for ( i = 0x1A100100 + 0x40 * bank;
            i < 0x1A100120 + 0x40 * bank; i += 8 )
        printk( "%08X ", ctrl_inl( P2SEGADDR( i )));
    printk( "\n" );
#endif
}
#endif
#endif

#if defined( CONFIG_MACH_SMDK2410KBUS )
    set_external_irq(IRQ_EINT0, EINT_LOWLEVEL, EINT_PULLUP_EN);
#endif

    memset( ProcPortInfo, 0, sizeof( TProcPortInfo ) * 3 * PROC_LAST );
    for ( i = 0; i < MAX_DEVICES; i++ ) {
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 21))
        sprintf( NetDevice[ i ].name, "eth%%d" );
#endif

        /* Copy the parameters from insmod into the device structure. */
        NetDevice[ i ].base_addr = io;
        NetDevice[ i ].irq       = irq;
        NetDevice[ i ].if_port   = ifport;
        NetDevice[ i ].init      = ks8842i_netdev_init;
        result = register_netdev( &NetDevice[ i ] );
        if ( result < 0 ) {
            if ( device_present )
                break;
            return result;
        }
        if ( io )
            io += BASE_IO_RANGE;
    }

    return 0;
}  /* init_module */


/*
    cleanup_module

    Description:
        This routine unloads the device driver.

    Parameters:
        None.

    Return (None):
*/

#ifdef MODULE
void __exit cleanup_module ( void )
#else
static
void __exit netdev_cleanup_module ( void )
#endif
{
    struct dev_priv*   priv;
    struct dev_info*   hw_priv;
    int                i;
    int                j;
    char               proc_name[ 40 ];

#ifdef DBG
    if ( strDebug ) {
        if ( strMsg != strDebug ) {
            printk( "%s\n", strDebug );
        }
        if ( strIntMsg != strIntBuf ) {
            printk( "%s\n", strIntBuf );
        }
        kfree( strDebug );
    }
#endif
    for ( i = 0; i < device_present; i++ ) {
        priv = NetDevice[ i ].priv;
        hw_priv = &priv->dev_info;

        unregister_netdev( &NetDevice[ i ] );

        /* Not a pseudo device. */
        if ( hw_priv == priv->pDevInfo ) {
            if ( hw_priv->proc_main ) {
                int port = 2;

#ifdef DEF_KS8842
                int port_count = 2;

#else
                int port_count = 1;
#endif

                for ( j = 0; j < PROC_LAST; j++ ) {
                    if ( ProcPortInfo[ port ][ j ].info ) {
                        remove_proc_entry( ProcInfo[ j ].proc_name,
                            hw_priv->proc_main );
                    }
                }
                for ( port = 0; port < port_count; port++ ) {
                    if ( hw_priv->proc_port[ port ] ) {
                        for ( j = 0; j < PROC_LAST; j++ ) {
                            if ( ProcPortInfo[ port ][ j ].info ) {
                                remove_proc_entry( ProcInfo[ j ].proc_name,
                                    hw_priv->proc_port[ port ]);
                            }
                        }
                    }
                    sprintf( proc_name, "%d", port );
                    remove_proc_entry( proc_name, hw_priv->proc_main );
                }
            }
            sprintf( proc_name, "%s-%d", proc_dir_name, hw_priv->id );
            remove_proc_entry( proc_name, NULL );

            kfree( hw_priv->hw.m_hwfn );

#ifdef IOREMAP_RANGE
            if ( ( ULONG ) hw_priv->hw.m_pVirtualMemory !=
                    hw_priv->hw.m_ulVIoAddr )
                iounmap(( void* ) hw_priv->hw.m_pVirtualMemory );
#endif

            release_mem_region( NetDevice[ i ].mem_start, BASE_IO_RANGE );
        }
        kfree( NetDevice[ i ].priv );
    }
}  /* cleanup_module */


#ifndef MODULE
module_init(netdev_init_module);
module_exit(netdev_cleanup_module);
#endif
